@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: color.c,v 1.114.2.1 2015/04/16 05:11:21 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - color.c */
 
 /*[
@@ -36,6 +40,7 @@ t_sm_palette sm_palette;  /* initialized in plot.c on program entry */
 
 /* Copy of palette previously in use.
  * Exported so that change_term() can invalidate contents
+ * FIXME: better naming 
  */
 static t_sm_palette prev_palette = {
 	-1, -1, -1, -1, -1, -1, -1, -1,
@@ -44,10 +49,11 @@ static t_sm_palette prev_palette = {
 
 /* Internal prototype declarations: */
 
-static void draw_inside_color_smooth_box_postscript(void);
-static void draw_inside_color_smooth_box_bitmap(void);
-static void cbtick_callback(struct axis *, double place, char *text, int ticlevel,
-			struct lp_style_type grid, struct ticmark *userlabels);
+static void draw_inside_color_smooth_box_postscript __PROTO((void));
+static void draw_inside_color_smooth_box_bitmap __PROTO((void));
+void cbtick_callback __PROTO((AXIS_INDEX axis, double place, char *text, int ticlevel,
+			struct lp_style_type grid, struct ticmark *userlabels));
+
 
 
 /* *******************************************************************
@@ -185,7 +191,7 @@ set_rgbcolor_var(unsigned int rgbvalue)
     color.type = TC_RGB;
     *(unsigned int *)(&color.lt) = rgbvalue;
     color.value = -1;	/* -1 flags that this came from "rgb variable" */
-    apply_pm3dcolor(&color);
+    apply_pm3dcolor(&color, term);
 }
 
 void
@@ -195,9 +201,110 @@ set_rgbcolor_const(unsigned int rgbvalue)
     color.type = TC_RGB;
     *(unsigned int *)(&color.lt) = rgbvalue;
     color.value = 0;	/* 0 flags that this is a constant color */
-    apply_pm3dcolor(&color);
+    apply_pm3dcolor(&color, term);
 }
 
+void
+ifilled_quadrangle(gpiPoint* icorners)
+{
+    if (default_fillstyle.fillstyle == FS_EMPTY)
+	icorners->style = FS_OPAQUE;
+    else
+	icorners->style = style_from_fill(&default_fillstyle);
+    term->filled_polygon(4, icorners);
+
+    if (pm3d.border.l_type != LT_NODRAW) {
+	int i;
+
+	/* It should be sufficient to set only the color, but for some */
+	/* reason this causes the svg terminal to lose the fill type.  */
+	term_apply_lp_properties(&pm3d_border_lp);
+
+	term->move(icorners[0].x, icorners[0].y);
+	for (i = 3; i >= 0; i--) {
+	    term->vector(icorners[i].x, icorners[i].y);
+	}
+    }
+}
+
+
+/* The routine above for 4 points explicitly.
+ * This is the only routine which supportes extended
+ * color specs currently.
+ */
+#ifdef EXTENDED_COLOR_SPECS
+void
+filled_quadrangle(gpdPoint * corners, gpiPoint * icorners)
+#else
+void
+filled_quadrangle(gpdPoint * corners)
+#endif
+{
+    int i;
+    double x, y;
+#ifndef EXTENDED_COLOR_SPECS
+    gpiPoint icorners[4];
+#endif
+    for (i = 0; i < 4; i++) {
+	map3d_xy_double(corners[i].x, corners[i].y, corners[i].z, &x, &y);
+	icorners[i].x = x;
+	icorners[i].y = y;
+    }
+
+    ifilled_quadrangle(icorners);
+}
+
+#ifdef PM3D_CONTOURS
+/*
+   Makes mapping from real 3D coordinates, passed as coords array,
+   to 2D terminal coordinates, then draws filled polygon
+ */
+void
+filled_polygon_common(int points, struct coordinate GPHUGE * coords, TBOOLEAN fixed, double z)
+{
+    int i;
+    double x, y;
+    gpiPoint *icorners;
+    icorners = gp_alloc(points * sizeof(gpiPoint), "filled_polygon3d corners");
+    for (i = 0; i < points; i++) {
+	if (fixed)
+	    z = coords[i].z;
+	map3d_xy_double(coords[i].x, coords[i].y, z, &x, &y);
+	icorners[i].x = x;
+	icorners[i].y = y;
+    }
+#ifdef EXTENDED_COLOR_SPECS
+    if ((term->flags & TERM_EXTENDED_COLOR)) {
+	icorners[0].spec.gray = -1;	/* force solid color */
+    }
+#endif
+    if (default_fillstyle.fillstyle == FS_EMPTY)
+	icorners->style = FS_OPAQUE;
+    else
+	icorners->style = style_from_fill(&default_fillstyle);
+    term->filled_polygon(points, icorners);
+    free(icorners);
+}
+
+void
+filled_polygon_3dcoords(int points, struct coordinate GPHUGE * coords)
+{
+    filled_polygon_common(points, coords, FALSE, 0.0);
+}
+
+/*
+   Makes mapping from real 3D coordinates, passed as coords array, but at z coordinate
+   fixed (base_z, for instance) to 2D terminal coordinates, then draws filled polygon
+ */
+void
+filled_polygon_3dcoords_zfixed(int points, struct coordinate GPHUGE * coords, double z)
+{
+    filled_polygon_common(points, coords, TRUE, z);
+}
+
+#endif /* PM3D_CONTOURS */
+
+
 /*
    Draw colour smooth box
 
@@ -307,6 +414,10 @@ draw_inside_color_smooth_box_bitmap()
 	    corners[0].x = corners[3].x = xy;
 	    corners[1].x = corners[2].x = GPMIN(xy_to,xy2+1);
 	}
+#ifdef EXTENDED_COLOR_SPECS
+	if ((term->flags & TERM_EXTENDED_COLOR))
+	    corners[0].spec.gray = -1;	/* force solid color */
+#endif
 	/* print the rectangle with the given colour */
 	if (default_fillstyle.fillstyle == FS_EMPTY)
 	    corners->style = FS_OPAQUE;
@@ -316,27 +427,22 @@ draw_inside_color_smooth_box_bitmap()
     }
 }
 
-static void
+/* Notice HBB 20010720: would be static, but HP-UX gcc bug forbids
+ * this, for now */
+void
 cbtick_callback(
-    struct axis *this_axis,
+    AXIS_INDEX axis,
     double place,
     char *text,
     int ticlevel,
     struct lp_style_type grid, /* linetype or -2 for no grid */
     struct ticmark *userlabels)
 {
-    int len = tic_scale(ticlevel, this_axis)
-	* (this_axis->tic_in ? -1 : 1) * (term->h_tic);
+    int len = TIC_SCALE(ticlevel, COLOR_AXIS)
+	* (CB_AXIS.tic_in ? -1 : 1) * (term->h_tic);
+    double cb_place = (place - CB_AXIS.min) / (CB_AXIS.max - CB_AXIS.min);
+	/* relative z position along the colorbox axis */
     unsigned int x1, y1, x2, y2;
-    double cb_place;
-
-    /* position of tic as a fraction of the full palette range */
-    if (this_axis->linked_to_primary) {
-	AXIS * primary = this_axis->linked_to_primary;
-	place = eval_link_function(primary, place);
-	cb_place = (place - primary->min) / (primary->max - primary->min);
-    } else 
-	cb_place = (place - this_axis->min) / (this_axis->max - this_axis->min);
 
     /* calculate tic position */
     if (color_box.rotation == 'h') {
@@ -384,43 +490,43 @@ cbtick_callback(
 #	undef MINIMUM_SEPARATION
 
 	/* get offset */
-	map3d_position_r(&(this_axis->ticdef.offset),
+	map3d_position_r(&(axis_array[axis].ticdef.offset),
 			 &offsetx, &offsety, "cbtics");
 	/* User-specified different color for the tics text */
-	if (this_axis->ticdef.textcolor.type != TC_DEFAULT)
-	    apply_pm3dcolor(&(this_axis->ticdef.textcolor));
+	if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT)
+	    apply_pm3dcolor(&(axis_array[axis].ticdef.textcolor), term);
 	if (color_box.rotation == 'h') {
 	    int y3 = color_box.bounds.ybot - (term->v_char);
 	    int hrotate = 0;
 
-	    if (this_axis->tic_rotate
-		&& (*term->text_angle)(this_axis->tic_rotate))
-		    hrotate = this_axis->tic_rotate;
+	    if (axis_array[axis].tic_rotate
+		&& (*term->text_angle)(axis_array[axis].tic_rotate))
+		    hrotate = axis_array[axis].tic_rotate;
 	    if (len > 0) y3 -= len; /* add outer tics len */
 	    if (y3<0) y3 = 0;
 	    just = hrotate ? LEFT : CENTRE;
-	    if (this_axis->manual_justify)
-		just = this_axis->tic_pos;
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;
 	    write_multiline(x2+offsetx, y3+offsety, text,
 			    just, JUST_CENTRE, hrotate,
-			    this_axis->ticdef.font);
+			    axis_array[axis].ticdef.font);
 	    if (hrotate)
 		(*term->text_angle)(0);
 	} else {
 	    unsigned int x3 = color_box.bounds.xright + (term->h_char);
 	    if (len > 0) x3 += len; /* add outer tics len */
 	    just = LEFT;
-	    if (this_axis->manual_justify)
-		just = this_axis->tic_pos;	    
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;	    
 	    write_multiline(x3+offsetx, y2+offsety, text,
 			    just, JUST_CENTRE, 0.0,
-			    this_axis->ticdef.font);
+			    axis_array[axis].ticdef.font);
 	}
 	term_apply_lp_properties(&border_lp);	/* border linetype */
     }
 
     /* draw tic on the mirror side */
-    if (this_axis->ticmode & TICS_MIRROR) {
+    if (CB_AXIS.ticmode & TICS_MIRROR) {
 	if (color_box.rotation == 'h') {
 	    y1 = color_box.bounds.ytop;
 	    y2 = color_box.bounds.ytop + len;
@@ -439,6 +545,8 @@ cbtick_callback(
 void
 draw_color_smooth_box(int plot_mode)
 {
+    double tmp;
+
     if (color_box.where == SMCOLOR_BOX_NO)
 	return;
     if (!term->filled_polygon)
@@ -479,13 +587,20 @@ draw_color_smooth_box(int plot_mode)
 
     } else { /* color_box.where == SMCOLOR_BOX_DEFAULT */
 	if (plot_mode == MODE_SPLOT && !splot_map) {
-	    /* general 3D plot */
+	    /* HBB 20031215: new code.  Constants fixed to what the result
+	     * of the old code in default view (set view 60,30,1,1)
+	     * happened to be. Somebody fix them if they're not right! */
 	    color_box.bounds.xleft = xmiddle + 0.709 * xscaler;
 	    color_box.bounds.xright   = xmiddle + 0.778 * xscaler;
 	    color_box.bounds.ybot = ymiddle - 0.147 * yscaler;
 	    color_box.bounds.ytop   = ymiddle + 0.497 * yscaler;
-	} else {
-	    /* 2D plot (including splot map) */
+
+	} else if (is_3d_plot) {
+	    /* MWS 09-Dec-05, make color box full size for splot maps. */
+	    double dx = (X_AXIS.max - X_AXIS.min);
+	    map3d_xy(X_AXIS.max + dx * 0.025, Y_AXIS.min, base_z, &color_box.bounds.xleft, &color_box.bounds.ybot);
+	    map3d_xy(X_AXIS.max + dx * 0.075, Y_AXIS.max, ceiling_z, &color_box.bounds.xright, &color_box.bounds.ytop);
+	} else { /* 2D plot */
 	    struct position default_origin = {graph,graph,graph, 1.025, 0, 0};
 	    struct position default_size = {graph,graph,graph, 0.05, 1.0, 0};
 	    double xtemp, ytemp;
@@ -514,18 +629,11 @@ draw_color_smooth_box(int plot_mode)
 	}
     }
 
-    if (color_box.bounds.ybot > color_box.bounds.ytop) {
-	double tmp = color_box.bounds.ytop;
+    if (color_box.bounds.ybot > color_box.bounds.ytop) { /* switch them */
+	tmp = color_box.bounds.ytop;
 	color_box.bounds.ytop = color_box.bounds.ybot;
 	color_box.bounds.ybot = tmp;
     }
-    if (color_box.invert && color_box.rotation == 'v') {
-	double tmp = color_box.bounds.ytop;
-	color_box.bounds.ytop = color_box.bounds.ybot;
-	color_box.bounds.ybot = tmp;
-    }
-
-    term->layer(TERM_LAYER_BEGIN_COLORBOX);
 
     /* The PostScript terminal has an Optimized version */
     if ((term->flags & TERM_IS_POSTSCRIPT) != 0)
@@ -533,8 +641,6 @@ draw_color_smooth_box(int plot_mode)
     else
 	draw_inside_color_smooth_box_bitmap();
 
-    term->layer(TERM_LAYER_END_COLORBOX);
-
     if (color_box.border) {
 	/* now make boundary around the colour box */
 	if (color_box.border_lt_tag >= 0) {
@@ -561,39 +667,54 @@ draw_color_smooth_box(int plot_mode)
     /* draw tics */
     if (axis_array[COLOR_AXIS].ticmode) {
 	term_apply_lp_properties(&border_lp); /* border linetype */
-	gen_tics(&axis_array[COLOR_AXIS], cbtick_callback );
+	gen_tics(COLOR_AXIS, cbtick_callback );
     }
 
     /* write the colour box label */
     if (CB_AXIS.label.text) {
 	int x, y;
-	int len;
-	int save_rotation = CB_AXIS.label.rotate;
-	apply_pm3dcolor(&(CB_AXIS.label.textcolor));
+	apply_pm3dcolor(&(CB_AXIS.label.textcolor),term);
 	if (color_box.rotation == 'h') {
-	    len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? 1 : -1) * (term->v_tic);
+	    int len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? 1 : -1) * 
+		(term->v_tic);
 
-	    x = (color_box.bounds.xleft + color_box.bounds.xright) / 2;
-	    y = color_box.bounds.ybot - 2.7 * term->v_char;
+	    map3d_position_r(&(CB_AXIS.label.offset), &x, &y, "smooth_box");
+	    x += (color_box.bounds.xleft + color_box.bounds.xright) / 2;
 
+#define DEFAULT_Y_DISTANCE 1.0
+	    y += color_box.bounds.ybot + (- DEFAULT_Y_DISTANCE - 1.7) * term->v_char;
+#undef DEFAULT_Y_DISTANCE
 	    if (len < 0) y += len;
-	    if (CB_AXIS.label.rotate == TEXT_VERTICAL)
-		CB_AXIS.label.rotate = 0;
+	    if (x<0) x = 0;
+	    if (y<0) y = 0;
+	    write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_CENTRE, 0,
+			    CB_AXIS.label.font);
 	} else {
-	    len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? -1 : 1) * (term->h_tic);
+	    int len = CB_AXIS.ticscale * (CB_AXIS.tic_in ? -1 : 1) *
+		(term->h_tic);
 	    /* calculate max length of cb-tics labels */
 	    widest_tic_strlen = 0;
-	    if (CB_AXIS.ticmode & TICS_ON_BORDER) /* Recalculate widest_tic_strlen */
-		gen_tics(&axis_array[COLOR_AXIS], widest_tic_callback);
-	    x = color_box.bounds.xright + (widest_tic_strlen + 1.5) * term->h_char;
+	    if (CB_AXIS.ticmode & TICS_ON_BORDER) {
+	      	widest_tic_strlen = 0; /* reset the global variable */
+		gen_tics(COLOR_AXIS, /* 0, */ widest_tic_callback);
+	    }
+	    map3d_position_r(&(CB_AXIS.label.offset), &x, &y, "smooth_box");
+#define DEFAULT_X_DISTANCE 0.0
+	    x += color_box.bounds.xright + (widest_tic_strlen + DEFAULT_X_DISTANCE + 1.5) * term->h_char;
+#undef DEFAULT_X_DISTANCE
 	    if (len > 0) x += len;
-	    y = (color_box.bounds.ybot + color_box.bounds.ytop) / 2;
+	    y += (color_box.bounds.ybot + color_box.bounds.ytop) / 2;
+	    if (x<0) x = 0;
+	    if (y<0) y = 0;
+	    if ((*term->text_angle)(CB_AXIS.label.rotate)) {
+		write_multiline(x, y, CB_AXIS.label.text, CENTRE, JUST_TOP,
+				CB_AXIS.label.rotate, CB_AXIS.label.font);
+		(*term->text_angle)(0);
+	    } else {
+		write_multiline(x, y, CB_AXIS.label.text, LEFT, JUST_TOP, 0, CB_AXIS.label.font);
+	    }
 	}
-	if (x<0) x = 0;
-	if (y<0) y = 0;
-	write_label(x, y, &(CB_AXIS.label));
-	reset_textcolor(&(CB_AXIS.label.textcolor));
-	CB_AXIS.label.rotate = save_rotation;
+	reset_textcolor(&(CB_AXIS.label.textcolor),term);
     }
 
 }
@@ -641,26 +762,3 @@ f_hsv2rgb(union argument *arg)
     (void) Ginteger(&result, hsv2rgb(&color));
     push(&result);
 }
-
-/*
- * user-callable lookup of palette color for specific z-value
- */
-void
-f_palette(union argument *arg)
-{
-    struct value result;
-    double z;
-    rgb255_color color;
-    unsigned int rgb;
-
-    pop(&result);
-    z = real(&result);
-    if (((CB_AXIS.set_autoscale & AUTOSCALE_BOTH) != 0)
-    && (fabs(CB_AXIS.min) >= VERYLARGE || fabs(CB_AXIS.max) >= VERYLARGE))
-	int_error(NO_CARET, "palette(z) requires known cbrange");
-    rgb255maxcolors_from_gray(cb2gray(z), &color);
-    rgb = (unsigned int)color.r << 16 | (unsigned int)color.g << 8 | (unsigned int)color.b;
-
-    push(Ginteger(&result, rgb));
-}
-
