@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: command.c,v 1.292.2.7 2015/05/22 23:34:06 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: command.c,v 1.292.2.10 2015/08/01 05:19:16 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - command.c */
@@ -724,78 +724,56 @@ lower_command(void)
 
 #ifdef USE_MOUSE
 
-#define WHITE_AFTER_TOKEN(x) \
-(' ' == gp_input_line[token[x].start_index + token[x].length] \
-|| '\t' == gp_input_line[token[x].start_index + token[x].length] \
-|| '\0' == gp_input_line[token[x].start_index + token[x].length])
-
 /* process the 'bind' command */
+/* EAM - rewritten 2015 */
 void
 bind_command()
 {
-    char* lhs = (char*) 0;
-    char* rhs = (char*) 0;
+    char* lhs = NULL;
+    char* rhs = NULL;
     TBOOLEAN allwindows = FALSE;
     ++c_token;
 
-    if (!END_OF_COMMAND && equals(c_token,"!")) {
-	bind_remove_all();
-	++c_token;
-	return;
-    }
-
-    if (!END_OF_COMMAND && almost_equals(c_token,"all$windows")) {
+    if (almost_equals(c_token,"all$windows")) {
 	allwindows = TRUE;
 	c_token++;
     }
 
-    /* get left hand side: the key or key sequence */
-    if (!END_OF_COMMAND) {
+    /* get left hand side: the key or key sequence
+     * either (1) entire sequence is in quotes 
+     * or (2) sequence goes until the first whitespace
+     */
+    if (END_OF_COMMAND) {
+	; /* Fall through */
+    } else if (isstringvalue(c_token) && (lhs = try_to_get_string())) {
+	FPRINTF((stderr,"Got bind quoted lhs = \"%s\"\n",lhs));
+    } else {
 	char *first = gp_input_line + token[c_token].start_index;
-	int size = (int) (strchr(first, ' ') - first);
-	if (size < 0) {
-	    size = (int) (strchr(first, '\0') - first);
-	}
-	if (size < 0) {
-	    fprintf(stderr, "(bind_command) %s:%d\n", __FILE__, __LINE__);
-	    return;
-	}
-	if (!(isstring(c_token)) || !((lhs = try_to_get_string()))) {
-	    char *ptr = gp_alloc(size + 1, "bind_command->lhs");
-	    lhs = ptr;
-	    while (!END_OF_COMMAND) {
-		copy_str(ptr, c_token, token_len(c_token) + 1);
-		ptr += token_len(c_token);
-		if (WHITE_AFTER_TOKEN(c_token)) {
-		    break;
-		}
-		++c_token;
-	    }
-	    ++c_token;
-	}
+	int size = strcspn(first, " \";");
+	lhs = gp_alloc(size + 1, "bind_command->lhs");
+	strncpy(lhs, first, size);
+	lhs[size] = '\0';
+	FPRINTF((stderr,"Got bind unquoted lhs = \"%s\"\n",lhs));
+	while (gp_input_line + token[c_token].start_index < first+size)
+	    c_token++;
     }
 
-    /* get right hand side: the command. allocating the size
-     * of gp_input_line is too big, but shouldn't hurt too much. */
-    if (!END_OF_COMMAND) {
-	if (!(isstringvalue(c_token)) || !((rhs = try_to_get_string()))) {
-	    char *ptr = gp_alloc(strlen(gp_input_line) + 1, "bind_command->rhs");
-	    rhs = ptr;
-	    while (!END_OF_COMMAND) {
-		/* bind <lhs> ... ... ... */
-		copy_str(ptr, c_token, token_len(c_token) + 1);
-		ptr += token_len(c_token);
-		if (WHITE_AFTER_TOKEN(c_token)) {
-		    *ptr++ = ' ';
-		    *ptr = '\0';
-		}
-		c_token++;
-	    }
-	}
+    /* get right hand side: the command to bind
+     * either (1) quoted command
+     * or (2) the rest of the line
+     */
+    if (END_OF_COMMAND) {
+	; /* Fall through */
+    } else if (isstringvalue(c_token) && (rhs = try_to_get_string())) {
+        FPRINTF((stderr,"Got bind quoted rhs = \"%s\"\n",rhs));
+    } else {
+	int save_token = c_token;
+	while (!END_OF_COMMAND)
+	    c_token++;
+	m_capture( &rhs, save_token, c_token-1 );
+        FPRINTF((stderr,"Got bind unquoted rhs = \"%s\"\n",rhs));
     }
 
-    FPRINTF((stderr, "(bind_command) |%s| |%s|\n", lhs, rhs));
-
     /* bind_process() will eventually free lhs / rhs ! */
     bind_process(lhs, rhs, allwindows);
 
@@ -1592,6 +1570,7 @@ plot_command()
 {
     plot_token = c_token++;
     plotted_data_from_stdin = FALSE;
+    refresh_nplots = 0;
     SET_CURSOR_WAIT;
 #ifdef USE_MOUSE
     plot_mode(MODE_PLOT);
@@ -1806,6 +1785,7 @@ refresh_request()
 
     if (   ((first_plot == NULL) && (refresh_ok == E_REFRESH_OK_2D))
         || ((first_3dplot == NULL) && (refresh_ok == E_REFRESH_OK_3D))
+	|| (!*replot_line && (refresh_ok == E_REFRESH_NOT_OK))
        )
 	int_error(NO_CARET, "no active plot; cannot refresh");
 
@@ -1983,6 +1963,7 @@ splot_command()
 {
     plot_token = c_token++;
     plotted_data_from_stdin = FALSE;
+    refresh_nplots = 0;
     SET_CURSOR_WAIT;
 #ifdef USE_MOUSE
     plot_mode(MODE_SPLOT);
@@ -2332,12 +2313,13 @@ replotrequest()
 
     screen_ok = FALSE;
     num_tokens = scanner(&gp_input_line, &gp_input_line_len);
-    c_token = 1;		/* skip the 'plot' part */
+    c_token = 1;	/* Skip the "plot" token */
 
-    if (almost_equals(0,"s$plot"))
-	plot3drequest();
-    else if (almost_equals(0,"test"))
+    if (almost_equals(0,"test")) {
+    	c_token = 0;
 	test_command();
+    } else if (almost_equals(0,"s$plot"))
+	plot3drequest();
     else
 	plotrequest();
 }
