@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: dynarray.c,v 1.12 2015/04/23 20:10:02 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: dynarray.c,v 1.11.10.1 2015/04/23 20:10:30 sfeam Exp $"); }
 #endif
 
 /*[
@@ -43,10 +43,7 @@ static char *RCSid() { return RCSid("$Id: dynarray.c,v 1.12 2015/04/23 20:10:02
 #include "dynarray.h"
 
 #include "alloc.h"
-#include "util.h"	/* for int_error() */
-
-static char *init_failure_msg = "dynarray wasn't initialized";
-
+#include "util.h"		/* for graph_error() */
 
 /* The 'constructor' of a dynarray object: initializes all the
  * variables to well-defined startup values */
@@ -77,7 +74,7 @@ void
 resize_dynarray(dynarray *this, long newsize)
 {
     if (!this->v)
-	int_error(NO_CARET, init_failure_msg);
+	graph_error("resize_dynarray: dynarray wasn't initialized!");
 
     if (newsize == 0)
 	free_dynarray(this);
@@ -101,7 +98,7 @@ void GPHUGE *
 nextfrom_dynarray(dynarray *this)
 {
     if (!this->v)
-	int_error(NO_CARET, init_failure_msg);
+	graph_error("nextfrom_dynarray: dynarray wasn't initialized!");
 
     if (this->end >= this->size)
 	extend_dynarray(this, this->increment);
@@ -114,7 +111,7 @@ void
 droplast_dynarray(dynarray *this)
 {
     if (!this->v)
-	int_error(NO_CARET, init_failure_msg);
+	graph_error("droplast_dynarray: dynarray wasn't initialized!");
 
     if (this->end)
 	this->end--;
