@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: eval.c,v 1.119.2.2 2015/08/01 05:07:23 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - eval.c */
 
 /*[
@@ -30,6 +34,10 @@
  * to the extent permitted by applicable law.
 ]*/
 
+/* HBB 20010724: I moved several variables and functions from parse.c
+ * to here, because they're involved with *evaluating* functions, not
+ * with parsing them: evaluate_at(), fpe(), and fpe_env */
+
 #include "eval.h"
 
 #include "syscfg.h"
@@ -44,17 +52,15 @@
 #include "util.h"
 #include "version.h"
 #include "term_api.h"
-#include "voxelgrid.h"
 
 #include <signal.h>
 #include <setjmp.h>
 
 /* Internal prototypes */
-static RETSIGTYPE fpe(int an_int);
+static RETSIGTYPE fpe __PROTO((int an_int));
 
 /* Global variables exported by this module */
-struct udvt_entry udv_pi = { NULL, "pi", {INTGR, {0} } };
-struct udvt_entry *udv_I;
+struct udvt_entry udv_pi = { NULL, "pi", FALSE, {INTGR, {0} } };
 struct udvt_entry *udv_NaN;
 /* first in linked list */
 struct udvt_entry *first_udv = &udv_pi;
@@ -62,16 +68,8 @@ struct udft_entry *first_udf = NULL;
 /* pointer to first udv users can delete */
 struct udvt_entry **udv_user_head;
 
-/* Various abnormal conditions during evaluation of an action table
- * (the stored form of an expression) are signalled by setting
- * undefined = TRUE.
- * NB:  A test for  "if (undefined)"  is only valid immediately
- * following a call to evaluate_at() or eval_link_function().
- */
 TBOOLEAN undefined;
 
-enum int64_overflow overflow_handling = INT64_OVERFLOW_TO_FLOAT;
-
 /* The stack this operates on */
 static struct value stack[STACK_DEPTH];
 static int s_p = -1;		/* stack pointer */
@@ -81,7 +79,7 @@ static int jump_offset;		/* to be modified by 'jump' operators */
 
 /* The table of built-in functions */
 /* These must strictly parallel enum operators in eval.h */
-const struct ft_entry ft[] =
+const struct ft_entry GPFAR ft[] =
 {
     /* internal functions: */
     {"push",  f_push},
@@ -96,7 +94,6 @@ const struct ft_entry ft[] =
     {"lnot",  f_lnot},
     {"bnot",  f_bnot},
     {"uminus",  f_uminus},
-    {"nop",  f_nop},
     {"lor",  f_lor},
     {"land",  f_land},
     {"bor",  f_bor},
@@ -123,8 +120,6 @@ const struct ft_entry ft[] =
     {"eqs",  f_eqs},			/* for string variables only */
     {"nes",  f_nes},			/* for string variables only */
     {"[]",  f_range},			/* for string variables only */
-    {"[]",  f_index},			/* for array variables only */
-    {"||",  f_cardinality},		/* for array variables only */
     {"assign", f_assign},		/* assignment operator '=' */
     {"jump",  f_jump},
     {"jumpz",  f_jumpz},
@@ -143,7 +138,6 @@ const struct ft_entry ft[] =
     {"stringcolumn",  f_stringcolumn},	/* for using specs */
     {"strcol",  f_stringcolumn},	/* shorthand form */
     {"columnhead",  f_columnhead},
-    {"columnheader",  f_columnhead},
     {"valid",  f_valid},
     {"timecolumn",  f_timecolumn},
 
@@ -172,14 +166,10 @@ const struct ft_entry ft[] =
     {"exp",  f_exp},
     {"log10",  f_log10},
     {"log",  f_log},
-    {"besi0",  f_besi0},
-    {"besi1",  f_besi1},
     {"besj0",  f_besj0},
     {"besj1",  f_besj1},
-    {"besjn",  f_besjn},
     {"besy0",  f_besy0},
     {"besy1",  f_besy1},
-    {"besyn",  f_besyn},
     {"erf",  f_erf},
     {"erfc",  f_erfc},
     {"gamma",  f_gamma},
@@ -190,6 +180,7 @@ const struct ft_entry ft[] =
     {"rand",  f_rand},
     {"floor",  f_floor},
     {"ceil",  f_ceil},
+    {"defined", f_exists},	/* DEPRECATED syntax defined(foo) */
 
     {"norm",  f_normal},	/* XXX-JG */
     {"inverf",  f_inverse_erf},	/* XXX-JG */
@@ -200,7 +191,6 @@ const struct ft_entry ft[] =
     {"lambertw",  f_lambertw}, /* HBB, from G.Kuhnle 20001107 */
     {"airy",  f_airy},         /* janert, 20090905 */
     {"expint",  f_expint},     /* Jim Van Zandt, 20101010 */
-    {"besin",  f_besin},
 
 #ifdef HAVE_LIBCERF
     {"cerf", f_cerf},		/* complex error function */
@@ -210,24 +200,20 @@ const struct ft_entry ft[] =
     {"faddeeva", f_faddeeva},	/* Faddeeva rescaled complex error function "w_of_z" */
 #endif
 
-    {"tm_sec",  f_tmsec},	/* time function */
-    {"tm_min",  f_tmmin},	/* time function */
-    {"tm_hour", f_tmhour},	/* time function */
-    {"tm_mday", f_tmmday},	/* time function */
-    {"tm_mon",  f_tmmon},	/* time function */
-    {"tm_year", f_tmyear},	/* time function */
-    {"tm_wday", f_tmwday},	/* time function */
-    {"tm_yday", f_tmyday},	/* time function */
-    {"tm_week", f_tmweek},	/* time function */
-    {"weekdate_iso", f_weekdate_iso},
-    {"weekdate_cdc", f_weekdate_cdc},
+    {"tm_sec",  f_tmsec},	/* for timeseries */
+    {"tm_min",  f_tmmin},	/* for timeseries */
+    {"tm_hour",  f_tmhour},	/* for timeseries */
+    {"tm_mday",  f_tmmday},	/* for timeseries */
+    {"tm_mon",  f_tmmon},	/* for timeseries */
+    {"tm_year",  f_tmyear},	/* for timeseries */
+    {"tm_wday",  f_tmwday},	/* for timeseries */
+    {"tm_yday",  f_tmyday},	/* for timeseries */
 
     {"sprintf",  f_sprintf},	/* for string variables only */
     {"gprintf",  f_gprintf},	/* for string variables only */
     {"strlen",  f_strlen},	/* for string variables only */
     {"strstrt",  f_strstrt},	/* for string variables only */
     {"substr",  f_range},	/* for string variables only */
-    {"trim",  f_trim},		/* for string variables only */
     {"word",  f_word},		/* for string variables only */
     {"words", f_words},		/* implemented as word(s,-1) */
     {"strftime",  f_strftime},  /* time to string */
@@ -239,11 +225,6 @@ const struct ft_entry ft[] =
     {"value", f_value},		/* retrieve value of variable known by name */
 
     {"hsv2rgb", f_hsv2rgb},	/* color conversion */
-    {"palette", f_palette},	/* palette color lookup */
-
-#ifdef VOXEL_GRID_SUPPORT
-    {"voxel", f_voxel},		/* extract value of single voxel */
-#endif
 
     {NULL, NULL}
 };
@@ -257,7 +238,7 @@ static JMP_BUF fpe_env;
 static RETSIGTYPE
 fpe(int an_int)
 {
-#if defined(MSDOS) && !defined(__EMX__) && !defined(DJGPP)
+#if defined(MSDOS) && !defined(__EMX__) && !defined(DJGPP) && !defined(_Windows)
     /* thanks to lotto@wjh12.UUCP for telling us about this  */
     _fpreset();
 #endif
@@ -283,10 +264,8 @@ real(struct value *val)
 	return ((double) val->v.int_val);
     case CMPLX:
 	return (val->v.cmplx_val.real);
-    case STRING:    /* FIXME is this ever used? */
+    case STRING:              /* is this ever used? */
 	return (atof(val->v.string_val));
-    case NOTDEFINED:
-	return not_a_number();
     default:
 	int_error(NO_CARET, "unknown type in real()");
     }
@@ -295,6 +274,25 @@ real(struct value *val)
 }
 
 
+/* returns the real part of val, converted to int if necessary */
+int
+real_int(struct value *val)
+{
+    switch (val->type) {
+    case INTGR:
+	return val->v.int_val;
+    case CMPLX:
+	return (int) val->v.cmplx_val.real;
+    case STRING:
+	return atoi(val->v.string_val);
+    default:
+	int_error(NO_CARET, "unknown type in real_int()");
+    }
+    /* NOTREACHED */
+    return 0;
+}
+
+
 /* returns the imag part of val */
 double
 imag(struct value *val)
@@ -309,8 +307,6 @@ imag(struct value *val)
 	/*     x = 2;  plot sprintf(format,x)         */
 	int_warn(NO_CARET, "encountered a string when expecting a number");
 	int_error(NO_CARET, "Did you try to generate a file name using dummy variable x or y?");
-    case NOTDEFINED:
-	return not_a_number();
     default:
 	int_error(NO_CARET, "unknown type in imag()");
     }
@@ -326,7 +322,7 @@ magnitude(struct value *val)
 {
     switch (val->type) {
     case INTGR:
-	return (fabs((double)val->v.int_val));
+	return ((double) abs(val->v.int_val));
     case CMPLX:
 	{
 	    /* The straightforward implementation sqrt(r*r+i*i)
@@ -394,7 +390,7 @@ Gcomplex(struct value *a, double realpart, double imagpart)
 
 
 struct value *
-Ginteger(struct value *a, intgr_t i)
+Ginteger(struct value *a, int i)
 {
     a->type = INTGR;
     a->v.int_val = i;
@@ -405,48 +401,24 @@ struct value *
 Gstring(struct value *a, char *s)
 {
     a->type = STRING;
-    a->v.string_val = s ? s : strdup("");
+    a->v.string_val = s;
     return (a);
 }
 
-/* Common interface for freeing data structures attached to a struct value.
- * Each of the type-specific routines will ignore values of other types.
- */
-void
-free_value(struct value *a)
-{
-    gpfree_string(a);
-    gpfree_datablock(a);
-    gpfree_array(a);
-}
-
 /* It is always safe to call gpfree_string with a->type is INTGR or CMPLX.
  * However it would be fatal to call it with a->type = STRING if a->string_val
  * was not obtained by a previous call to gp_alloc(), or has already been freed.
- * Thus 'a->type' is set to NOTDEFINED afterwards to make subsequent calls safe.
+ * Thus 'a->type' is set to INTGR afterwards to make subsequent calls safe.
  */
-void
+struct value *
 gpfree_string(struct value *a)
 {
     if (a->type == STRING) {
 	free(a->v.string_val);
-	a->type = NOTDEFINED;
-    }
-}
-
-void
-gpfree_array(struct value *a)
-{
-    int i;
-    int size;
-
-    if (a->type == ARRAY) {
-	size = a->v.value_array[0].v.int_val;
-	for (i=1; i<=size; i++)
-	    gpfree_string(&(a->v.value_array[i]));
-	free(a->v.value_array);
-	a->type = NOTDEFINED;
+	/* I would have set it to INVALID if such a type existed */
+	a->type = INTGR;
     }
+    return a;
 }
 
 /* some machines have trouble with exp(-x) for large x
@@ -505,44 +477,34 @@ pop(struct value *x)
 /*
  * Allow autoconversion of string variables to floats if they
  * are dereferenced in a numeric context.
- * Jun 2022: Stricter error checking for non-numeric string.
  */
 struct value *
 pop_or_convert_from_string(struct value *v)
 {
-    pop(v);
+    (void) pop(v);
 
+    /* DEBUG Dec 2014 - Consolidate sanity check for variable type */
     /* FIXME: Test for INVALID_VALUE? Other corner cases? */
     if (v->type == INVALID_NAME)
 	int_error(NO_CARET, "invalid dummy variable name");
 
     if (v->type == STRING) {
-	char *string = v->v.string_val;
-	char *eov = string;
-	char trailing = *eov;
+	char *eov;
 
-	/* If the string contains no decimal point, try to interpret it as an integer.
-	 * strtoll handles decimal, octal, and hexadecimal.
-	 */
-	if (strcspn(string, ".") == strlen(string)) {
-	    long long li = strtoll( string, &eov, 0 );
-	    trailing = *eov;
-	    Ginteger(v, li);
-	}
-	/* Successful interpretation as an integer leaves (eov != string).
-	 * Otherwise try again as a floating point, including oddball cases like
-	 * "NaN" or "-Inf" that contain no decimal point.
-	 */
-	if (eov == string) {
-	    double d = strtod(string, &eov);
-	    trailing = *eov;
+	if (strspn(v->v.string_val,"0123456789 ") == strlen(v->v.string_val)) {
+	    int i = atoi(v->v.string_val);
+	    gpfree_string(v);
+	    Ginteger(v, i);
+	} else {
+	    double d = strtod(v->v.string_val,&eov);
+	    if (v->v.string_val == eov) {
+		gpfree_string(v);
+		int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
+	    }
+	    gpfree_string(v);
 	    Gcomplex(v, d, 0.);
+	    FPRINTF((stderr,"converted string to CMPLX value %g\n",real(v)));
 	}
-	free(string);	/* NB: invalidates dereference of eov */
-	if (eov == string)
-	    int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
-	if (trailing && !isspace(trailing))
-	    int_error(NO_CARET,"Trailing characters after numeric expression");
     }
     return(v);
 }
@@ -553,7 +515,6 @@ push(struct value *x)
     if (s_p == STACK_DEPTH - 1)
 	int_error(NO_CARET, "stack overflow");
     stack[++s_p] = *x;
-
     /* WARNING - This is a memory leak if the string is not later freed */
     if (x->type == STRING && x->v.string_val)
 	stack[s_p].v.string_val = gp_strdup(x->v.string_val);
@@ -666,18 +627,13 @@ execute_at(struct at_type *at_ptr)
     jump_offset = saved_jump_offset;
 }
 
-/* As of May 2013 input of Inf/NaN values through evaluation is treated */
-/* equivalently to direct input of a formatted value.  See imageNaN.dem. */
+/* May 2013: Old hackery #ifdef'ed out so that input of Inf/NaN */
+/* values through evaluation is treated equivalently to direct  */
+/* input of a formated value.  See revised imageNaN demo.       */
 void
 evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 {
-    /* A test for if (undefined) is allowed only immediately following
-     * evalute_at() or eval_link_function().  Both must clear it on entry
-     * so that the value on return reflects what really happened.
-     */
     undefined = FALSE;
-    val_ptr->type = NOTDEFINED;
-
     errno = 0;
     reset_stack();
 
@@ -694,38 +650,54 @@ evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 
     if (errno == EDOM || errno == ERANGE)
 	undefined = TRUE;
+#if (1)	/* New code */
     else if (!undefined) {
 	(void) pop(val_ptr);
 	check_stack();
     }
 
-    if (!undefined && val_ptr->type == ARRAY) {
-	/* Aug 2016: error rather than warning because too many places
-	 * cannot deal with UNDEFINED or NaN where they were expecting a number
-	 * E.g. load_one_range()
-	 */
-	val_ptr->type = NOTDEFINED;
-	if (!string_result_only)
-	    int_error(NO_CARET, "evaluate_at: unsupported array operation");
+#else /* Old hackery */
+    else if (!undefined) { /* undefined (but not errno) may have been set by matherr */
+	(void) pop(val_ptr);
+	check_stack();
+	/* At least one machine (ATT 3b1) computes Inf without a SIGFPE */
+	if (val_ptr->type != STRING) {
+	    double temp = real(val_ptr);
+	    if (temp > VERYLARGE || temp < -VERYLARGE)
+		undefined = TRUE;
+	}
     }
+#if defined(NeXT) || defined(ultrix)
+    /*
+     * linux was able to fit curves which NeXT gave up on -- traced it to
+     * silently returning NaN for the undefined cases and plowing ahead
+     * I can force that behavior this way.  (0.0/0.0 generates NaN)
+     */
+    if (undefined && (errno == EDOM || errno == ERANGE)) {	/* corey@cac */
+	undefined = FALSE;
+	errno = 0;
+	Gcomplex(val_ptr, 0.0 / 0.0, 0.0 / 0.0);
+    }
+#endif /* NeXT || ultrix */
+#endif /* old hackery */
 }
 
 void
-real_free_at(struct at_type *at_ptr)
+free_at(struct at_type *at_ptr)
 {
     int i;
     /* All string constants belonging to this action table have to be
      * freed before destruction. */
     if (!at_ptr)
         return;
-    for (i=0; i<at_ptr->a_count; i++) {
+    for(i=0; i<at_ptr->a_count; i++) {
 	struct at_entry *a = &(at_ptr->actions[i]);
 	/* if union a->arg is used as a->arg.v_arg free potential string */
 	if ( a->index == PUSHC || a->index == DOLLARS )
 	    gpfree_string(&(a->arg.v_arg));
 	/* a summation contains its own action table wrapped in a private udf */
 	if (a->index == SUM) {
-	    real_free_at(a->arg.udf_arg->at);
+	    free_at(a->arg.udf_arg->at);
 	    free(a->arg.udf_arg);
 	}
 #ifdef HAVE_EXTERNAL_FUNCTIONS
@@ -757,7 +729,8 @@ add_udv_by_name(char *key)
 	gp_alloc(sizeof(struct udvt_entry), "value");
     (*udv_ptr)->next_udv = NULL;
     (*udv_ptr)->udv_name = gp_strdup(key);
-    (*udv_ptr)->udv_value.type = NOTDEFINED;
+    (*udv_ptr)->udv_undef = TRUE;
+    (*udv_ptr)->udv_value.type = 0;
     return (*udv_ptr);
 }
 
@@ -791,17 +764,17 @@ del_udv_by_name(char *key, TBOOLEAN wildcard)
 
  	/* exact match */
 	else if (!wildcard && !strcmp(key, udv_ptr->udv_name)) {
-	    gpfree_vgrid(udv_ptr);
-	    free_value(&(udv_ptr->udv_value));
-	    udv_ptr->udv_value.type = NOTDEFINED;
+	    udv_ptr->udv_undef = TRUE;
+	    gpfree_string(&(udv_ptr->udv_value));
+	    gpfree_datablock(&(udv_ptr->udv_value));
 	    break;
 	}
 
 	/* wildcard match: prefix matches */
 	else if ( wildcard && !strncmp(key, udv_ptr->udv_name, strlen(key)) ) {
-	    gpfree_vgrid(udv_ptr);
-	    free_value(&(udv_ptr->udv_value));
-	    udv_ptr->udv_value.type = NOTDEFINED;
+	    udv_ptr->udv_undef = TRUE;
+	    gpfree_string(&(udv_ptr->udv_value));
+	    gpfree_datablock(&(udv_ptr->udv_value));
 	    /* no break - keep looking! */
 	}
 
@@ -827,13 +800,12 @@ clear_udf_list()
     first_udf = NULL;
 }
 
-static void update_plot_bounds(void);
-static void fill_gpval_axis(AXIS_INDEX axis);
-static void fill_gpval_sysinfo(void);
-static void set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value);
+static void update_plot_bounds __PROTO((void));
+static void fill_gpval_axis __PROTO((AXIS_INDEX axis));
+static void set_gpval_axis_sth_double __PROTO((const char *prefix, AXIS_INDEX axis, const char *suffix, double value, int is_int));
 
 static void
-set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value)
+set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value, int is_int)
 {
     struct udvt_entry *v;
     char *cc, s[24];
@@ -841,9 +813,13 @@ set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffi
     for (cc=s; *cc; cc++)
 	*cc = toupper((unsigned char)*cc); /* make the name uppercase */
     v = add_udv_by_name(s);
-    if (!v)
+    if (!v) 
 	return; /* should not happen */
-    Gcomplex(&v->udv_value, value, 0);
+    v->udv_undef = FALSE;
+    if (is_int)
+	Ginteger(&v->udv_value, (int)(value+0.5));
+    else
+	Gcomplex(&v->udv_value, value, 0);
 }
 
 static void
@@ -851,13 +827,15 @@ fill_gpval_axis(AXIS_INDEX axis)
 {
     const char *prefix = "GPVAL";
     AXIS *ap = &axis_array[axis];
-    set_gpval_axis_sth_double(prefix, axis, "MIN", ap->min);
-    set_gpval_axis_sth_double(prefix, axis, "MAX", ap->max);
-    set_gpval_axis_sth_double(prefix, axis, "LOG", ap->base);
+    double a = AXIS_DE_LOG_VALUE(axis, ap->min);
+    double b = AXIS_DE_LOG_VALUE(axis, ap->max);
+    set_gpval_axis_sth_double(prefix, axis, "MIN", a, 0);
+    set_gpval_axis_sth_double(prefix, axis, "MAX", b, 0);
+    set_gpval_axis_sth_double(prefix, axis, "LOG", ap->base, 0);
 
     if (axis < POLAR_AXIS) {
-	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MIN", ap->data_min);
-	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MAX", ap->data_max);
+	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MIN", AXIS_DE_LOG_VALUE(axis, ap->data_min), 0);
+	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MAX", AXIS_DE_LOG_VALUE(axis, ap->data_max), 0);
     }
 }
 
@@ -870,19 +848,22 @@ fill_gpval_string(char *var, const char *stringvalue)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
-    if (v->udv_value.type == STRING && !strcmp(v->udv_value.v.string_val, stringvalue))
+    if (v->udv_undef == FALSE && !strcmp(v->udv_value.v.string_val, stringvalue))
 	return;
+    if (v->udv_undef)
+	v->udv_undef = FALSE;
     else
 	gpfree_string(&v->udv_value);
     Gstring(&v->udv_value, gp_strdup(stringvalue));
 }
 
 void
-fill_gpval_integer(char *var, intgr_t value)
+fill_gpval_integer(char *var, int value)
 {
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
+    v->udv_undef = FALSE;
     Ginteger(&v->udv_value, value);
 }
 
@@ -892,6 +873,7 @@ fill_gpval_float(char *var, double value)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
+    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, value, 0);
 }
 
@@ -901,6 +883,7 @@ fill_gpval_complex(char *var, double areal, double aimag)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
+    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, areal, aimag);
 }
 
@@ -911,16 +894,13 @@ fill_gpval_complex(char *var, double areal, double aimag)
 static void
 update_plot_bounds(void)
 {
-    fill_gpval_float("GPVAL_TERM_XMIN", (double)axis_array[FIRST_X_AXIS].term_lower / term->tscale);
-    fill_gpval_float("GPVAL_TERM_XMAX", (double)axis_array[FIRST_X_AXIS].term_upper / term->tscale);
-    fill_gpval_float("GPVAL_TERM_YMIN", (double)axis_array[FIRST_Y_AXIS].term_lower / term->tscale);
-    fill_gpval_float("GPVAL_TERM_YMAX", (double)axis_array[FIRST_Y_AXIS].term_upper / term->tscale);
+    fill_gpval_integer("GPVAL_TERM_XMIN", axis_array[FIRST_X_AXIS].term_lower / term->tscale);
+    fill_gpval_integer("GPVAL_TERM_XMAX", axis_array[FIRST_X_AXIS].term_upper / term->tscale);
+    fill_gpval_integer("GPVAL_TERM_YMIN", axis_array[FIRST_Y_AXIS].term_lower / term->tscale);
+    fill_gpval_integer("GPVAL_TERM_YMAX", axis_array[FIRST_Y_AXIS].term_upper / term->tscale);
     fill_gpval_integer("GPVAL_TERM_XSIZE", canvas.xright+1);
     fill_gpval_integer("GPVAL_TERM_YSIZE", canvas.ytop+1);
     fill_gpval_integer("GPVAL_TERM_SCALE", term->tscale);
-    /* May be useful for debugging font problems */
-    fill_gpval_integer("GPVAL_TERM_HCHAR", term->h_char);
-    fill_gpval_integer("GPVAL_TERM_VCHAR", term->v_char);
 }
 
 /*
@@ -959,17 +939,6 @@ update_gpval_variables(int context)
 	fill_gpval_float("GPVAL_VIEW_ROT_Z", surface_rot_z);
 	fill_gpval_float("GPVAL_VIEW_SCALE", surface_scale);
 	fill_gpval_float("GPVAL_VIEW_ZSCALE", surface_zscale);
-	fill_gpval_float("GPVAL_VIEW_AZIMUTH", azimuth);
-
-	/* Screen coordinates of 3D rotational center and radius of the sphere */
-	/* in which x/y axes are drawn after 'set view equal xy[z]' */
-	fill_gpval_float("GPVAL_VIEW_XCENT",
-		(double)(canvas.xright+1 - xmiddle)/(double)(canvas.xright+1));
-	fill_gpval_float("GPVAL_VIEW_YCENT",
-		1.0 - (double)(canvas.ytop+1 - ymiddle)/(double)(canvas.ytop+1));
-	fill_gpval_float("GPVAL_VIEW_RADIUS",
-		0.5 * surface_scale * xscaler/(double)(canvas.xright+1));
-
 	return;
     }
 
@@ -986,9 +955,6 @@ update_gpval_variables(int context)
 	fill_gpval_string("GPVAL_TERMOPTIONS", term_options);
 	fill_gpval_string("GPVAL_OUTPUT", (outstr) ? outstr : "");
 	fill_gpval_string("GPVAL_ENCODING", encoding_names[encoding]);
-	fill_gpval_string("GPVAL_MINUS_SIGN", minus_sign ? minus_sign : "-");
-	fill_gpval_string("GPVAL_MICRO", micro ? micro : "u");
-	fill_gpval_string("GPVAL_DEGREE_SIGN", degree_sign);
     }
 
     /* If we are called from int_error() then set the error state */
@@ -999,13 +965,15 @@ update_gpval_variables(int context)
     if (context == 3) {
 	struct udvt_entry *v = add_udv_by_name("GPVAL_VERSION");
 	char *tmp;
-	if (v && v->udv_value.type == NOTDEFINED)
+	if (v && v->udv_undef == TRUE) {
+	    v->udv_undef = FALSE;
 	    Gcomplex(&v->udv_value, atof(gnuplot_version), 0);
+	}
 	v = add_udv_by_name("GPVAL_PATCHLEVEL");
-	if (v && v->udv_value.type == NOTDEFINED)
+	if (v && v->udv_undef == TRUE)
 	    fill_gpval_string("GPVAL_PATCHLEVEL", gnuplot_patchlevel);
 	v = add_udv_by_name("GPVAL_COMPILE_OPTIONS");
-	if (v && v->udv_value.type == NOTDEFINED)
+	if (v && v->udv_undef == TRUE)
 	    fill_gpval_string("GPVAL_COMPILE_OPTIONS", compile_options);
 
 	/* Start-up values */
@@ -1022,27 +990,21 @@ update_gpval_variables(int context)
 	/* Permanent copy of user-clobberable variables pi and NaN */
 	fill_gpval_float("GPVAL_pi", M_PI);
 	fill_gpval_float("GPVAL_NaN", not_a_number());
-
-	/* System information */
-	fill_gpval_sysinfo();
     }
 
     if (context == 3 || context == 4) {
 	fill_gpval_integer("GPVAL_ERRNO", 0);
 	fill_gpval_string("GPVAL_ERRMSG","");
-	fill_gpval_integer("GPVAL_SYSTEM_ERRNO", 0);
-	fill_gpval_string("GPVAL_SYSTEM_ERRMSG","");
     }
 
-    /* GPVAL_PWD is unreliable.  If the current directory becomes invalid,
-     * GPVAL_PWD does not reflect this.  If this matters, the user can
-     * instead do something like    MY_PWD = "`pwd`"
-     */
     if (context == 3 || context == 5) {
-	char *save_file = gp_alloc(PATH_MAX, "GPVAL_PWD");
-	int ierror = (GP_GETCWD(save_file, PATH_MAX) == NULL);
-	fill_gpval_string("GPVAL_PWD", ierror ? "" : save_file);
-	free(save_file);
+	char *save_file = NULL;
+	save_file = (char *) gp_alloc(PATH_MAX, "filling GPVAL_PWD");
+	if (save_file) {
+	    GP_GETCWD(save_file, PATH_MAX);
+	    fill_gpval_string("GPVAL_PWD", save_file);
+	    free(save_file);
+	}
     }
 
     if (context == 6) {
@@ -1050,58 +1012,6 @@ update_gpval_variables(int context)
     }
 }
 
-/* System information is stored in GPVAL_BITS GPVAL_MACHINE GPVAL_SYSNAME */
-#ifdef HAVE_UNAME
-# include <sys/utsname.h>
-#elif defined(_WIN32)
-# include <windows.h>
-#endif
-
-void
-fill_gpval_sysinfo()
-{
-/* For linux/posix systems with uname */
-#ifdef HAVE_UNAME
-    struct utsname uts;
-
-    if (uname(&uts) < 0)
-	return;
-    fill_gpval_string("GPVAL_SYSNAME", uts.sysname);
-    fill_gpval_string("GPVAL_MACHINE", uts.machine);
-
-/* For Windows systems */
-#elif defined(_WIN32)
-    SYSTEM_INFO stInfo;
-    OSVERSIONINFO osvi;
-    char s[30];
-
-    ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
-    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&osvi);
-    snprintf(s, 30, "Windows_NT-%ld.%ld", osvi.dwMajorVersion, osvi.dwMinorVersion);
-    fill_gpval_string("GPVAL_SYSNAME", s);
-
-    GetSystemInfo(&stInfo);
-    switch (stInfo.wProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_INTEL:
-        fill_gpval_string("GPVAL_MACHINE", "x86");
-        break;
-    case PROCESSOR_ARCHITECTURE_IA64:
-        fill_gpval_string("GPVAL_MACHINE", "ia64");
-       break;
-    case PROCESSOR_ARCHITECTURE_AMD64:
-        fill_gpval_string("GPVAL_MACHINE", "x86_64");
-        break;
-    default:
-        fill_gpval_string("GPVAL_MACHINE", "unknown");
-    }
-#endif
-
-/* For all systems */
-    fill_gpval_integer("GPVAL_BITS", 8 * sizeof(void *));
-}
-
 /* Callable wrapper for the words() internal function */
 int
 gp_words(char *string)
@@ -1122,10 +1032,33 @@ gp_word(char *string, int i)
     struct value a;
 
     push(Gstring(&a, string));
-    push(Ginteger(&a, (intgr_t)i));
+    push(Ginteger(&a, i));
     f_word((union argument *)NULL);
     pop(&a);
 
     return a.v.string_val;
 }
 
+
+/* Evaluate the function linking secondary axis to primary axis */
+double
+eval_link_function(int axis, double raw_coord)
+{
+    udft_entry *link_udf = axis_array[axis].link_udf;
+    int dummy_var;
+    struct value a;
+
+    if (axis == FIRST_Y_AXIS || axis == SECOND_Y_AXIS)
+	dummy_var = 1;
+    else
+	dummy_var = 0;
+    link_udf->dummy_values[1-dummy_var].type = INVALID_NAME;
+
+    Gcomplex(&link_udf->dummy_values[dummy_var], raw_coord, 0.0);
+    evaluate_at(link_udf->at, &a);
+
+    if (a.type != CMPLX)
+	a = udv_NaN->udv_value;
+
+    return a.v.cmplx_val.real;
+}
