@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.11 2015/06/07 15:19:59 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.13 2016/01/06 05:19:56 sfeam Exp $"); }
 #endif
 
 /*  NOTICE: Change of Copyright Status
@@ -2205,6 +2205,8 @@ fit_command()
 	    continue;
 	case DF_COLUMN_HEADERS:
 	    continue;
+	case DF_FOUND_KEY_TITLE:
+	    continue;
 	case 0:
 	    Eex2("bad data on line %d of datafile", df_line_number);
 	    break;
@@ -2363,11 +2365,11 @@ fit_command()
 
 	static char *viafile = NULL;
 	free(viafile);			/* Free previous name, if any */
-	viafile = try_to_get_string();	/* Cannot fail since isstringvalue succeeded */
+	viafile = try_to_get_string();
+	if (!viafile || !(f = loadpath_fopen(viafile, "r")))
+	    Eex2("could not read parameter-file \"%s\"", viafile);
 	if (!fit_suppress_log)
 	    fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", viafile);
-	if (!(f = loadpath_fopen(viafile, "r")))
-	    Eex2("could not read parameter-file \"%s\"", viafile);
 
 	/* get parameters and values out of file and ignore fixed ones */
 
