@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: interpol.c,v 1.46.2.3 2015/02/09 19:48:27 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: interpol.c,v 1.46.2.5 2015/10/28 20:18:31 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - interpol.c */
@@ -1266,7 +1266,7 @@ cp_implode(struct curve_points *cp)
 	    cp->points[j].x = x;
 	    if ( cp->plot_smooth == SMOOTH_FREQUENCY ||
 		 cp->plot_smooth == SMOOTH_CUMULATIVE ||
-		 cp->plot_smooth == SMOOTH_CUMULATIVE)
+		 cp->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED)
 		k = 1;
 	    cp->points[j].y = y /= (double) k;
 	    cp->points[j].xhigh = sux / (double) k;
@@ -1354,6 +1354,10 @@ mcs_interp(struct curve_points *plot)
 #define C2	yhigh
 #define C3	z
 
+    /* Work with the un-logged y values */
+    for (i = 0; i < N-1; i++)
+	p[i].y = AXIS_DE_LOG_VALUE(plot->y_axis, p[i].y);
+
     for (i = 0; i < N-1; i++) {
 	p[i].DX = p[i+1].x - p[i].x;
 	p[i].SLOPE = (p[i+1].y - p[i].y) / p[i].DX;
