@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: plot.c,v 1.164.2.1 2014/12/31 04:32:09 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - plot.c */
 
 /*[
@@ -45,8 +49,6 @@
 #include "util.h"
 #include "variable.h"
 #include "version.h"
-#include "voxelgrid.h"
-#include "encoding.h"
 
 #include <signal.h>
 #include <setjmp.h>
@@ -60,8 +62,13 @@
 # include <os2.h>
 #endif /* OS2 */
 
+#ifdef USE_MOUSE
+# include "mouse.h" /* for mouse_setting */
+# include "gpexecute.h"
+#endif
+
 /* on OS/2 this is needed even without USE_MOUSE */
-#if defined(OS2_IPC)
+#if defined(OS2_IPC) && !defined(USE_MOUSE)
 # include "gpexecute.h"
 #endif
 
@@ -81,12 +88,11 @@ extern int vms_ktid;
 extern smg$create_key_table();
 #endif /* VMS */
 
-#ifdef _WIN32
+#ifdef _Windows
 # include <windows.h>
 # include "win/winmain.h"
 # include "win/wcommon.h"
-# include <io.h>           // for isatty
-#endif /* _WIN32 */
+#endif /* _Windows */
 
 /* GNU readline
  * Only required by two files directly,
@@ -116,16 +122,15 @@ extern smg$create_key_table();
  */
 static char *expanded_history_filename;
 
-static void wrapper_for_write_history(void);
+static void wrapper_for_write_history __PROTO((void));
 
 #endif				/* GNUPLOT_HISTORY */
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
 TBOOLEAN noinputfiles = TRUE;	/* FALSE if there are script files */
-TBOOLEAN reading_from_dash=FALSE;	/* True if processing "-" as an input file */
-TBOOLEAN skip_gnuplotrc = FALSE;	/* skip system gnuplotrc and ~/.gnuplot */
-TBOOLEAN persist_cl = FALSE; 		/* --persist command line option */
-TBOOLEAN slow_font_startup = FALSE;	/* --slow command line option */
+TBOOLEAN reading_from_dash=FALSE; /* True if processing "-" as an input file */
+TBOOLEAN skip_gnuplotrc = FALSE;/* skip system gnuplotrc and ~/.gnuplot */
+TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
 
 /* user home directory */
 static const char *user_homedir = NULL;
@@ -133,32 +138,29 @@ static const char *user_homedir = NULL;
 /* user shell */
 const char *user_shell = NULL;
 
-/* not static because unset.c refers to it when in debugging mode */
-TBOOLEAN successful_initialization = FALSE;
+static TBOOLEAN successful_initialization = FALSE;
 
 #ifdef X11
-extern int X11_args(int, char **); /* FIXME: defined in term/x11.trm */
+extern int X11_args __PROTO((int, char **)); /* FIXME: defined in term/x11.trm */
 #endif
 
 /* patch to get home dir, see command.c */
-#if defined(MSDOS) || defined(OS2)
-char HelpFile[PATH_MAX];
-static char progpath[PATH_MAX] = "";
-#endif
+#ifdef DJGPP
+# include <dir.h>               /* MAXPATH */
+char HelpFile[MAXPATH];
+#endif /*   - DJL */
 
 /* a longjmp buffer to get back to the command line */
 static JMP_BUF command_line_env;
 
-static void load_rcfile(int where);
-static RETSIGTYPE inter(int anint);
-static void init_memory(void);
+static void load_rcfile __PROTO((int where));
+static RETSIGTYPE inter __PROTO((int anint));
+static void init_memory __PROTO((void));
 
 static int exit_status = EXIT_SUCCESS;
 
 /* Flag for asynchronous handling of Ctrl-C. Used by fit.c and Windows */
 TBOOLEAN ctrlc_flag = FALSE;
-/* Flag for (asynchronous) term signal on Windows. */
-TBOOLEAN terminate_flag = FALSE;
 
 #ifdef OS2
 # include <process.h>
@@ -169,7 +171,7 @@ TBOOLEAN CallFromRexx = FALSE;
 static RETSIGTYPE
 inter(int anint)
 {
-    (void) anint;		/* avoid -Wunused warning */
+    (void) anint;		/* aovid -Wunused warning */
     (void) signal(SIGINT, (sigfunc) inter);
     (void) signal(SIGFPE, SIG_DFL);	/* turn off FPE trapping */
 
@@ -253,32 +255,23 @@ take_privilege()
 void
 bail_to_command_line()
 {
-#ifdef _WIN32
+#ifdef _Windows
     kill_pending_Pause_dialog();
     ctrlc_flag = FALSE;
 #endif
     LONGJMP(command_line_env, TRUE);
 }
 
-#if defined(_WIN32)
+#if defined(_Windows)
 int
-gnu_main(int argc_orig, char **argv)
+gnu_main(int argc, char **argv)
 #else
 int
-main(int argc_orig, char **argv)
+main(int argc, char **argv)
 #endif
 {
     int i;
 
-    /* We want the current value of argc to persist across a LONGJMP from int_error().
-     * Without this the compiler may put it on the stack, which LONGJMP clobbers.
-     * Here we try make it a volatile variable that optimization will not affect.
-     * Why do we not have to do the same for argv?   I don't know.
-     * But the test cases that broke with generic argc seem fine with generic argv.
-     */
-    static volatile int argc;
-    argc = argc_orig;
-
 #ifdef LINUXVGA
     LINUX_setup();		/* setup VGA before dropping privilege DBT 4/5/99 */
     drop_privilege();
@@ -286,12 +279,13 @@ main(int argc_orig, char **argv)
 /* make sure that we really have revoked root access, this might happen if
    gnuplot is compiled without vga support but is installed suid by mistake */
 #ifdef __linux__
-    if (setuid(getuid()) != 0) {
-	fprintf(stderr,"gnuplot: refusing to run at elevated privilege\n");
-	exit(EXIT_FAILURE);
-    }
+    setuid(getuid());
 #endif
 
+#if defined(MSDOS) && !defined(_Windows) && !defined(__GNUC__)
+    PC_setup();
+#endif /* MSDOS !Windows */
+
 /* HBB: Seems this isn't needed any more for DJGPP V2? */
 /* HBB: disable all floating point exceptions, just keep running... */
 #if defined(DJGPP) && (DJGPP!=2)
@@ -299,18 +293,15 @@ main(int argc_orig, char **argv)
 #endif
 
 #if defined(OS2)
-    {
-	int rc;
+    int rc;
 #ifdef OS2_IPC
-	char semInputReadyName[40];
-
-	sprintf(semInputReadyName, "\\SEM32\\GP%i_Input_Ready", getpid());
-	rc = DosCreateEventSem(semInputReadyName, &semInputReady, 0, 0);
-	if (rc != 0)
-	    fputs("DosCreateEventSem error\n", stderr);
+    char semInputReadyName[40];
+    sprintf( semInputReadyName, "\\SEM32\\GP%i_Input_Ready", getpid() );
+    rc = DosCreateEventSem(semInputReadyName,&semInputReady,0,0);
+    if (rc != 0)
+      fputs("DosCreateEventSem error\n",stderr);
 #endif
-	rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
-    }
+    rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
 #endif
 
 /* malloc large blocks, otherwise problems with fragmented mem */
@@ -318,44 +309,23 @@ main(int argc_orig, char **argv)
     malloc_debug(7);
 #endif
 
-
-/* init progpath and get helpfile from executable directory */
-#if defined(MSDOS) || defined(OS2)
+/* get helpfile from home directory */
+#ifdef __DJGPP__
     {
 	char *s;
-
-#ifdef __EMX__
-	_execname(progpath, sizeof(progpath));
-#else
-	safe_strncpy(progpath, argv[0], sizeof(progpath));
-#endif
-	/* convert '/' to '\\' */
-	for (s = progpath; *s != NUL; s++)
-	    if (*s == DIRSEP2)
-		*s = DIRSEP1;
-	/* cut program name */
-	s = strrchr(progpath, DIRSEP1);
-	if (s != NULL)
-	    s++;
-	else
-	    s = progpath;
-	*s = NUL;
-	/* init HelpFile */
-	strcpy(HelpFile, progpath);
-	strcat(HelpFile, "gnuplot.gih");
-	/* remove trailing "bin/" from progpath */
-	if ((s != NULL) && (s - progpath >= 4)) {
-	    s -= 4;
-	    if (strncasecmp(s, "bin", 3) == 0)
-		*s = NUL;
-	}
-    }
+	strcpy(HelpFile, argv[0]);
+	for (s = HelpFile; *s; s++)
+	    if (*s == DIRSEP1)
+		*s = DIRSEP2;	/* '\\' to '/' */
+	strcpy(strrchr(HelpFile, DIRSEP2), "/gnuplot.gih");
+    }			/* Add also some "paranoid" tests for '\\':  AP */
 #endif /* DJGPP */
 
-#if (defined(PIPE_IPC) || defined(_WIN32)) && (defined(HAVE_LIBREADLINE) || (defined(HAVE_LIBEDITLINE) && defined(X11)))
-    /* Editline needs this to be set before the very first call to readline(). */
-    /* Support for rl_getc_function is broken for utf-8 in editline. Since it is only
-       really required for X11, disable this section when building without X11. */
+#ifdef VMS
+    unsigned int status[2] = { 1, 0 };
+#endif
+
+#if defined(HAVE_LIBEDITLINE)
     rl_getc_function = getc_wrapper;
 #endif
 
@@ -393,7 +363,6 @@ main(int argc_orig, char **argv)
 		    "  -V, --version\n"
 		    "  -h, --help\n"
 		    "  -p  --persist\n"
-		    "  -s  --slow\n"
 		    "  -d  --default-settings\n"
 		    "  -c  scriptfile ARG1 ARG2 ... \n"
 		    "  -e  \"command1; command2; ...\"\n"
@@ -412,13 +381,11 @@ main(int argc_orig, char **argv)
 	    return 0;
 
 	} else if (!strncmp(argv[i], "-persist", 2) || !strcmp(argv[i], "--persist")
-#ifdef _WIN32
+#ifdef _Windows
 		|| !stricmp(argv[i], "-noend") || !stricmp(argv[i], "/noend")
 #endif
 		) {
 	    persist_cl = TRUE;
-	} else if (!strncmp(argv[i], "-slow", 2) || !strcmp(argv[i], "--slow")) {
-	    slow_font_startup = TRUE;
 	} else if (!strncmp(argv[i], "-d", 2) || !strcmp(argv[i], "--default-settings")) {
 	    /* Skip local customization read from ~/.gnuplot */
 	    skip_gnuplotrc = TRUE;
@@ -451,6 +418,7 @@ main(int argc_orig, char **argv)
      * EAM - Jan 2013 YES.
      */
     setvbuf(stdin, (char *) NULL, _IONBF, 0);
+
 #endif
 
     gpoutfile = stdout;
@@ -465,11 +433,15 @@ main(int argc_orig, char **argv)
     init_memory();
 
     interactive = FALSE;
+    init_terminal();		/* can set term type if it likes */
+    push_terminal(0);		/* remember the default terminal */
 
-    /* April 2017:  We used to call init_terminal() here, but now   */
-    /* We defer initialization until error handling has been set up. */
+    /* reset the terminal when exiting */
+    /* this is done through gp_atexit so that other terminal functions
+     * can be registered to be executed before the terminal is reset. */
+    gp_atexit(term_reset);
 
-# if defined(_WIN32) && !defined(WGP_CONSOLE)
+# if defined(WIN32) && !defined(WGP_CONSOLE)
     interactive = TRUE;
 # else
     interactive = isatty(fileno(stdin));
@@ -482,7 +454,7 @@ main(int argc_orig, char **argv)
      * command line arguments are file names or an explicit in-line "-e command".
      */
     for (i = 1; i < argc; i++) {
-# ifdef _WIN32
+# ifdef _Windows
 	if (!stricmp(argv[i], "/noend"))
 	    continue;
 # endif
@@ -499,52 +471,52 @@ main(int argc_orig, char **argv)
     else
 	show_version(NULL); /* Only load GPVAL_COMPILE_OPTIONS */
 
+#ifdef WGP_CONSOLE
+#ifdef CONSOLE_SWITCH_CP
+    if (cp_changed && interactive) {
+	fprintf(stderr,
+	    "\ngnuplot changed the codepage of this console from %i to %i to\n" \
+	    "match the graph window. Some characters might only display correctly\n" \
+	    "if you change the font to a non-raster type.\n",
+	    cp_input, GetConsoleCP());
+    }
+#else
+    if ((GetConsoleCP() != GetACP()) && interactive) {
+	fprintf(stderr,
+	    "\nWarning: The codepage of the graph window (%i) and that of the\n" \
+	    "console (%i) differ. Use `set encoding` or `!chcp` if extended\n" \
+	    "characters don't display correctly.\n",
+	    GetACP(), GetConsoleCP());
+    }
+#endif
+#endif
+
     update_gpval_variables(3);  /* update GPVAL_ variables available to user */
 
 #ifdef VMS
     /* initialise screen management routines for command recall */
-    {
-    unsigned int ierror;
-    if (ierror = smg$create_virtual_keyboard(&vms_vkid) != SS$_NORMAL)
-	done(ierror);
-    if (ierror = smg$create_key_table(&vms_ktid) != SS$_NORMAL)
-	done(ierror);
-    }
+    if (status[1] = smg$create_virtual_keyboard(&vms_vkid) != SS$_NORMAL)
+	done(status[1]);
+    if (status[1] = smg$create_key_table(&vms_ktid) != SS$_NORMAL)
+	done(status[1]);
 #endif /* VMS */
 
     if (!SETJMP(command_line_env, 1)) {
 	/* first time */
 	interrupt_setup();
+	/* should move this stuff to another initialisation routine,
+	 * something like init_set() maybe */
 	get_user_env();
 	init_loadpath();
 	init_locale();
 
 	memset(&sm_palette, 0, sizeof(sm_palette));
 	init_fit();		/* Initialization of fitting module */
-#ifdef READLINE
-	/* When using the built-in readline, we set the initial
-	   encoding according to the locale as this is required
-	   to properly handle keyboard input. */
-	init_encoding();
-#endif
-	init_gadgets();
-
-	/* April 2017: Now that error handling is in place, it is safe parse
-	 * GNUTERM during terminal initialization.
-	 * atexit processing is done in reverse order. We want
-	 * the generic terminal shutdown in term_reset to be executed before
-	 * any terminal specific cleanup requested by individual terminals.
-	 */
-	init_terminal();
-	push_terminal(0);	/* remember the initial terminal */
-	gp_atexit(term_reset);
-
-	/* Execute commands in ~/.gnuplot */
 	init_session();
 
 	if (interactive && term != 0) {		/* not unknown */
 #ifdef GNUPLOT_HISTORY
-#if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && !defined(_WIN32)
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
 	    expanded_history_filename = tilde_expand(GNUPLOT_HISTORY_FILE);
 #else
 	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
@@ -555,15 +527,13 @@ main(int argc_orig, char **argv)
 	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
 	     * 'on_exit()'. In the worst case, there is no history of your
-	     * current session and you have to type all again in your next
+	     * currrent session and you have to type all again in your next
 	     * session.
 	     */
 	    gp_atexit(wrapper_for_write_history);
 #endif /* GNUPLOT_HISTORY */
 
-#if defined(READLINE) && defined(WGP_CONSOLE)
-	    fprintf(stderr, "Encoding set to '%s'.\n", encoding_names[encoding]);
-#endif
+	    fprintf(stderr, "\nTerminal type set to '%s'\n", term->name);
 	}			/* if (interactive && term != 0) */
     } else {
 	/* come back here from int_error() */
@@ -575,7 +545,8 @@ main(int argc_orig, char **argv)
 	if (interactive == FALSE)
 	    exit_status = EXIT_FAILURE;
 #ifdef HAVE_READLINE_RESET
-	else {
+	else
+	{
 	    /* reset properly readline after a SIGINT+longjmp */
 	    rl_reset_after_signal ();
 	}
@@ -606,6 +577,8 @@ main(int argc_orig, char **argv)
 	/* entering the loop again from the top finds them messed up.  */
 	/* If we reenter the loop via a goto then there is some hope   */
 	/* that code reordering does not hurt us.                      */
+	/* NB: the test for interactive means that execution from a    */
+	/* pipe will not continue after an error. Do we want this?     */
 	if (reading_from_dash && interactive)
 	    goto RECOVER_FROM_ERROR_IN_DASH;
 	reading_from_dash = FALSE;
@@ -621,13 +594,13 @@ main(int argc_orig, char **argv)
 	    ++argv;
 	    c_token = 0;
 	    if (!strncmp(*argv, "-persist", 2) || !strcmp(*argv, "--persist")
-#ifdef _WIN32
+#ifdef _Windows
 		|| !stricmp(*argv, "-noend") || !stricmp(*argv, "/noend")
 #endif
 	    ) {
 		FPRINTF((stderr,"'persist' command line option recognized\n"));
 	    } else if (strcmp(*argv, "-") == 0) {
-#if defined(_WIN32) && !defined(WGP_CONSOLE)
+#if defined(_Windows) && !defined(WGP_CONSOLE)
 		TextShow(&textwin);
 		interactive = TRUE;
 #else
@@ -639,10 +612,8 @@ RECOVER_FROM_ERROR_IN_DASH:
 		while (!com_line());
 		reading_from_dash = FALSE;
 		interactive = FALSE;
-		noinputfiles = FALSE;
 
 	    } else if (strcmp(*argv, "-e") == 0) {
-		int save_state = interactive;
 		--argc; ++argv;
 		if (argc <= 0) {
 		    fprintf(stderr, "syntax:  gnuplot -e \"commands\"\n");
@@ -651,10 +622,6 @@ RECOVER_FROM_ERROR_IN_DASH:
 		interactive = FALSE;
 		noinputfiles = FALSE;
 		do_string(*argv);
-		interactive = save_state;
-
-	    } else if (!strncmp(*argv, "-slow", 2) || !strcmp(*argv, "--slow")) {
-		slow_font_startup = TRUE;
 
 	    } else if (!strncmp(*argv, "-d", 2) || !strcmp(*argv, "--default-settings")) {
 		/* Ignore this; it already had its effect */
@@ -671,11 +638,10 @@ RECOVER_FROM_ERROR_IN_DASH:
 		    fprintf(stderr, "syntax:  gnuplot -c scriptname args\n");
 		    gp_exit(EXIT_FAILURE);
 		}
-		call_argc = GPMIN(9, argc - 1);
-		for (i=0; i<=call_argc; i++) {
+		for (i=0; i<argc; i++)
 		    /* Need to stash argv[i] somewhere visible to load_file() */
 		    call_args[i] = gp_strdup(argv[i+1]);
-		}
+		call_argc = argc - 1;
 
 		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 5);
 		gp_exit(EXIT_SUCCESS);
@@ -690,12 +656,11 @@ RECOVER_FROM_ERROR_IN_DASH:
     }
 
     /* take commands from stdin */
-    if (noinputfiles) {
+    if (noinputfiles)
 	while (!com_line())
 	    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
-    }
 
-#ifdef _WIN32
+#ifdef _Windows
     /* On Windows, handle 'persist' by keeping the main input loop running (windows/wxt), */
     /* but only if there are any windows open. Note that qt handles this properly. */
     if (persist_cl) {
@@ -730,7 +695,7 @@ RECOVER_FROM_ERROR_IN_DASH:
 
     /* HBB 20040223: Not all compilers like exit() to end main() */
     /* exit(exit_status); */
-#if ! defined(_WIN32)
+#if ! defined(_Windows)
     /* Windows does the cleanup later */
     gp_exit_cleanup();
 #endif
@@ -760,6 +725,7 @@ init_constants()
     (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
     udv_NaN = get_udv_by_name("NaN");
     (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
+    udv_NaN->udv_undef = FALSE;
 }
 
 /*
@@ -775,15 +741,9 @@ init_session()
 	/* Undefine any previously-used variables */
 	del_udv_by_name("",TRUE);
 
-	/* Restore default colors before loading local preferences */
+	/* Restore default colors before loadin local preferences */
 	set_colorsequence(1);
 
-	/* Reset program variables not handled by 'reset' */
-	overflow_handling = INT64_OVERFLOW_TO_FLOAT;
-
-	/* Reset voxel data structures if supported */
-	init_voxelsupport();
-
 	/* Make sure all variables start in the same state 'reset'
 	 * would set them to.
 	 */
@@ -814,13 +774,18 @@ load_rcfile(int where)
 
     if (where == 0) {
 #ifdef GNUPLOT_SHARE_DIR
-# if defined(_WIN32) || defined(MSDOS) || defined(OS2)
-	rcfile = RelativePathToGnuplot(GNUPLOT_SHARE_DIR "\\gnuplotrc");
+# if defined(_Windows)
+	/* retrieve path relative to gnuplot executable,
+	 * whose path is in szModuleName (winmain.c) */
+	rcfile = gp_alloc(strlen((char *)szPackageDir) + 1
+	       + strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
+	strcpy(rcfile, (char *)szPackageDir);
+	PATH_CONCAT(rcfile, GNUPLOT_SHARE_DIR);
 # else
 	rcfile = (char *) gp_alloc(strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
 	strcpy(rcfile, GNUPLOT_SHARE_DIR);
-	PATH_CONCAT(rcfile, "gnuplotrc");
 # endif
+	PATH_CONCAT(rcfile, "gnuplotrc");
 	plotrc = fopen(rcfile, "r");
 #endif
 
@@ -850,7 +815,6 @@ load_rcfile(int where)
     free(rcfile);
 }
 
-
 void
 get_user_env()
 {
@@ -858,21 +822,24 @@ get_user_env()
 	const char *env_home;
 
 	if ((env_home = getenv(HOME))
-#ifdef _WIN32
+#ifdef WIN32
 	    || (env_home = appdata_directory())
 	    || (env_home = getenv("USERPROFILE"))
 #endif
+#ifndef VMS
 	    || (env_home = getenv("HOME"))
+#endif
 	   )
 	    user_homedir = (const char *) gp_strdup(env_home);
 	else if (interactive)
 	    int_warn(NO_CARET, "no HOME found");
     }
+    /* Hhm ... what about VMS? */
     if (user_shell == NULL) {
 	const char *env_shell;
 
 	if ((env_shell = getenv("SHELL")) == NULL)
-#if defined(MSDOS) || defined(_WIN32) || defined(OS2)
+#if defined(MSDOS) || defined(_Windows) || defined(OS2)
 	    if ((env_shell = getenv("COMSPEC")) == NULL)
 #endif
 		env_shell = SHELL;
@@ -930,10 +897,33 @@ ExecuteMacro(char *argv, int namelength)
 
     if (namelength >= sizeof(pszName))
 	return 1;
-    safe_strncpy(pszName, argv, sizeof(pszName));
+    /* FIXME HBB 20010121: 3rd argument doesn't make sense. Either
+     * this should be sizeof(pszName), or it shouldn't use
+     * safe_strncpy(), here */
+    safe_strncpy(pszName, argv, namelength + 1);
     rxArgStr = &argv[namelength];
     RXSTRPTR(rxRc) = NULL;
 
+#if 0
+    /*
+       C-like calling of function: program name is first
+       parameter.
+       In REXX you would have to use
+       Parse Arg param0, param1
+       to get the program name in param0 and the arguments in param1.
+
+       Some versions before gnuplot 3.7pl1 used a similar approach but
+       passed program name and arguments in a single string:
+       (==> Parse Arg param0 param1)
+     */
+
+    MAKERXSTRING(rxArg[0], pszName, strlen(pszName));
+    rxArgCount++;
+    if (*rxArgStr) {
+	MAKERXSTRING(rxArg[1], rxArgStr, strlen(rxArgStr));
+	rxArgCount++;
+    }
+#else
     /*
        REXX standard calling (gnuplot 3.7pl1 and above):
        The program name is not supplied and so all actual arguments
@@ -946,6 +936,7 @@ ExecuteMacro(char *argv, int namelength)
 	MAKERXSTRING(rxArg[0], rxArgStr, strlen(rxArgStr));
 	rxArgCount++;
     }
+#endif
 
     CallFromRexx = TRUE;
     rc = RexxStart(
@@ -1001,7 +992,10 @@ RexxInterface(PRXSTRING rxCmd, PUSHORT pusErr, PRXSTRING rxRc)
 	/* Set variable gp_input_line.
 	   Watch out for line length of NOT_ZERO_TERMINATED strings ! */
 	cmdlen = rxCmd->strlength + 1;
-	safe_strncpy(gp_input_line, rxCmd->strptr, gp_input_line_len);
+	/* FIXME HBB 20010121: 3rd argument doesn't make sense. Either
+	 * this should be gp_input_line_len, or it shouldn't use
+	 * safe_strncpy(), here */
+	safe_strncpy(gp_input_line, rxCmd->strptr, cmdlen);
 	gp_input_line[cmdlen] = NUL;
 	rc = do_line();
 	*pusErr = RXSUBCOM_OK;
@@ -1065,7 +1059,7 @@ wrapper_for_write_history()
 	return;
     remove(expanded_history_filename);
     if (gnuplot_history_size < 0)
-	write_history(expanded_history_filename);
+    	write_history(expanded_history_filename);
     else
 	write_history_n(gnuplot_history_size, expanded_history_filename, "w");
 }
@@ -1079,23 +1073,3 @@ restrict_popen()
     if (!successful_initialization)
 	int_error(NO_CARET,"Pipes and shell commands not permitted during initialization");
 }
-
-
-#if defined(MSDOS) || defined(OS2)
-/* retrieve path relative to gnuplot executable */
-char *
-RelativePathToGnuplot(const char * path)
-{
-    char * rel_path, *s;
-
-    rel_path = (char * ) gp_alloc(strlen(progpath) + strlen(path) + 1, "RelativePathToGnuplot");
-    strcpy(rel_path, progpath);
-    /* progpath is guaranteed to have a trailing slash */
-    strcat(rel_path, path);
-    /* convert slashes to backslashes */
-    for (s = rel_path;*s != NUL; s++)
-	if (*s == DIRSEP2)
-	    *s = DIRSEP1;
-    return rel_path;
-}
-#endif
