@@ -1,5 +1,5 @@
 /*
- * $Id: setshow.h,v 1.56 2016/11/18 08:20:57 markisch Exp $
+ * $Id: setshow.h,v 1.50.2.2 2015/03/29 18:08:05 sfeam Exp $
  */
 
 /* GNUPLOT - setshow.h */
@@ -53,7 +53,6 @@ void show_command __PROTO((void));
 void show_version __PROTO((FILE *fp));
 void set_format __PROTO((void));
 void set_colorsequence __PROTO((int option));
-void init_special_chars(void);
 char *conv_text __PROTO((const char *s));
 void delete_linestyle __PROTO((struct linestyle_def **, struct linestyle_def *, struct linestyle_def *));
 void delete_dashtype __PROTO((struct custom_dashtype_def *, struct custom_dashtype_def *));
@@ -62,12 +61,11 @@ void reset_key __PROTO((void));
 void free_marklist __PROTO((struct ticmark * list));
 extern int enable_reset_palette;
 void reset_palette __PROTO((void));
-void reset_bars __PROTO((void));
 void rrange_to_xy __PROTO((void));
 void unset_monochrome __PROTO((void));
 
 /* Called from set_label(), plot2d.c and plot3d.c */
-extern void parse_label_options __PROTO((struct text_label *, int ndim));
+extern void parse_label_options __PROTO((struct text_label *this_label, TBOOLEAN));
 extern struct text_label * new_text_label __PROTO((int tag));
 extern void disp_value __PROTO((FILE *, struct value *, TBOOLEAN));
 extern struct ticmark * prune_dataticks __PROTO((struct ticmark *list));
