@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: specfun.c,v 1.53 2013/10/09 02:41:22 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - specfun.c */
 
 /*[
@@ -1186,13 +1190,10 @@ f_normal(union argument *arg)
     (void) arg;				/* avoid -Wunused warning */
     x = real(pop(&a));
 
-    /* using erfc instead of erf produces accurate values for -38 < arg < -8 */
-    if (x > -38) {
-	x = 0.5 * SQRT_TWO * x;
-	x = 0.5 * erfc(-x);
-    } else {
-	x = 0.0;
-    }
+    x = 0.5 * SQRT_TWO * x;
+    x = 0.5 * erfc(-x);		/* by using erfc instead of erf, we
+				   can get accurate values for -38 <
+				   arg < -8 */
     push(Gcomplex(&a, x, 0.0));
 }
 
@@ -3036,7 +3037,8 @@ double polevl(), p1evl(), sin(), cos();
 #endif
 
 int 
-airy( double x, double *ai, double *aip, double *bi, double *bip )
+airy( x, ai, aip, bi, bip )
+double x, *ai, *aip, *bi, *bip;
 {
     double z, zz, t, f, g, uf, ug, k, zeta, theta;
     int domflg;
