@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: stdfn.c,v 1.30 2014/03/20 00:58:35 markisch Exp $"); }
+#endif
+
 /* GNUPLOT - stdfn.c */
 
 /*[
@@ -37,23 +41,18 @@
  * - Lars Hecking
  */
 
-#if defined(_WIN32) && defined(__WATCOMC__)
-#  include <direct.h>
-#endif
-
 #include "stdfn.h"
 
-#ifdef _WIN32
+#ifdef WIN32
 /* the WIN32 API has a Sleep function that does not consume CPU cycles */
-# define WIN32_LEAN_AND_MEAN
-# include <windows.h>
-# include "term_api.h"
-# include "win/winmain.h"
-# include <io.h> /* _findfirst and _findnext set errno iff they return -1 */
+#include <windows.h>
+#ifndef HAVE_DIRENT_H
+#include <io.h> /* _findfirst and _findnext set errno iff they return -1 */
+#endif
 #endif
 #ifdef NEED_CEXP
-# include <math.h>
-# include <complex.h>
+#include <math.h>
+#include <complex.h>
 #endif
 
 /*
@@ -187,7 +186,7 @@ sleep(unsigned int delay)
     while (time(NULL) < time_is_up)
 	/* wait */ ;
 # endif
-#elif defined(_WIN32)
+#elif defined(WIN32)
     Sleep((DWORD) delay * 1000);
 #endif /* MSDOS ... */
 
@@ -269,8 +268,8 @@ gp_strnicmp(const char *s1, const char *s2, size_t n)
 #endif /* !HAVE_STRNICMP */
 
 
-#ifndef HAVE_STRNLEN
-size_t
+#ifndef HAVE_STRNLEN    
+size_t 
 strnlen(const char *str, size_t n)
 {
     const char * stop = (char *)memchr(str, '\0', n);
@@ -280,7 +279,7 @@ strnlen(const char *str, size_t n)
 
 
 #ifndef HAVE_STRNDUP
-char *
+char * 
 strndup(const char * str, size_t n)
 {
     char * ret = NULL;
@@ -295,6 +294,7 @@ strndup(const char * str, size_t n)
 
 /* Safe, '\0'-terminated version of strncpy()
  * safe_strncpy(dest, src, n), where n = sizeof(dest)
+ * This is basically the old fit.c(copy_max) function
  */
 char *
 safe_strncpy(char *d, const char *s, size_t n)
@@ -312,7 +312,7 @@ safe_strncpy(char *d, const char *s, size_t n)
 #ifndef HAVE_STRCSPN
 /*
  * our own substitute for strcspn()
- * return the length of the initial segment of str1
+ * return the length of the inital segment of str1
  * consisting of characters not in str2
  * returns strlen(str1) if none of the characters
  * from str2 are in str1
@@ -336,7 +336,7 @@ gp_strcspn(const char *str1, const char *str2)
 
 
 /* Standard compliant replacement functions for MSVC */
-#if defined(_MSC_VER) && (_MSC_VER < 1900)
+#if defined(_MSC_VER)
 int
 ms_vsnprintf(char *str, size_t size, const char *format, va_list ap)
 {
@@ -370,7 +370,7 @@ ms_snprintf(char *str, size_t size, const char * format, ...)
 double
 not_a_number(void)
 {
-#if defined (_MSC_VER) || defined (DJGPP) || defined(__DJGPP__) || defined(__MINGW32__)
+#if defined (__MSC__) || defined (DJGPP) || defined(__DJGPP__) || defined(__MINGW32__)
 	unsigned long lnan[2]={0xffffffff, 0x7fffffff};
     return *( double* )lnan;
 #else
@@ -455,7 +455,8 @@ void gp_exit_cleanup(void)
 static void debug_exit_handler(void)
 {
     if (exit_handlers) {
-        fprintf(stderr, "Gnuplot exiting abnormally. Trying to execute exit handlers anyway.\n");
+        fprintf(stderr, "Gnuplot not exited using gp_exit(). Exit handlers may"
+                " not work correctly!\n");
         gp_exit_cleanup();
     }
 }
@@ -485,22 +486,8 @@ void gp_exit(int status)
     exit(status);
 }
 
-#ifdef _WIN32
-char *
-gp_getcwd(char *path, size_t len)
-{
-    wchar_t wpath[MAX_PATH + 1];
-    if (_wgetcwd(wpath, MAX_PATH) != NULL) {
-	WideCharToMultiByte(WinGetCodepage(encoding), 0, wpath, -1, path, len, NULL, 0);
-	return path;
-    }
-    return NULL;
-}
-#endif
-
-#ifdef _WIN32
-/* Note: OpenWatcom has dirent functions in direct.h but we use our functions
-		 since they support encodings. */
+#if !defined(HAVE_DIRENT_H) && defined(WIN32)  && (!defined(__WATCOMC__))
+/* BM: OpenWatcom has dirent functions in direct.h!*/
 /*
 
     Implementation of POSIX directory browsing functions and types for Win32.
@@ -513,105 +500,88 @@ gp_getcwd(char *path, size_t len)
 
 struct DIR
 {
-    intptr_t            handle; /* -1 for failed rewind */
-    struct _wfinddata_t info;
-    struct gp_dirent    result; /* d_name null iff first time */
-    WCHAR               *name;  /* null-terminated string */
-    char                info_mbname[4*260];
+    long                handle; /* -1 for failed rewind */
+    struct _finddata_t  info;
+    struct dirent       result; /* d_name null iff first time */
+    char                *name;  /* null-terminated char string */
 };
 
-
-DIR *
-gp_opendir(const char *name)
+DIR *opendir(const char *name)
 {
     DIR *dir = 0;
-    char *mbname;
 
     if (name && name[0]) {
-	size_t base_length = strlen(name);
-	    /* search pattern must end with suitable wildcard */
-	const char *all = strchr("/\\", name[base_length - 1]) ? "*" : "/*";
+        size_t base_length = strlen(name);
+         /* search pattern must end with suitable wildcard */
+        const char *all = strchr("/\\", name[base_length - 1]) ? "*" : "/*";
 
-	if ((dir = (DIR *) malloc(sizeof *dir)) != NULL &&
-	    (mbname = (char *) malloc(base_length + strlen(all) + 1)) != NULL) {
-	    strcat(strcpy(mbname, name), all);
-	    dir->name = UnicodeText(mbname, encoding);
-	    free(mbname);
+        if ((dir = (DIR *) malloc(sizeof *dir)) != 0 &&
+           (dir->name = (char *) malloc(base_length + strlen(all) + 1)) != 0) {
+            strcat(strcpy(dir->name, name), all);
 
-	    if ((dir->name != NULL) &&
-		((dir->handle = (long) _wfindfirst(dir->name, &dir->info)) != -1)) {
-		dir->result.d_name = NULL;
-	    } else { /* rollback */
-		free(dir->name);
-		free(dir);
-		dir = NULL;
-	    }
-	} else { /* rollback */
-	    free(dir);
-	    dir   = NULL;
-	    errno = ENOMEM;
-	}
+            if ((dir->handle = (long) _findfirst(dir->name, &dir->info)) != -1) {
+                dir->result.d_name = 0;
+            } else { /* rollback */
+                free(dir->name);
+                free(dir);
+                dir = 0;
+            }
+        } else { /* rollback */
+            free(dir);
+            dir   = 0;
+            errno = ENOMEM;
+        }
     } else {
-	errno = EINVAL;
+        errno = EINVAL;
     }
 
     return dir;
 }
 
-
-int
-gp_closedir(DIR *dir)
+int closedir(DIR *dir)
 {
     int result = -1;
 
     if (dir) {
-	if (dir->handle != -1) {
-	    result = _findclose(dir->handle);
-	}
-	free(dir->name);
-	free(dir);
+        if(dir->handle != -1) {
+            result = _findclose(dir->handle);
+        }
+        free(dir->name);
+        free(dir);
     }
 
     if (result == -1) { /* map all errors to EBADF */
-	errno = EBADF;
+        errno = EBADF;
     }
 
     return result;
 }
 
-
-struct gp_dirent *
-gp_readdir(DIR *dir)
+struct dirent *readdir(DIR *dir)
 {
-    struct gp_dirent *result = 0;
+    struct dirent *result = 0;
 
     if (dir && dir->handle != -1) {
-	if (!dir->result.d_name || _wfindnext(dir->handle, &dir->info) != -1) {
-	    result         = &dir->result;
-	    WideCharToMultiByte(WinGetCodepage(encoding), 0,
-				dir->info.name, sizeof(dir->info.name) / sizeof(wchar_t),
-				dir->info_mbname, sizeof(dir->info_mbname) / sizeof(char),
-				NULL, 0);
-	    result->d_name = dir->info_mbname;
-	}
+        if (!dir->result.d_name || _findnext(dir->handle, &dir->info) != -1) {
+            result         = &dir->result;
+            result->d_name = dir->info.name;
+        }
     } else {
-	errno = EBADF;
+        errno = EBADF;
     }
 
     return result;
 }
 
-
-void
-gp_rewinddir(DIR *dir)
+void rewinddir(DIR *dir)
 {
     if (dir && dir->handle != -1) {
-	_findclose(dir->handle);
-	dir->handle = (long) _wfindfirst(dir->name, &dir->info);
-	dir->result.d_name = NULL;
+        _findclose(dir->handle);
+        dir->handle = (long) _findfirst(dir->name, &dir->info);
+        dir->result.d_name = 0;
     }
     else {
-	errno = EBADF;
+        errno = EBADF;
     }
 }
 
@@ -623,10 +593,10 @@ gp_rewinddir(DIR *dir)
     documentation for any purpose is hereby granted without fee, provided
     that this copyright and permissions notice appear in all copies and
     derivatives.
-
+    
     This software is supplied "as is" without express or implied warranty.
 
     But that said, if there are any problems please get in touch.
 
 */
-#endif /* _WIN32 */
+#endif /* !HAVE_DIRENT_H && WIN32 */
