@@ -1,5 +1,5 @@
 /*
- * $Id: tables.h,v 1.103 2017/05/20 01:27:30 sfeam Exp $
+ * $Id: tables.h,v 1.91.2.1 2015/03/29 18:08:06 sfeam Exp $
  */
 
 /* GNUPLOT - tables.h */
@@ -68,7 +68,7 @@ enum plot_axes_id {
 /* plot smooth parameters in plot.h */
 
 /* options for 'save' command */
-enum save_id { SAVE_INVALID, SAVE_FUNCS, SAVE_TERMINAL, SAVE_SET, SAVE_VARS, SAVE_FIT };
+enum save_id { SAVE_INVALID, SAVE_FUNCS, SAVE_TERMINAL, SAVE_SET, SAVE_VARS };
 
 /* options for 'show' and 'set' commands
  * this is rather big, we might be better off with a hash table */
@@ -79,22 +79,23 @@ enum set_id {
     S_COLOR, S_COLORSEQUENCE, S_DASHTYPE, S_DATA, S_DATAFILE,
     S_FUNCTIONS, S_DGRID3D, S_DUMMY, S_ENCODING, S_DECIMALSIGN, S_FIT,
     S_FONTPATH, S_FORMAT,
-    S_GRID, S_HIDDEN3D, S_HISTORY, S_HISTORYSIZE, S_ISOSAMPLES, S_JITTER, S_KEY,
-    S_LABEL, S_LINK, S_NONLINEAR,
+    S_GRID, S_HIDDEN3D, S_HISTORY, S_HISTORYSIZE, S_ISOSAMPLES, S_KEY,
+    S_LABEL, S_LINK,
     S_LINESTYLE, S_LINETYPE, S_LOADPATH, S_LOCALE, S_LOGSCALE, S_MACROS,
     S_MAPPING, S_MARGIN, S_LMARGIN, S_RMARGIN, S_TMARGIN, S_BMARGIN, S_MISSING,
-    S_MICRO, S_MINUS_SIGN,
+#ifdef USE_MOUSE
     S_MOUSE,
+#endif
     S_MONOCHROME, S_MULTIPLOT, S_MX2TICS, S_NOMX2TICS, S_MXTICS, S_NOMXTICS,
     S_MY2TICS, S_NOMY2TICS, S_MYTICS, S_NOMYTICS,
-    S_MZTICS, S_NOMZTICS, S_MRTICS,
+    S_MZTICS, S_NOMZTICS, S_MRTICS, S_NOMRTICS,
     S_OFFSETS, S_ORIGIN, SET_OUTPUT, S_PARAMETRIC,
     S_PALETTE, S_PM3D, S_COLORBOX, S_COLORNAMES,
     S_CBLABEL, S_CBRANGE, S_CBTICS, S_NOCBTICS, S_MCBTICS, S_NOMCBTICS,
     S_CBDATA, S_CBDTICS, S_NOCBDTICS, S_CBMTICS, S_NOCBMTICS, S_OBJECT,
     S_PLOT, S_POINTINTERVALBOX, S_POINTSIZE, S_POLAR, S_PRINT, S_PSDIR,
-    S_RGBMAX, S_SAMPLES, S_SIZE, S_SURFACE, S_STYLE, 
-    S_TABLE, S_TERMINAL, S_TERMOPTIONS, S_THETA,
+    S_SAMPLES, S_SIZE, S_SURFACE, S_STYLE, 
+    S_TABLE, S_TERMINAL, S_TERMOPTIONS,
     S_TICS, S_TICSCALE, S_TICSLEVEL, S_TIMEFMT, S_TIMESTAMP, S_TITLE,
     S_TRANGE, S_URANGE, S_VARIABLES, S_VERSION, S_VIEW, S_VRANGE,
 
@@ -111,7 +112,7 @@ enum set_id {
     S_ZDATA, S_ZDTICS, S_NOZDTICS, S_ZLABEL, S_ZMTICS, S_NOZMTICS, S_ZRANGE,
     S_ZTICS, S_NOZTICS,
 
-    S_RTICS, S_RRANGE, S_RAXIS, S_RLABEL, S_PAXIS, S_TTICS, S_MTTICS, 
+    S_RTICS, S_NORTICS, S_RRANGE, S_RAXIS, S_PAXIS,
 
     S_ZERO, S_ZEROAXIS, S_XZEROAXIS, S_X2ZEROAXIS, S_YZEROAXIS, S_Y2ZEROAXIS,
     S_ZZEROAXIS
@@ -129,7 +130,7 @@ enum set_key_id {
     S_KEY_INVALID,
     S_KEY_TOP, S_KEY_BOTTOM, S_KEY_LEFT, S_KEY_RIGHT, S_KEY_CENTER,
     S_KEY_VERTICAL, S_KEY_HORIZONTAL, S_KEY_OVER, S_KEY_UNDER, S_KEY_MANUAL,
-    S_KEY_INSIDE, S_KEY_OUTSIDE, S_KEY_FIXED, S_KEY_ABOVE, S_KEY_BELOW,
+    S_KEY_INSIDE, S_KEY_OUTSIDE, S_KEY_ABOVE, S_KEY_BELOW,
     S_KEY_TMARGIN, S_KEY_BMARGIN, S_KEY_LMARGIN, S_KEY_RMARGIN,
     S_KEY_LLEFT, S_KEY_RRIGHT, S_KEY_REVERSE, S_KEY_NOREVERSE,
     S_KEY_INVERT, S_KEY_NOINVERT,
@@ -149,7 +150,6 @@ enum set_colorbox_id {
     S_COLORBOX_DEFAULT, S_COLORBOX_USER,
     S_COLORBOX_BORDER, S_COLORBOX_BDEFAULT, S_COLORBOX_NOBORDER,
     S_COLORBOX_ORIGIN, S_COLORBOX_SIZE,
-    S_COLORBOX_INVERT, S_COLORBOX_NOINVERT,
     S_COLORBOX_FRONT, S_COLORBOX_BACK
 };
 
@@ -169,12 +169,11 @@ enum set_pm3d_id {
     S_PM3D_SCANSFORWARD, S_PM3D_SCANSBACKWARD, S_PM3D_SCANS_AUTOMATIC,
     S_PM3D_DEPTH,
     S_PM3D_FLUSH, S_PM3D_FTRIANGLES, S_PM3D_NOFTRIANGLES,
-    S_PM3D_CLIP_1IN, S_PM3D_CLIP_4IN, S_PM3D_CLIPCB, S_PM3D_NOCLIPCB,
+    S_PM3D_CLIP_1IN, S_PM3D_CLIP_4IN,
     S_PM3D_MAP, S_PM3D_BORDER, S_PM3D_NOBORDER, S_PM3D_HIDDEN, S_PM3D_NOHIDDEN,
     S_PM3D_SOLID, S_PM3D_NOTRANSPARENT, S_PM3D_NOSOLID, S_PM3D_TRANSPARENT,
     S_PM3D_IMPLICIT, S_PM3D_NOEXPLICIT, S_PM3D_NOIMPLICIT, S_PM3D_EXPLICIT,
-    S_PM3D_WHICH_CORNER,
-    S_PM3D_LIGHTING_MODEL, S_PM3D_NOLIGHTING_MODEL
+    S_PM3D_WHICH_CORNER
 };
 
 enum test_id {
