@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.22.2.1 2015/01/16 18:47:57 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - tabulate.c */
 
 /*[
@@ -58,10 +62,8 @@
 FILE *table_outfile = NULL;
 udvt_entry *table_var = NULL;
 TBOOLEAN table_mode = FALSE;
-char *table_sep = NULL;
 
 static char *expand_newline __PROTO((const char *in));
-static TBOOLEAN imploded __PROTO((curve_points *this_plot));
 
 static FILE *outfile;
 
@@ -91,11 +93,9 @@ output_number(double coord, int axis, char *buffer) {
 	    gstrftime(buffer+1, BUFFERSIZE-1, axis_array[axis].formatstring, coord);
 	while (strchr(buffer,'\n')) {*(strchr(buffer,'\n')) = ' ';}
 	strcat(buffer,"\"");
-#if !defined(NONLINEAR_AXES) || (NONLINEAR_AXES == 0)
     } else if (axis_array[axis].log) {
 	double x = pow(axis_array[axis].base, coord);
 	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, x);
-#endif
     } else
 	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, coord);
     strcat(buffer, " ");
@@ -225,9 +225,6 @@ print_table(struct curve_points *current_plot, int plot_num)
 
 	} else {
 	    int plotstyle = current_plot->plot_style;
-	    int type;
-	    TBOOLEAN replace_undefined_with_blank = imploded(current_plot);
-
 	    if (plotstyle == HISTOGRAMS && current_plot->histogram->type == HT_ERRORBARS)
 		plotstyle = YERRORBARS;
 
@@ -320,19 +317,12 @@ print_table(struct curve_points *current_plot, int plot_num)
 		    }
 		}
 
-		type = current_plot->points[i].type;
-
 		snprintf(buffer, BUFFERSIZE, " %c",
-			type == INRANGE ? 'i' : type == OUTRANGE ? 'o' : 'u');
+		    current_plot->points[i].type == INRANGE
+		    ? 'i' : current_plot->points[i].type == OUTRANGE
+		    ? 'o' : 'u');
 		strappend(&line, &size, len, buffer);
-
-		/* cp_implode() inserts dummy undefined point between curves */
-		/* but datafiles use a blank line for this purpose */
-		if (type == UNDEFINED && replace_undefined_with_blank)
-		    print_line("");
-		else 
-		    print_line(line);
-
+		print_line(line);
 	    } /* for(point i) */
 	}
 
@@ -401,12 +391,8 @@ print_3dtable(int pcount)
 	case IMPULSES:
 	case DOTS:
 	case VECTOR:
-	    break;
 	case IMAGE:
-	case RGBIMAGE:
-	case RGBA_IMAGE:
 	    break;
-
 	default:
 	    fprintf(stderr, "Tabular output of this 3D plot style not implemented\n");
 	    continue;
@@ -426,22 +412,11 @@ print_3dtable(int pcount)
 			curve, icrvs->p_count);
 		print_line(line);
 		len = sprintf(line, "# x y z");
-		tail = NULL;  /* Just to shut up a compiler warning */
-
-		switch (this_plot->plot_style) {
-		case VECTOR:
+		if (this_plot->plot_style == VECTOR) {
 		    tail = icrvs->next->points;
 		    len = strappend(&line, &size, len, " delta_x delta_y delta_z");
-		    break;
-		case IMAGE:
-		    len = strappend(&line, &size, len, "  pixel");
-		    break;
-		case RGBIMAGE:
-		case RGBA_IMAGE:
-		    len = strappend(&line, &size, len, "  red green blue alpha");
-		    break;
-		default:
-		    break;
+		} else {
+		    tail = NULL;  /* Just to shut up a compiler warning */
 		}
 
 		strappend(&line, &size, len, " type");
@@ -463,12 +438,6 @@ print_3dtable(int pcount)
 		    } else if (this_plot->plot_style == IMAGE) {
 			snprintf(buffer, BUFFERSIZE, "%g ", point->CRD_COLOR);
 			len = strappend(&line, &size, len, buffer);
-		    } else if (this_plot->plot_style == RGBIMAGE
-			   ||  this_plot->plot_style == RGBA_IMAGE) {
-			snprintf(buffer, BUFFERSIZE, "%4d %4d %4d %4d ", 
-			        (int)point->CRD_R, (int)point->CRD_G,
-			        (int)point->CRD_B, (int)point->CRD_A);
-			len = strappend(&line, &size, len, buffer);
 		    }
 		    snprintf(buffer, BUFFERSIZE, "%c",
 			    point->type == INRANGE
@@ -476,10 +445,10 @@ print_3dtable(int pcount)
 			    ? 'o' : 'u');
 		    strappend(&line, &size, len, buffer);
 		    print_line(line);
-		} /* for (point) */
-	    } /* for (icrvs) */
+		} /* for(point) */
+	    } /* for(icrvs) */
 	    print_line("");
-	} /* if (draw_surface) */
+	} /* if(draw_surface) */
 
 	if (draw_contour) {
 	    int number = 0;
@@ -489,7 +458,7 @@ print_3dtable(int pcount)
 		int count = c->num_pts;
 		struct coordinate *point = c->coords;
 
-		if (c->isNewLevel) {
+		if (c->isNewLevel)
 		    /* don't display count - contour split across chunks */
 		    /* put # in case user wants to use it for a plot */
 		    /* double blank line to allow plot ... index ... */
@@ -497,7 +466,6 @@ print_3dtable(int pcount)
 		    snprintf(line, size, "# Contour %d, label: %s",
 			    number++, c->label);
 		    print_line(line);
-		}
 
 		for (; --count >= 0; ++point) {
 		    line[0] = NUL;
@@ -526,7 +494,7 @@ print_3dtable(int pcount)
 static char *
 expand_newline(const char *in)
 {
-    char *tmpstr = (char *) gp_alloc(2 * strlen(in) + 1, "enl");
+    char *tmpstr = (char *) gp_alloc(2 * strlen(in), "enl");
     const char *s = in;
     char *t = tmpstr;
     do {
@@ -538,74 +506,3 @@ expand_newline(const char *in)
     } while (*s++);
     return tmpstr;
 }
-
-static TBOOLEAN
-imploded(curve_points *this_plot)
-{
-    switch (this_plot->plot_smooth) {
-	/* These smooth styles called cp_implode() */
-	case SMOOTH_UNIQUE:
-	case SMOOTH_FREQUENCY:
-	case SMOOTH_FREQUENCY_NORMALISED:
-	case SMOOTH_CUMULATIVE:
-	case SMOOTH_CUMULATIVE_NORMALISED:
-	case SMOOTH_CSPLINES:
-	case SMOOTH_ACSPLINES:
-	case SMOOTH_SBEZIER:
-	case SMOOTH_MONOTONE_CSPLINE:
-	    return TRUE;
-	/* These ones did not */
-	case SMOOTH_NONE:
-	case SMOOTH_BEZIER:
-	case SMOOTH_KDENSITY:
-	default:
-	    break;
-    }
-    return FALSE;
-}
-
-/*
- * Called from plot2d.c (get_data) for "plot with table"
- */
-TBOOLEAN
-tabulate_one_line(double v[MAXDATACOLS], struct value str[MAXDATACOLS], int ncols)
-{
-    int col;
-    FILE *outfile = (table_outfile) ? table_outfile : gpoutfile;
-
-    if (table_var == NULL) {
-	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
-	for (col = 0; col < ncols; col++) {
-	    if (str[col].type == STRING)
-		fprintf(outfile, " %s", str[col].v.string_val);
-	    else
-		fprintf(outfile, " %g", v[col]);
-	    if (col < ncols-1)
-		fprintf(outfile, "%c", sep);
-	}
-	fprintf(outfile, "\n");
-    } else {
-	char buf[64]; /* buffer large enough to hold %g + 2 extra chars */
-	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
-	size_t size = sizeof(buf);
-	char *line = (char *) gp_alloc(size, "");
-	size_t len = 0;
-
-	line[0] = NUL;
-	for (col = 0; col < ncols; col++) {
-	    if (str[col].type == STRING) {
-		len = strappend(&line, &size, 0, str[col].v.string_val);
-	    } else {
-		snprintf(buf, sizeof(buf), " %g", v[col]);
-		len = strappend(&line, &size, len, buf);
-	    }
-	    if (col < ncols-1) {
-		snprintf(buf, sizeof(buf), " %c", sep);
-		len = strappend(&line, &size, len, buf);
-	    }
-	}
-	append_to_datablock(&table_var->udv_value, line);
-    }
- 
-    return TRUE;
-}
