@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: unset.c,v 1.206.2.9 2016/01/11 23:32:51 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - unset.c */
 
 /*[
@@ -39,9 +43,7 @@
 #include "fit.h"
 #include "gp_hist.h"
 #include "hidden3d.h"
-#include "jitter.h"
 #include "misc.h"
-#include "multiplot.h"
 #include "parse.h"
 #include "plot.h"
 #include "plot2d.h"
@@ -52,103 +54,94 @@
 #include "util.h"
 #include "variable.h"
 #include "pm3d.h"
+
+static void unset_angles __PROTO((void));
+static void unset_arrow __PROTO((void));
+static void unset_arrowstyles __PROTO((void));
+static void free_arrowstyle __PROTO((struct arrowstyle_def *));
+static void delete_arrow __PROTO((struct arrow_def *, struct arrow_def *));
+static void unset_autoscale __PROTO((void));
+static void unset_bars __PROTO((void));
+static void unset_border __PROTO((void));
+static void unset_boxplot __PROTO((void));
+static void unset_boxwidth __PROTO((void));
+static void unset_fillstyle __PROTO((void));
+static void unset_clip __PROTO((void));
+static void unset_cntrparam __PROTO((void));
+static void unset_cntrlabel __PROTO((void));
+static void unset_contour __PROTO((void));
+static void unset_dashtype __PROTO((void));
+static void unset_dgrid3d __PROTO((void));
+static void unset_dummy __PROTO((void));
+static void unset_encoding __PROTO((void));
+static void unset_decimalsign __PROTO((void));
+static void unset_fit __PROTO((void));
+static void unset_grid __PROTO((void));
+static void unset_hidden3d __PROTO((void));
+static void unset_histogram __PROTO((void));
+#ifdef EAM_BOXED_TEXT
+static void unset_textbox_style __PROTO((void));
+#endif
+static void unset_historysize __PROTO((void));
+static void unset_isosamples __PROTO((void));
+static void unset_key __PROTO((void));
+static void unset_label __PROTO((void));
+static void delete_label __PROTO((struct text_label * prev, struct text_label * this));
+static void unset_linestyle __PROTO((struct linestyle_def **head));
+static void unset_linetype __PROTO((void));
+#ifdef EAM_OBJECTS
+static void unset_object __PROTO((void));
+static void delete_object __PROTO((struct object * prev, struct object * this));
+static void unset_style_rectangle __PROTO(());
+static void unset_style_circle __PROTO(());
+static void unset_style_ellipse __PROTO(());
+#endif
+static void unset_loadpath __PROTO((void));
+static void unset_locale __PROTO((void));
+static void reset_logscale __PROTO((AXIS_INDEX));
+static void unset_logscale __PROTO((void));
+static void unset_mapping __PROTO((void));
+static void unset_margin __PROTO((t_position *));
+static void unset_missing __PROTO((void));
 #ifdef USE_MOUSE
-#include "mouse.h"
+static void unset_mouse __PROTO((void));
 #endif
-#include "voxelgrid.h"
 
-static void unset_angles(void);
-static void unset_arrow(void);
-static void unset_arrowstyles(void);
-static void free_arrowstyle(struct arrowstyle_def *);
-static void delete_arrow(struct arrow_def *, struct arrow_def *);
-static void unset_autoscale(void);
-static void unset_bars(void);
-static void unset_border(void);
-static void unset_boxplot(void);
-static void unset_boxdepth(void);
-static void unset_boxwidth(void);
-static void unset_fillstyle(void);
-static void unset_clip(void);
-static void unset_cntrparam(void);
-static void unset_cntrlabel(void);
-static void unset_contour(void);
-static void unset_dashtype(void);
-static void unset_dgrid3d(void);
-static void unset_dummy(void);
-static void unset_encoding(void);
-static void unset_decimalsign(void);
-static void unset_fit(void);
-static void unset_grid(void);
-static void unset_hidden3d(void);
-static void unset_histogram(void);
-static void unset_textbox_style(void);
-static void unset_historysize(void);
-static void unset_pixmaps(void);
-static void unset_pixmap(int);
-static void unset_isosamples(void);
-static void unset_key(void);
-static void unset_label(void);
-static void delete_label(struct text_label * prev, struct text_label * this);
-static void unset_linestyle(struct linestyle_def **head);
-static void unset_linetype(void);
-static void unset_object(void);
-static void delete_object(struct object * prev, struct object * this);
-static void unset_style_rectangle(void);
-static void unset_style_circle(void);
-static void unset_style_ellipse(void);
-static void unset_style_parallel(void);
-static void unset_style_spiderplot(void);
-static void unset_wall(int which);
-static void unset_loadpath(void);
-static void unset_locale(void);
-static void reset_logscale(struct axis *);
-static void unset_logscale(void);
-static void unset_mapping(void);
-static void unset_margin(t_position *);
-static void unset_missing(void);
-static void unset_micro(void);
-static void unset_minus_sign(void);
-static void unset_mouse(void);
+static void unset_month_day_tics __PROTO((AXIS_INDEX));
+static void unset_minitics __PROTO((AXIS_INDEX));
 
-static void unset_month_day_tics(AXIS_INDEX);
-static void unset_minitics(struct axis *);
+static void unset_offsets __PROTO((void));
+static void unset_origin __PROTO((void));
+static void unset_output __PROTO((void));
+static void unset_parametric __PROTO((void));
+static void unset_pm3d __PROTO((void));
+static void unset_palette __PROTO((void));
+static void reset_colorbox __PROTO((void));
+static void unset_colorbox __PROTO((void));
+static void unset_pointsize __PROTO((void));
+static void unset_pointintervalbox __PROTO((void));
+static void unset_polar __PROTO((void));
+static void unset_print __PROTO((void));
+static void unset_psdir __PROTO((void));
+static void unset_samples __PROTO((void));
+static void unset_size __PROTO((void));
+static void unset_style __PROTO((void));
+static void unset_surface __PROTO((void));
+static void unset_table __PROTO((void));
+static void unset_terminal __PROTO((void));
+static void unset_tics __PROTO((AXIS_INDEX));
+static void unset_ticslevel __PROTO((void));
+static void unset_timefmt __PROTO((void));
+static void unset_timestamp __PROTO((void));
+static void unset_view __PROTO((void));
+static void unset_zero __PROTO((void));
+static void unset_timedata __PROTO((AXIS_INDEX));
+static void unset_range __PROTO((AXIS_INDEX));
+static void unset_zeroaxis __PROTO((AXIS_INDEX));
+static void unset_all_zeroaxes __PROTO((void));
 
-static void unset_offsets(void);
-static void unset_origin(void);
-static void unset_output(void);
-static void unset_parametric(void);
-static void unset_pm3d(void);
-static void unset_palette(void);
-static void reset_colorbox(void);
-static void unset_colorbox(void);
-static void unset_pointsize(void);
-static void unset_pointintervalbox(void);
-static void unset_polar(void);
-static void unset_print(void);
-static void unset_psdir(void);
-static void unset_samples(void);
-static void unset_size(void);
-static void unset_style(void);
-static void unset_spiderplot(void);
-static void unset_surface(void);
-static void unset_table(void);
-static void unset_terminal(void);
-static void unset_tics(struct axis *);
-static void unset_ticslevel(void);
-static void unset_timefmt(void);
-static void unset_timestamp(void);
-static void unset_view(void);
-static void unset_zero(void);
-static void unset_timedata(AXIS_INDEX);
-static void unset_range(AXIS_INDEX);
-static void unset_zeroaxis(AXIS_INDEX);
-static void unset_all_zeroaxes(void);
-
-static void unset_axislabel_or_title(text_label *);
-static void unset_axislabel(AXIS_INDEX);
-
-static void reset_mouse(void);
+static void unset_axislabel_or_title __PROTO((text_label *));
+static void unset_axislabel __PROTO((AXIS_INDEX));
 
 /******** The 'unset' command ********/
 void
@@ -170,12 +163,12 @@ unset_command()
     }
     if (forever_iteration(set_iterator)) {
 	set_iterator = cleanup_iteration(set_iterator);
-	int_error(save_token, "unbounded iteration not accepted here");
+	int_error(save_token, "unbounded iteration");
     }
 
     found_token = lookup_table(&set_tbl[0],c_token);
 
-    /* HBB 20000506: rationalize occurrences of c_token++ ... */
+    /* HBB 20000506: rationalize occurences of c_token++ ... */
     if (found_token != S_INVALID)
 	c_token++;
 
@@ -198,9 +191,6 @@ unset_command()
     case S_BORDER:
 	unset_border();
 	break;
-    case S_BOXDEPTH:
-	unset_boxdepth();
-	break;
     case S_BOXWIDTH:
 	unset_boxwidth();
 	break;
@@ -225,9 +215,6 @@ unset_command()
     case S_DGRID3D:
 	unset_dgrid3d();
 	break;
-    case S_DEBUG:
-	debug = 0;
-	break;
     case S_DUMMY:
 	unset_dummy();
 	break;
@@ -255,20 +242,9 @@ unset_command()
     case S_HISTORYSIZE:	/* Deprecated */
 	unset_historysize();
 	break;
-    case S_PIXMAP:
-	if (END_OF_COMMAND)
-	    unset_pixmaps();
-	else {
-	    i = int_expression();
-	    unset_pixmap(i);
-	}
-	break;
     case S_ISOSAMPLES:
 	unset_isosamples();
 	break;
-    case S_JITTER:
-	unset_jitter();
-	break;
     case S_KEY:
 	unset_key();
 	break;
@@ -279,7 +255,6 @@ unset_command()
 	unset_linetype();
 	break;
     case S_LINK:
-    case S_NONLINEAR:
 	c_token--;
 	link_command();
 	break;
@@ -343,10 +318,6 @@ unset_command()
 	    df_nofpe_trap = FALSE;
 	    c_token++;
 	    break;
-	} else if (almost_equals(c_token,"columnhead$ers")) {
-	    df_columnheaders = FALSE;
-	    c_token++;
-	    break;
 	}
 	df_fortran_constants = FALSE;
 	unset_missing();
@@ -355,20 +326,15 @@ unset_command()
 	free(df_commentschars);
 	df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
 	df_unset_datafile_binary();
-	df_columnheaders = FALSE;
-	break;
-    case S_MICRO:
-	unset_micro();
-	break;
-    case S_MINUS_SIGN:
-	unset_minus_sign();
 	break;
     case S_MONOCHROME:
 	unset_monochrome();
 	break;
+#ifdef USE_MOUSE
     case S_MOUSE:
 	unset_mouse();
 	break;
+#endif
     case S_MULTIPLOT:
 	term_end_multiplot();
 	break;
@@ -381,9 +347,6 @@ unset_command()
     case SET_OUTPUT:
 	unset_output();
 	break;
-    case S_OVERFLOW:
-	overflow_handling = INT64_OVERFLOW_IGNORE;
-	break;
     case S_PARAMETRIC:
 	unset_parametric();
 	break;
@@ -411,39 +374,29 @@ unset_command()
     case S_PSDIR:
 	unset_psdir();
 	break;
+#ifdef EAM_OBJECTS
     case S_OBJECT:
 	unset_object();
 	break;
-    case S_WALL:
-	for (i=0; i<5; i++)
-	    unset_wall(i);
-	break;
+#endif
     case S_RTICS:
-	unset_tics(&axis_array[POLAR_AXIS]);
-	break;
-    case S_TTICS:
-	unset_tics(&THETA_AXIS);
+	unset_tics(POLAR_AXIS);
 	break;
     case S_PAXIS:
 	i = int_expression();
+	if (i <= 0 || i > MAX_PARALLEL_AXES)
+	    int_error(c_token, "expecting parallel axis number");
 	if (almost_equals(c_token, "tic$s")) {
-	    if (i > 0 && i <= num_parallel_axes)
-		unset_tics(&parallel_axis_array[i-1]);
+	    unset_tics(PARALLEL_AXES+i-1);
 	    c_token++;
 	}
 	break;
-    case S_RGBMAX:
-	rgbmax = 255;
-	break;
     case S_SAMPLES:
 	unset_samples();
 	break;
     case S_SIZE:
 	unset_size();
 	break;
-    case S_SPIDERPLOT:
-	unset_spiderplot();
-	break;
     case S_STYLE:
 	unset_style();
 	break;
@@ -457,7 +410,7 @@ unset_command()
 	unset_terminal();
 	break;
     case S_TICS:
-	unset_all_tics();
+	unset_tics(ALL_AXES);
 	break;
     case S_TICSCALE:
 	int_warn(c_token, "Deprecated syntax - use 'set tics scale default'");
@@ -478,78 +431,72 @@ unset_command()
     case S_VIEW:
 	unset_view();
 	break;
-    case S_VGRID:
-	unset_vgrid();
-	break;
     case S_ZERO:
 	unset_zero();
 	break;
 /* FIXME - are the tics correct? */
     case S_MXTICS:
-	unset_minitics(&axis_array[FIRST_X_AXIS]);
+	unset_minitics(FIRST_X_AXIS);
 	break;
     case S_XTICS:
-	unset_tics(&axis_array[FIRST_X_AXIS]);
+	unset_tics(FIRST_X_AXIS);
 	break;
     case S_XDTICS:
     case S_XMTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MYTICS:
-	unset_minitics(&axis_array[FIRST_Y_AXIS]);
+	unset_minitics(FIRST_Y_AXIS);
 	break;
     case S_YTICS:
-	unset_tics(&axis_array[FIRST_Y_AXIS]);
+	unset_tics(FIRST_Y_AXIS);
 	break;
     case S_YDTICS:
     case S_YMTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MX2TICS:
-	unset_minitics(&axis_array[SECOND_X_AXIS]);
+	unset_minitics(SECOND_X_AXIS);
 	break;
     case S_X2TICS:
-	unset_tics(&axis_array[SECOND_X_AXIS]);
+	unset_tics(SECOND_X_AXIS);
 	break;
     case S_X2DTICS:
     case S_X2MTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MY2TICS:
-	unset_minitics(&axis_array[SECOND_Y_AXIS]);
+	unset_minitics(SECOND_Y_AXIS);
 	break;
     case S_Y2TICS:
-	unset_tics(&axis_array[SECOND_Y_AXIS]);
+	unset_tics(SECOND_Y_AXIS);
 	break;
     case S_Y2DTICS:
     case S_Y2MTICS:
-	unset_month_day_tics(SECOND_Y_AXIS);
+	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MZTICS:
-	unset_minitics(&axis_array[FIRST_Z_AXIS]);
+	unset_minitics(FIRST_Z_AXIS);
 	break;
     case S_ZTICS:
-	unset_tics(&axis_array[FIRST_Z_AXIS]);
+	unset_tics(FIRST_Z_AXIS);
 	break;
     case S_ZDTICS:
     case S_ZMTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MCBTICS:
-	unset_minitics(&axis_array[COLOR_AXIS]);
+	unset_minitics(COLOR_AXIS);
 	break;
     case S_CBTICS:
-	unset_tics(&axis_array[COLOR_AXIS]);
+	unset_tics(COLOR_AXIS);
 	break;
     case S_CBDTICS:
     case S_CBMTICS:
 	unset_month_day_tics(FIRST_X_AXIS);
 	break;
     case S_MRTICS:
-	unset_minitics(&axis_array[POLAR_AXIS]);
-	break;
-    case S_MTTICS:
-	unset_minitics(&THETA_AXIS);
+	unset_minitics(POLAR_AXIS);
 	break;
     case S_XDATA:
 	unset_timedata(FIRST_X_AXIS);
@@ -581,9 +528,6 @@ unset_command()
     case S_CBLABEL:
 	unset_axislabel(COLOR_AXIS);
 	break;
-    case S_RLABEL:
-	unset_axislabel(POLAR_AXIS);
-	break;
     case S_X2LABEL:
 	unset_axislabel(SECOND_X_AXIS);
 	break;
@@ -622,6 +566,7 @@ unset_command()
 	break;
     case S_RAXIS:
 	raxis = FALSE;
+	c_token++;
 	break;
     case S_XZEROAXIS:
 	unset_zeroaxis(FIRST_X_AXIS);
@@ -730,47 +675,6 @@ free_arrowstyle(struct arrowstyle_def *arrowstyle)
     }
 }
 
-/*
- * Deletes all pixmaps.
- */
-static void
-unset_pixmaps(void)
-{
-    t_pixmap *pixmap, *next;
-    for (pixmap = pixmap_listhead; pixmap; pixmap = next) {
-	free(pixmap->filename);
-	free(pixmap->image_data);
-	next = pixmap->next;
-	free(pixmap);
-    }
-    pixmap_listhead = NULL;
-}
-/*
- * Deletes a single pixmap
- */
-static void
-unset_pixmap(int i)
-{
-    t_pixmap *pixmap = pixmap_listhead;
-    t_pixmap *prev = pixmap_listhead;
-    while (pixmap) {
-	if (pixmap->tag == i) {
-	    if (pixmap == pixmap_listhead)
-		prev = pixmap_listhead = pixmap->next;
-	    else
-		prev->next = pixmap->next;
-	    free(pixmap->filename);
-	    free(pixmap->image_data);
-	    free(pixmap);
-	    pixmap = prev ? prev->next : NULL;
-	} else {
-	    prev = pixmap;
-	    pixmap = pixmap->next;
-	}
-    }
-}
-
-
 /* process 'unset autoscale' command */
 static void
 unset_autoscale()
@@ -779,8 +683,6 @@ unset_autoscale()
 	int axis;
 	for (axis=0; axis<AXIS_ARRAY_SIZE; axis++)
 	    axis_array[axis].set_autoscale = FALSE;
-	for (axis=0; axis<num_parallel_axes; axis++)
-	    parallel_axis_array[axis].set_autoscale = FALSE;
     } else if (equals(c_token, "xy") || equals(c_token, "tyx")) {
 	axis_array[FIRST_X_AXIS].set_autoscale
 	    = axis_array[FIRST_Y_AXIS].set_autoscale = AUTOSCALE_NONE;
@@ -788,7 +690,7 @@ unset_autoscale()
     } else {
 	/* HBB 20000506: parse axis name, and unset the right element
 	 * of the array: */
-	AXIS_INDEX axis = lookup_table(axisname_tbl, c_token);
+	int axis = lookup_table(axisname_tbl, c_token);
 	if (axis >= 0) {
 	    axis_array[axis].set_autoscale = AUTOSCALE_NONE;
 	c_token++;
@@ -804,18 +706,6 @@ unset_bars()
     bar_size = 0.0;
 }
 
-/* reset is different from unset */
-/* This gets called from 'set bars default' also */
-void
-reset_bars()
-{
-    struct lp_style_type def = DEFAULT_LP_STYLE_TYPE;
-    bar_lp = def;
-    bar_lp.l_type = LT_DEFAULT;
-    bar_lp.pm3d_color.type = TC_VARIABLE;
-    bar_size = 1.0;
-    bar_layer = LAYER_FRONT;
-}
 
 /* process 'unset border' command */
 static void
@@ -836,14 +726,6 @@ unset_boxplot()
 }
 
 
-/* process 'unset boxdepth' command */
-static void
-unset_boxdepth()
-{
-    boxdepth = 0.0;
-}
-
-
 /* process 'unset boxwidth' command */
 static void
 unset_boxwidth()
@@ -873,17 +755,14 @@ unset_clip()
 	clip_points = FALSE;
 	clip_lines1 = FALSE;
 	clip_lines2 = FALSE;
-	clip_radial = FALSE;
-    } else if (almost_equals(c_token, "r$adial") || equals(c_token, "polar"))
-	clip_radial = FALSE;
-    else if (almost_equals(c_token, "p$oints"))
+    } else if (almost_equals(c_token, "p$oints"))
 	clip_points = FALSE;
     else if (almost_equals(c_token, "o$ne"))
 	clip_lines1 = FALSE;
     else if (almost_equals(c_token, "t$wo"))
 	clip_lines2 = FALSE;
     else
-	int_error(c_token, "expecting 'points', 'one', 'two', or 'radial'");
+	int_error(c_token, "expecting 'points', 'one', or 'two'");
     c_token++;
 }
 
@@ -897,8 +776,6 @@ unset_cntrparam()
     contour_order = DEFAULT_CONTOUR_ORDER;
     contour_levels = DEFAULT_CONTOUR_LEVELS;
     contour_levels_kind = LEVELS_AUTO;
-    contour_firstlinetype = 0;
-    contour_sortlevels = FALSE;
 }
 
 /* process 'unset cntrlabel' command */
@@ -1023,13 +900,10 @@ unset_grid()
     AXIS_INDEX i = 0;
 
     /* grid_selection = GRID_OFF; */
-    for (; i < NUMBER_OF_MAIN_VISIBLE_AXES; i++) {
+    for (; i < AXIS_ARRAY_SIZE; i++) {
 	axis_array[i].gridmajor = FALSE;
 	axis_array[i].gridminor = FALSE;
     }
-    polar_grid_angle = 0;
-    grid_vertical_lines = FALSE;
-    grid_spiderweb = FALSE;
 }
 
 
@@ -1049,18 +923,14 @@ unset_histogram()
     histogram_opts = foo;
 }
 
+#ifdef EAM_BOXED_TEXT
 static void
 unset_textbox_style()
 {
     textbox_style foo = DEFAULT_TEXTBOX_STYLE;
-    int i;
-
-    for (i=0; i<NUM_TEXTBOX_STYLES; i++) {
-	textbox_opts[i] = foo;
-	if (i>0)
-	    textbox_opts[i].linewidth = 0.;
-    }
+    textbox_opts = foo;
 }
+#endif
 
 /* process 'unset historysize' command DEPRECATED */
 static void
@@ -1178,6 +1048,7 @@ unset_linetype()
 	unset_linestyle(&first_perm_linestyle);
 }
 
+#ifdef EAM_OBJECTS
 static void
 unset_object()
 {
@@ -1226,6 +1097,7 @@ delete_object(struct object *prev, struct object *this)
 	free (this);
     }
 }
+#endif
 
 
 /* process 'unset loadpath' command */
@@ -1244,13 +1116,13 @@ unset_locale()
 }
 
 static void
-reset_logscale(struct axis *this_axis)
+reset_logscale(AXIS_INDEX axis)
 {
-    TBOOLEAN undo_rlog = (this_axis->index == POLAR_AXIS && this_axis->log);
-    this_axis->log = FALSE;
+    TBOOLEAN undo_rlog = (axis == POLAR_AXIS && R_AXIS.log);
+    axis_array[axis].log = FALSE;
     /* Do not zero the base because we can still use it for gprintf formats
      * %L and %l with linear axis scaling.
-    this_axis->base = 0.0;
+    axis_array[axis].base = 0.0;
      */
     if (undo_rlog)
 	rrange_to_xy();
@@ -1260,41 +1132,34 @@ reset_logscale(struct axis *this_axis)
 static void
 unset_logscale()
 {
-    TBOOLEAN set_for_axis[AXIS_ARRAY_SIZE] = AXIS_ARRAY_INITIALIZER(FALSE);
     int axis;
 
     if (END_OF_COMMAND) {
-	for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++)
-	    set_for_axis[axis] = TRUE;
+	/* clean all the islog flags. This will hit some currently
+	 * unused ones, too, but that's actually a good thing, IMHO */
+	for(axis = 0; axis < AXIS_ARRAY_SIZE; axis++)
+	    reset_logscale(axis);
     } else {
-	/* do reverse search because of "x", "x1", "x2" sequence in axisname_tbl */
 	int i = 0;
+
+	/* do reverse search because of "x", "x1", "x2" sequence in
+	 * axisname_tbl */
 	while (i < token[c_token].length) {
-	    axis = lookup_table_nth_reverse(axisname_tbl, NUMBER_OF_MAIN_VISIBLE_AXES,
-		       gp_input_line + token[c_token].start_index + i);
+	    axis = lookup_table_nth_reverse(axisname_tbl, LAST_REAL_AXIS+1,
+					    gp_input_line + token[c_token].start_index + i);
 	    if (axis < 0) {
 		token[c_token].start_index += i;
 		int_error(c_token, "invalid axis");
 	    }
-	    set_for_axis[axisname_tbl[axis].value] = TRUE;
+	    reset_logscale(axisname_tbl[axis].value);
 	    i += strlen(axisname_tbl[axis].key);
 	}
-	c_token++;
+	++c_token;
     }
 
-    for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++) {
-	if (set_for_axis[axis]) {
-	    static char command[64];
-	    if (!isalpha((unsigned char)axis_name(axis)[0]))
-		continue;
-	    if (axis_array[axis].log) {
-		sprintf(command, "unset nonlinear %s", axis_name(axis));
-		do_string(command); 
-		axis_array[axis].log = FALSE;
-	    }
-	    axis_array[axis].ticdef.logscaling = FALSE;
-	}
-    }
+    /* Because the log scaling is applied during data input, a quick refresh */
+    /* using existing stored data will not work if the log setting changes.  */
+    SET_REFRESH_OK(E_REFRESH_NOT_OK, 0);
 }
 
 /* process 'unset mapping3d' command */
@@ -1313,20 +1178,6 @@ unset_margin(t_position *margin)
     margin->x = -1;
 }
 
-/* process 'unset micro' command */
-static void
-unset_micro()
-{
-    use_micro = FALSE;
-}
-
-/* process 'unset minus_sign' command */
-static void
-unset_minus_sign()
-{
-    use_minus_sign = FALSE;
-}
-
 /* process 'unset datafile' command */
 static void
 unset_missing()
@@ -1335,64 +1186,64 @@ unset_missing()
     missing_val = NULL;
 }
 
+#ifdef USE_MOUSE
 /* process 'unset mouse' command */
 static void
 unset_mouse()
 {
-#ifdef USE_MOUSE
     mouse_setting.on = 0;
 #ifdef OS2
     PM_update_menu_items();
 #endif
     UpdateStatusline(); /* wipe status line */
-#endif
 }
+#endif
 
 /* process 'unset mxtics' command */
 static void
-unset_minitics(struct axis *this_axis)
+unset_minitics(AXIS_INDEX axis)
 {
-    this_axis->minitics = MINI_OFF;
-    this_axis->mtic_freq = 10.0;
+    axis_array[axis].minitics = MINI_OFF;
+    axis_array[axis].mtic_freq = 10.0;
 }
 
+
 /*process 'unset {x|y|x2|y2|z}tics' command */
-void
-unset_all_tics()
-{
-    int i;
-    for (i = 0; i < NUMBER_OF_MAIN_VISIBLE_AXES; i++)
-	unset_tics(&axis_array[i]);
-}
-
 static void
-unset_tics(struct axis *this_axis)
+unset_tics(AXIS_INDEX axis)
 {
     struct position tics_nooffset = { character, character, character, 0., 0., 0.};
+    unsigned int istart = 0;
+    unsigned int iend = AXIS_ARRAY_SIZE;
+    unsigned int i;
 
-    this_axis->ticmode = NO_TICS;
-
-    if (this_axis->ticdef.font) {
-	free(this_axis->ticdef.font);
-	this_axis->ticdef.font = NULL;
+    if (axis != ALL_AXES) {
+	istart = axis;
+	iend = axis + 1;
     }
-    this_axis->ticdef.textcolor.type = TC_DEFAULT;
-    this_axis->ticdef.textcolor.lt = 0;
-    this_axis->ticdef.textcolor.value = 0;
-    this_axis->ticdef.offset = tics_nooffset;
-    this_axis->ticdef.rangelimited = FALSE; 
-    this_axis->ticdef.enhanced = TRUE;
-    this_axis->tic_rotate = 0;
-    this_axis->ticscale = 1.0;
-    this_axis->miniticscale = 0.5;
-    this_axis->tic_in = TRUE;
-    this_axis->manual_justify = FALSE;
 
-    free_marklist(this_axis->ticdef.def.user);
-    this_axis->ticdef.def.user = NULL;
+    for (i = istart; i < iend; ++i) {
+	axis_array[i].ticmode = NO_TICS;
 
-    if (this_axis->index >= PARALLEL_AXES)
-	this_axis->ticdef.rangelimited = TRUE; 
+	if (axis_array[i].ticdef.font) {
+	    free(axis_array[i].ticdef.font);
+	    axis_array[i].ticdef.font = NULL;
+	}
+	axis_array[i].ticdef.textcolor.type = TC_DEFAULT;
+	axis_array[i].ticdef.textcolor.lt = 0;
+	axis_array[i].ticdef.textcolor.value = 0;
+	axis_array[i].ticdef.offset = tics_nooffset;
+	axis_array[i].ticdef.rangelimited = (i >= PARALLEL_AXES) ? TRUE : FALSE;
+	axis_array[i].ticdef.enhanced = TRUE;
+	axis_array[i].tic_rotate = 0;
+	axis_array[i].ticscale = 1.0;
+	axis_array[i].miniticscale = 0.5;
+	axis_array[i].tic_in = TRUE;
+	axis_array[i].manual_justify = FALSE;
+
+	free_marklist(axis_array[i].ticdef.def.user);
+	axis_array[i].ticdef.def.user = NULL;
+    }
 }
 
 static void
@@ -1512,6 +1363,7 @@ unset_pm3d()
     /* reset styles, required to 'plot something' after e.g. 'set pm3d map' */
     if (data_style == PM3DSURFACE) data_style = POINTSTYLE;
     if (func_style == PM3DSURFACE) func_style = LINES;
+    pm3d.border.l_type = LT_NODRAW;
 }
 
 
@@ -1547,23 +1399,6 @@ unset_polar()
 		(void) fprintf(stderr,"\n\tdummy variable is x for curves\n");
 	}
     }
-    raxis = FALSE;
-    theta_origin = 0.0;
-    theta_direction = 1.0;
-
-    /* Clear and reinitialize THETA axis structure */
-    unset_tics(&THETA_AXIS);
-    unset_minitics(&THETA_AXIS);
-    THETA_AXIS.min = 0.;
-    THETA_AXIS.max = 360.;
-    THETA_AXIS.ticdef = default_axis_ticdef;
-    THETA_AXIS.index = THETA_index;
-    free(THETA_AXIS.formatstring);
-    THETA_AXIS.formatstring = gp_strdup(DEF_FORMAT);
-    THETA_AXIS.ticscale = 1.0;
-    THETA_AXIS.miniticscale = 0.5;
-    THETA_AXIS.tic_in = TRUE;
-    THETA_AXIS.tic_rotate = 0;
 }
 
 
@@ -1572,7 +1407,7 @@ static void
 unset_samples()
 {
     /* HBB 20000506: unlike unset_isosamples(), this one *has* to
-     * clear 2D data structures! */
+     * clear 2D data structues! */
     cp_free(first_plot);
     first_plot = NULL;
 
@@ -1604,12 +1439,16 @@ unset_style()
 	while (first_linestyle != NULL)
 	    delete_linestyle(&first_linestyle, NULL, first_linestyle);
 	unset_fillstyle();
+#ifdef EAM_OBJECTS
 	unset_style_rectangle();
 	unset_style_circle();
 	unset_style_ellipse();
+#endif
 	unset_histogram();
 	unset_boxplot();
+#ifdef EAM_BOXED_TEXT
 	unset_textbox_style();
+#endif
 	c_token++;
 	return;
     }
@@ -1644,6 +1483,7 @@ unset_style()
 	unset_arrowstyles();
 	c_token++;
 	break;
+#ifdef EAM_OBJECTS
     case SHOW_STYLE_RECTANGLE:
 	unset_style_rectangle();
 	c_token++;
@@ -1656,43 +1496,22 @@ unset_style()
 	unset_style_ellipse();
 	c_token++;
 	break;
+#endif
+#ifdef EAM_BOXED_TEXT
     case SHOW_STYLE_TEXTBOX:
 	unset_textbox_style();
 	c_token++;
 	break;
+#endif
     case SHOW_STYLE_BOXPLOT:
 	unset_boxplot();
 	c_token++;
 	break;
-    case SHOW_STYLE_PARALLEL:
-	unset_style_parallel();
-	c_token++;
-	break;
-    case SHOW_STYLE_SPIDERPLOT:
-	unset_style_spiderplot();
-	c_token++;
-	break;
     default:
 	int_error(c_token, "unrecognized style");
     }
 }
 
-static void
-unset_spiderplot()
-{
-    if (spiderplot) {
-	spiderplot = FALSE;
-	data_style = POINTSTYLE;
-	aspect_ratio = 0;
-    }
-}
-
-static void
-unset_style_spiderplot()
-{
-    struct spider_web spiderweb = DEFAULT_SPIDERPLOT_STYLE;
-    spiderplot_style = spiderweb;
-}
 
 /* process 'unset surface' command */
 static void
@@ -1714,7 +1533,7 @@ unset_table()
 }
 
 
-/* process 'unset terminal' command */
+/* process 'unset terminal' comamnd */
 /* Aug 2012:  restore original terminal type */
 static void
 unset_terminal()
@@ -1726,14 +1545,9 @@ unset_terminal()
 
     term_reset();
 
-    /* FIXME: change is correct but reported result is truncated */
-    if (original_terminal && original_terminal->udv_value.type != NOTDEFINED) {
-	char *termname = gp_strdup(original_terminal->udv_value.v.string_val);
-	if (strchr(termname, ' '))
-	    *strchr(termname, ' ') = '\0';
-	*term_options = '\0';
+    if (original_terminal) {
+	char *termname = original_terminal->udv_value.v.string_val;
 	term = change_term(termname, strlen(termname));
-	free(termname);
     }
     screen_ok = FALSE;
 }
@@ -1762,7 +1576,7 @@ static void
 unset_timestamp()
 {
     unset_axislabel_or_title(&timelabel);
-    timelabel.rotate = 0;
+    timelabel_rotate = 0;
     timelabel_bottom = TRUE;
 }
 
@@ -1772,17 +1586,15 @@ static void
 unset_view()
 {
     splot_map = FALSE;
-    xz_projection = FALSE;
-    yz_projection = FALSE;
     aspect_ratio_3D = 0;
     surface_rot_z = 30.0;
     surface_rot_x = 60.0;
     surface_scale = 1.0;
     surface_lscale = 0.0;
     surface_zscale = 1.0;
-    azimuth = 0.0;
 }
 
+
 /* process 'unset zero' command */
 static void
 unset_zero()
@@ -1803,14 +1615,14 @@ unset_timedata(AXIS_INDEX axis)
 static void
 unset_range(AXIS_INDEX axis)
 {
-    struct axis *this_axis = &axis_array[axis];
-
-    this_axis->writeback_min = this_axis->set_min = axis_defaults[axis].min;
-    this_axis->writeback_max = this_axis->set_max = axis_defaults[axis].max;
-    this_axis->set_autoscale = AUTOSCALE_BOTH;
-    this_axis->min_constraint = CONSTRAINT_NONE;
-    this_axis->max_constraint = CONSTRAINT_NONE;
-    this_axis->range_flags = 0;
+    axis_array[axis].set_autoscale = AUTOSCALE_BOTH;
+    axis_array[axis].writeback_min = axis_array[axis].set_min
+	= axis_defaults[GPMIN(axis,PARALLEL_AXES)].min;
+    axis_array[axis].writeback_max = axis_array[axis].set_max
+	= axis_defaults[GPMIN(axis,PARALLEL_AXES)].max;
+    axis_array[axis].min_constraint = CONSTRAINT_NONE;
+    axis_array[axis].max_constraint = CONSTRAINT_NONE;
+    axis_array[axis].range_flags = 0;
 }
 
 /* process 'unset {x|y|x2|y2|z}zeroaxis' command */
@@ -1829,22 +1641,27 @@ unset_all_zeroaxes()
 {
     AXIS_INDEX axis;
 
-    for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++)
+    for(axis = 0; axis < AXIS_ARRAY_SIZE; axis++)
 	unset_zeroaxis(axis);
 }
 
 
+/* process 'unset [xyz]{2}label command */
 static void
 unset_axislabel_or_title(text_label *label)
 {
+    struct position default_offset = { character, character, character,
+				       0., 0., 0. };
     if (label) {
 	free(label->text);
+	label->text = NULL;
 	free(label->font);
-	*label = default_axis_label;
+	label->font = NULL;
+	label->offset = default_offset;
+	label->textcolor.type = TC_DEFAULT;
     }
 }
 
-/* process 'unset [xyz]{2}label command */
 static void
 unset_axislabel(AXIS_INDEX axis)
 {
@@ -1854,29 +1671,6 @@ unset_axislabel(AXIS_INDEX axis)
 	axis_array[axis].label.rotate = TEXT_VERTICAL;
 }
 
-/*
- * Free dynamic fields in an axis structure so that it can be safely deleted
- * or reinitialized.  Doesn't free the axis structure itself.
- * SAMPLE_AXIS is an exception because its link pointers are only copies of
- * those in the real axis being sampled.
- */
-static void
-free_axis_struct(struct axis *this_axis)
-{
-	free(this_axis->formatstring);
-	free(this_axis->ticfmt);
-	if (this_axis->link_udf && this_axis->index != SAMPLE_AXIS) {
-	    free(this_axis->link_udf->at);
-	    free(this_axis->link_udf->definition);
-	    free(this_axis->link_udf);
-	}
-	free_marklist(this_axis->ticdef.def.user);
-	free(this_axis->ticdef.font);
-	unset_axislabel_or_title(&this_axis->label);
-	if (this_axis->zeroaxis != &default_axis_zeroaxis)
-	    free(this_axis->zeroaxis);
-}
-
 /******** The 'reset' command ********/
 /* HBB 20000506: I moved this here, from set.c, because 'reset' really
  * is more like a big lot of 'unset' commands, rather than a bunch of
@@ -1900,7 +1694,6 @@ reset_command()
 	clear_udf_list();
 	init_constants();
 	init_session();
-	reset_mouse();
 	return;
     }
 
@@ -1921,28 +1714,11 @@ reset_command()
 #endif
 
     if (!(END_OF_COMMAND)) {
-	int_warn(c_token, "invalid option, expecting 'session', 'bind' or 'errorstate'");
+	int_warn(c_token, "invalid option, expecting 'bind' or 'errorstate'");
 	while (!(END_OF_COMMAND))
 	    c_token++;
     }
 
-#if (0)
-    /* DEBUG - enable this code for testing
-     * The *.dem unit tests all end with a "reset" command.
-     * In order to test save/load from a wide variety of states we can intercept
-     * this command and insert save/load before execution.
-     */
-    extern TBOOLEAN successful_initialization;
-    if (successful_initialization) {
-	FILE *fp = fopen("/tmp/gnuplot_debug.sav", "w+");
-	replot_line[0] = '\0';
-	save_all(fp);
-	rewind(fp);
-	load_file(fp, gp_strdup("/tmp/gnuplot_debug.sav"), 1);
-	/* load_file closes fp */
-    }
-#endif
-
     /* Kludge alert, HBB 20000506: set to noninteractive mode, to
      * suppress some of the commentary output by the individual
      * unset_...() routines. */
@@ -1951,8 +1727,6 @@ reset_command()
     unset_samples();
     unset_isosamples();
 
-    unset_jitter();
-
     /* delete arrows */
     while (first_arrow != NULL)
 	delete_arrow((struct arrow_def *) NULL, first_arrow);
@@ -1963,14 +1737,14 @@ reset_command()
     /* delete linestyles */
     while (first_linestyle != NULL)
 	delete_linestyle(&first_linestyle, NULL, first_linestyle);
+#ifdef EAM_OBJECTS
     /* delete objects */
     while (first_object != NULL)
 	delete_object((struct object *) NULL, first_object);
     unset_style_rectangle();
     unset_style_circle();
     unset_style_ellipse();
-    /* delete pixmaps */
-    unset_pixmaps();
+#endif
 
     /* 'polar', 'parametric' and 'dummy' are interdependent, so be
      * sure to keep the order intact */
@@ -1978,9 +1752,6 @@ reset_command()
     unset_parametric();
     unset_dummy();
 
-    unset_spiderplot();
-    unset_style_spiderplot();
-
     unset_axislabel_or_title(&title);
 
     reset_key();
@@ -1989,62 +1760,55 @@ reset_command()
 
     for (axis=0; axis<AXIS_ARRAY_SIZE; axis++) {
 
-	struct axis *this_axis = &axis_array[axis];
+	AXIS default_axis_state = DEFAULT_AXIS_STRUCT;
 
 	/* Free contents before overwriting with default values */
-	free_axis_struct(this_axis);
+	free(axis_array[axis].formatstring);
+	if (axis_array[axis].link_udf) {
+	    free(axis_array[axis].link_udf->at);
+	    free(axis_array[axis].link_udf->definition);
+	    free(axis_array[axis].link_udf);
+	}
+	free_marklist(axis_array[axis].ticdef.def.user);
+	free(axis_array[axis].ticdef.font);
+	unset_zeroaxis(axis);
+	unset_axislabel_or_title(&axis_array[axis].label);
 
-	/* Fill with generic values, then customize */
-	memcpy(this_axis, &default_axis_state, sizeof(AXIS));
+	memcpy(axis_array+axis, &default_axis_state, sizeof(AXIS));
 
-	this_axis->formatstring = gp_strdup(DEF_FORMAT);
-	this_axis->index = axis;
-	unset_axislabel(axis);	/* sets vertical label for y/y2/cb */
-	unset_range(axis);	/* copies min/max from axis_defaults */
+	unset_axislabel(axis);
+	axis_array[axis].formatstring = gp_strdup(DEF_FORMAT);
+	unset_timedata(axis);
+	unset_range(axis);
 
 	/* 'tics' default is on for some, off for the other axes: */
-	unset_tics(this_axis);
-	unset_minitics(this_axis);
-	this_axis->ticdef = default_axis_ticdef;
-	this_axis->minitics = MINI_DEFAULT;
-	this_axis->ticmode = axis_defaults[axis].ticmode;
+	unset_tics(axis);
+	axis_array[axis].ticmode = axis_defaults[GPMIN(axis,PARALLEL_AXES)].ticmode;
+	unset_minitics(axis);
+	axis_array[axis].ticdef = default_axis_ticdef;
+	axis_array[axis].minitics = MINI_DEFAULT;
+	if (axis >= PARALLEL_AXES) {
+	    axis_array[axis].ticdef.rangelimited = TRUE;
+	    axis_array[axis].set_autoscale |= AUTOSCALE_FIXMIN | AUTOSCALE_FIXMAX;
+	}
 
-	reset_logscale(this_axis);
+	axis_array[axis].linked_to_primary = FALSE;
+
+	reset_logscale(axis);
     }
-
-    /* Free all previously allocated parallel axis structures */
-    for (axis=0; axis<num_parallel_axes; axis++) {
-	struct axis *this_axis = &parallel_axis_array[axis];
-	free_axis_struct(this_axis);
-    }
-    free(parallel_axis_array);
-    parallel_axis_array = NULL;
-    num_parallel_axes = 0;
-    unset_style_parallel();
-
-    if (shadow_axis_array) {
-	for (i=0; i<NUMBER_OF_MAIN_VISIBLE_AXES; i++)
-	    free_axis_struct(&shadow_axis_array[i]);
-	free(shadow_axis_array);
-	shadow_axis_array = NULL;
-    }
-
-    raxis = FALSE;
+    raxis = TRUE;
     for (i=2; i<MAX_TICLEVEL; i++)
 	ticscale[i] = 1;
     unset_timefmt();
 
     unset_boxplot();
-    unset_boxdepth();
     unset_boxwidth();
 
     clip_points = FALSE;
     clip_lines1 = TRUE;
     clip_lines2 = FALSE;
-    clip_radial = FALSE;
 
     border_lp = default_border_lp;
-    border_layer = LAYER_FRONT;
     draw_border = 31;
 
     draw_surface = TRUE;
@@ -2057,14 +1821,15 @@ reset_command()
     filledcurves_opts_data.closeto = FILLEDCURVES_CLOSED;
     filledcurves_opts_func.closeto = FILLEDCURVES_CLOSED;
 
+    bar_size = 1.0;
+    bar_layer = LAYER_FRONT;
+
     unset_grid();
     grid_lp = default_grid_lp;
     mgrid_lp = default_grid_lp;
     polar_grid_angle = 0;
     grid_layer = LAYER_BEHIND;
     grid_tics_in_front = FALSE;
-    for (i=0; i<5; i++)
-	unset_wall(i);
 
     SET_REFRESH_OK(E_REFRESH_NOT_OK, 0);
 
@@ -2072,12 +1837,10 @@ reset_command()
     hidden3d = FALSE;
 
     unset_angles();
-    reset_bars();
     unset_mapping();
 
     unset_size();
     aspect_ratio = 0.0;		/* don't force it */
-    rgbmax = 255;
 
     unset_origin();
     unset_timestamp();
@@ -2100,23 +1863,22 @@ reset_command()
     df_unset_datafile_binary();
     unset_fillstyle();
     unset_histogram();
+#ifdef EAM_BOXED_TEXT
     unset_textbox_style();
+#endif
+#ifdef BACKWARDS_COMPATIBLE
     prefer_line_styles = FALSE;
+#endif
 
 #ifdef USE_MOUSE
     mouse_setting = default_mouse_setting;
 #endif
 
-    /* restore previous multiplot offset and margins */
-    if (multiplot)
-	multiplot_reset();
-
     unset_missing();
     free(df_separators);
     df_separators = NULL;
     free(df_commentschars);
     df_commentschars = gp_strdup(DEFAULT_COMMENTS_CHARS);
-    df_init();
 
     { /* Preserve some settings for `reset`, but not for `unset fit` */
 	verbosity_level save_verbosity = fit_verbosity;
@@ -2132,6 +1894,7 @@ reset_command()
     interactive = save_interactive;
 }
 
+#ifdef EAM_OBJECTS
 static void
 unset_style_rectangle()
 {
@@ -2153,29 +1916,4 @@ unset_style_ellipse()
     default_ellipse = foo;
     return;
 }
-static void
-unset_style_parallel()
-{
-    struct pa_style parallel_axis_default = DEFAULT_PARALLEL_AXIS_STYLE;
-    parallel_axis_style = parallel_axis_default;
-}
-static void
-unset_wall(int which)
-{
-    grid_wall[which].layer = LAYER_BEHIND;
-}
-
-/* Invoked by "reset session".  There is no command line "reset mouse" */
-static void
-reset_mouse()
-{
-#ifdef USE_MOUSE
-    free_at(mouse_readout_function.at);  /* sets to NULL */
-    free(mouse_readout_function.definition);
-    mouse_readout_function.definition = NULL;
-    free(mouse_alt_string);
-    mouse_alt_string = NULL;
-    mouse_mode = MOUSE_COORDINATES_REAL;
-    mouse_setting = default_mouse_setting;
 #endif
-}
