@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: util.c,v 1.128.2.3 2015/08/03 18:33:02 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - util.c */
 
 /*[
@@ -34,19 +38,23 @@
 
 #include "alloc.h"
 #include "command.h"
-#include "datablock.h"
 #include "datafile.h"		/* for df_showdata and df_reset_after_error */
 #include "internal.h"		/* for eval_reset_after_error */
 #include "misc.h"
 #include "plot.h"
-#include "pm3d.h"		/* for pm3d_reset_after_error */
+#include "term_api.h"		/* for term_end_plot() used by graph_error(), also to detect enhanced mode */
 #include "variable.h"		/* For locale handling */
 #include "setshow.h"		/* for conv_text() */
 #include "tabulate.h"		/* for table_mode */
-#include "voxelgrid.h"
-#include "encoding.h"
-#if defined(_MSC_VER) || defined(__WATCOMC__)
-# include <io.h>		/* for _access */
+
+#if defined(HAVE_DIRENT_H)
+# include <sys/types.h>
+# include <dirent.h>
+#elif defined(_Windows)
+# include <windows.h>
+#endif
+#if defined(__MSC__) || defined (__WATCOMC__)
+# include <io.h>
 #endif
 
 /* Exported (set-table) variables */
@@ -57,34 +65,20 @@ char *decimalsign = NULL;
 /* degree sign.  Defaults to UTF-8 but will be changed to match encoding */
 char degree_sign[8] = "°";
 
-/* encoding-specific characters used by gprintf() */
-const char *micro = NULL;
-const char *minus_sign = NULL;
-TBOOLEAN use_micro = FALSE;
-TBOOLEAN use_minus_sign = FALSE;
-
 /* Holds the name of the current LC_NUMERIC as set by "set decimal locale" */
 char *numeric_locale = NULL;
 
 /* Holds the name of the current LC_TIME as set by "set locale" */
-char *time_locale = NULL;
+char *current_locale = NULL;
 
 const char *current_prompt = NULL; /* to be set by read_line() */
 
-/* TRUE if command just typed; becomes FALSE whenever we
- * send some other output to screen.  If FALSE, the command line
- * will be echoed to the screen before the ^ error message.
- */
-TBOOLEAN screen_ok;
-
-int debug = 0;
-
 /* internal prototypes */
 
-static void mant_exp(double, double, TBOOLEAN, double *, int *, const char *);
-static void parse_sq(char *);
-static char *utf8_strchrn(const char *s, int N);
-static char *num_to_str(double r);
+static void mant_exp __PROTO((double, double, TBOOLEAN, double *, int *, const char *));
+static void parse_sq __PROTO((char *));
+static TBOOLEAN utf8_getmore __PROTO((unsigned long * wch, const char **str, int nbytes));
+static char *utf8_strchrn __PROTO((const char *s, int N));
 
 /*
  * equals() compares string value of token number t_num with str[], and
@@ -145,19 +139,7 @@ almost_equals(int t_num, const char *str)
     return (after || str[i] == '$' || str[i] == NUL);
 }
 
-/* Extract one token from the input line */
-char *
-token_to_string(int t)
-{
-    static char *token_string = NULL;
-    int token_length = token[t].length;
 
-    token_string = realloc(token_string, token_length+1);
-    memcpy(token_string, &gp_input_line[token[t].start_index], token_length);
-    token_string[token_length] = '\0';
-
-    return token_string;
-}
 
 int
 isstring(int t_num)
@@ -180,7 +162,7 @@ type_udv(int t_num)
 
     while (*udv_ptr) {
 	if (equals(t_num, (*udv_ptr)->udv_name)) {
-	    if ((*udv_ptr)->udv_value.type == NOTDEFINED)
+	    if ((*udv_ptr)->udv_undef)
 		return 0;
 	    else
 		return (*udv_ptr)->udv_value.type;
@@ -205,28 +187,12 @@ isletter(int t_num)
 	    (isalpha(c) || (c == '_') || ALLOWED_8BITVAR(c)));
 }
 
-/* Test whether following bit of command line might be parsable as a number.
- * constant, defined variable, function, ...
- */
-TBOOLEAN
-might_be_numeric(int t_num)
-{
-    if (END_OF_COMMAND)
-	return FALSE;
-    if (isanumber(t_num) || is_function(t_num))
-	return TRUE;
-    if (type_udv(t_num) == INTGR || type_udv(t_num) == CMPLX || type_udv(t_num) == ARRAY)
-	return TRUE;
-    if (equals(t_num, "("))
-	return TRUE;
-    return FALSE;
-}
 
 /*
  * is_definition() returns TRUE if the next tokens are of the form
  *   identifier =
  *              -or-
- *   identifier ( identifier {,identifier} ) =
+ *   identifier ( identifer {,identifier} ) =
  */
 int
 is_definition(int t_num)
@@ -238,11 +204,6 @@ is_definition(int t_num)
     /* function? */
     /* look for dummy variables */
     if (isletter(t_num) && equals(t_num + 1, "(") && isletter(t_num + 2)) {
-
-	/* Block redefinition of reserved function names */
-	if (is_builtin_function(t_num))
-	    return 0;
-
 	t_num += 3;		/* point past first dummy */
 	while (equals(t_num, ",")) {
 	    if (!isletter(++t_num))
@@ -294,9 +255,45 @@ token_len(int t_num)
     return (size_t)(token[t_num].length);
 }
 
+#ifdef NEXT
+/*
+ * quote_str() no longer has any callers in the core code.
+ * However, it is called by the next/openstep terminal.
+ */
+
+/*
+ * quote_str() does the same thing as copy_str, except it ignores the
+ *   quotes at both ends.  This seems redundant, but is done for
+ *   efficency.
+ */
+void
+quote_str(char *str, int t_num, int max)
+{
+    int i = 0;
+    int start = token[t_num].start_index + 1;
+    int count;
+
+    if ((count = token[t_num].length - 2) >= max) {
+	count = max - 1;
+	FPRINTF((stderr, "str buffer overflow in quote_str"));
+    }
+    if (count > 0) {
+	do {
+	    str[i++] = gp_input_line[start++];
+	} while (i != count);
+    }
+    str[i] = NUL;
+    /* convert \t and \nnn (octal) to char if in double quotes */
+    if (gp_input_line[token[t_num].start_index] == '"')
+	parse_esc(str);
+    else
+	parse_sq(str);
+}
+#endif
+
 /*
  * capture() copies into str[] the part of gp_input_line[] which lies between
- * the beginning of token[start] and end of token[end].
+ * the begining of token[start] and end of token[end].
  */
 void
 capture(char *str, int start, int end, int max)
@@ -358,7 +355,9 @@ m_quote_capture(char **str, int start, int end)
 }
 
 /*
- * Wrapper for isstring + m_quote_capture or const_string_express
+ * Wrapper for isstring + m_quote_capture that can be used with
+ * or without GP_STRING_VARS enabled.
+ * EAM Aug 2004
  */
 char *
 try_to_get_string()
@@ -416,6 +415,8 @@ gp_stradd(const char *a, const char *b)
     return new;
 }
 
+/* HBB 20020405: moved these functions here from axis.c, where they no
+ * longer truly belong. */
 /*{{{  mant_exp - split into mantissa and/or exponent */
 /* HBB 20010121: added code that attempts to fix rounding-induced
  * off-by-one errors in 10^%T and similar output formats */
@@ -522,31 +523,28 @@ mant_exp(
 
 /*}}} */
 
-/* Wrapper for gprintf_value() */
-void
-gprintf(char *outstring, size_t count, char *format, double log10_base, double x)
-{
-    struct value v;
-    Gcomplex(&v, x, 0.0);
-    gprintf_value(outstring, count, format, log10_base, &v);
-}
 
-/* Analogous to snprintf() but uses gnuplot's private format specs */
+/*{{{  gprintf */
+/* extended s(n)printf */
 /* HBB 20010121: added code to maintain consistency between mantissa
  * and exponent across sprintf() calls.  The problem: format string
  * '%t*10^%T' will display 9.99 as '10.0*10^0', but 10.01 as
  * '1.0*10^1'.  This causes problems for people using the %T part,
  * only, with logscaled axes, in combination with the occasional
  * round-off error. */
+/* EAM Nov 2012:
+ * Unbelievably, the count parameter has been silently ignored or
+ * improperly applied ever since this routine was introduced back
+ * in version 3.7.  Now fixed to prevent buffer overflow.
+ */
 void
-gprintf_value(
+gprintf(
     char *outstring,
     size_t count,
     char *format,
     double log10_base,
-    struct value *v)
+    double x)
 {
-    double x = real(v);
     char tempdest[MAX_LINE_LEN + 1];
     char temp[MAX_LINE_LEN + 1];
     char *t;
@@ -558,23 +556,17 @@ gprintf_value(
     char *dest  = &tempdest[0];
     char *limit = &tempdest[MAX_LINE_LEN];
     static double log10_of_1024; /* to avoid excess precision comparison in check of connection %b -- %B */
-
+    
     log10_of_1024 = log10(1024);
-
+    
 #define remaining_space (size_t)(limit-dest)
 
     *dest = '\0';
 
-    if (!evaluate_inside_using)
-	set_numeric_locale();
+    set_numeric_locale();
 
-    /* Should never happen but fuzzer managed to hit it */
-    if (!format)
-	format = DEF_FORMAT;
-
-    /* By default we wrap numbers output to latex terminals in $...$ */
-    if (!strcmp(format, DEF_FORMAT)  && !table_mode
-    &&  ((term->flags & TERM_IS_LATEX)))
+    /* Oct 2013 - default format is now expected to be "%h" */
+    if (((term->flags & TERM_IS_LATEX)) && !strcmp(format, DEF_FORMAT))
 	format = DEF_FORMAT_LATEX;
 
     for (;;) {
@@ -610,17 +602,22 @@ gprintf_value(
 
 	/*{{{  convert conversion character */
 	switch (*format) {
-	    /*{{{  x and o can handle 64bit unsigned integers */
+	    /*{{{  x and o */
 	case 'x':
 	case 'X':
 	case 'o':
 	case 'O':
-	    t[0] = 'l';
-	    t[1] = 'l';
-	    t[2] = *format;
-	    t[3] = '\0';
-	    snprintf(dest, remaining_space, temp,
-		     v->type == INTGR ? v->v.int_val : (intgr_t)real(v));
+	    if (fabs(x) >= (double)INT_MAX) {
+		t[0] = 'l';
+		t[1] = 'l';
+		t[2] = *format;
+		t[3] = '\0';
+		snprintf(dest, remaining_space, temp, (long long) x);
+	    } else {
+		t[0] = *format;
+		t[1] = '\0';
+		snprintf(dest, remaining_space, temp, (int) x);
+	    }
 	    break;
 	    /*}}} */
 	    /*{{{  e, f and g */
@@ -636,7 +633,7 @@ gprintf_value(
 	    break;
 	case 'h':
 	case 'H':
-	    /* g/G with enhanced formatting (if applicable) */
+	    /* g/G with enhanced formating (if applicable) */
 	    t[0] = (*format == 'h') ? 'g' : 'G';
 	    t[1] = 0;
 
@@ -864,16 +861,7 @@ gprintf_value(
 		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
 		    /* HBB 20010121: avoid division of -ve ints! */
 		    power = (power + 24) / 3;
-		    if (power == 8) { /* add no extra space */
-		        snprintf(dest, remaining_space, temp, '\0');
-		    } else {
-		        snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
-		    }
-
-		    /* Replace u with micro character */
-		    if (use_micro && power == 6)
-			snprintf(dest, remaining_space, "%s%s", micro, &temp[2]);
-
+		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
 		} else {
 		    /* please extend the range ! */
 		    /* fall back to simple exponential */
@@ -934,12 +922,15 @@ gprintf_value(
 	    }
 	    /*}}} */
 	default:
+	   reset_numeric_locale();
 	   int_error(NO_CARET, "Bad format character");
 	} /* switch */
 	/*}}} */
 
-	if (got_hash && (format != strpbrk(format,"oeEfFgG")))
+	if (got_hash && (format != strpbrk(format,"oeEfFgG"))) {
+	   reset_numeric_locale();
 	   int_error(NO_CARET, "Bad format character");
+	}
 
     /* change decimal '.' to the actual entry in decimalsign */
 	if (decimalsign != NULL) {
@@ -969,46 +960,6 @@ gprintf_value(
 	    }
 	}
 
-    /* Some people prefer a "real" minus sign to the hyphen that standard
-     * formatted input and output both use.  Unlike decimal signs, there is
-     * no internationalization mechanism to specify this preference.
-     * This code replaces all hyphens with the character string specified by
-     * 'set minus_sign "..."'   typically unicode character U+2212 "−".
-     * Use at your own risk.  Should be OK for graphical output, but text output
-     * will not be readable by standard formatted input routines.
-     */
-	if (use_minus_sign		/* set minussign */
-	    && minus_sign		/* current encoding provides one */
-	    && !table_mode		/* not used inside "set table" */
-	    && !(term->flags & TERM_IS_LATEX)	/* but LaTeX doesn't want it */
-	   ) {
-
-	    char *dotpos1 = dest;
-	    char *dotpos2;
-	    size_t newlength = strlen(minus_sign);
-
-	    /* dot is the default hyphen we will be replacing */
-	    int dot = '-';
-
-	    /* replace every dot by the contents of minus_sign */
-	    while ((dotpos2 = strchr(dotpos1,dot)) != NULL) {
-		if (newlength == 1) {	/* The normal case */
-		    *dotpos2 = *minus_sign;
-		    dotpos1++;
-		} else {		/* Some multi-byte minus marker */
-		    size_t taillength = strlen(dotpos2);
-		    dotpos1 = dotpos2 + newlength;
-		    if (dotpos1 + taillength > limit)
-			int_error(NO_CARET,
-				  "format too long due to minus_sign string");
-		    /* move tail end of string out of the way */
-		    memmove(dotpos1, dotpos2 + 1, taillength);
-		    /* insert minus_sign */
-		    memcpy(dotpos2, minus_sign, newlength);
-		}
-	    }
-	}
-
 	/* this was at the end of every single case, before: */
 	dest += strlen(dest);
 	++format;
@@ -1016,16 +967,35 @@ gprintf_value(
 
 done:
 
+#if (0)
+    /* Oct 2013 - Not safe because it fails to recognize LaTeX macros.	*/
+    /* For LaTeX terminals, if the user has not already provided a   	*/
+    /* format in math mode, wrap whatever we got by default in $...$ 	*/
+    if (((term->flags & TERM_IS_LATEX)) && !strchr(tempdest, '$')) {
+	*(outstring++) = '$';
+	strcat(tempdest, "$");
+	count -= 2;
+    }
+#endif
+
     /* Copy as much as fits */
     safe_strncpy(outstring, tempdest, count);
 
-    if (!evaluate_inside_using)
-	reset_numeric_locale();
+    reset_numeric_locale();
 }
 
+/*}}} */
+
 /* some macros for the error and warning functions below
  * may turn this into a utility function later
  */
+#define PRINT_MESSAGE_TO_STDERR				\
+do {							\
+    fprintf(stderr, "\n%s%s\n",				\
+	    current_prompt ? current_prompt : "",	\
+	    gp_input_line);				\
+} while (0)
+
 #define PRINT_SPACES_UNDER_PROMPT		\
 do {						\
     const char *p;				\
@@ -1036,76 +1006,29 @@ do {						\
 	(void) fputc(' ', stderr);		\
 } while (0)
 
-/*
- * Echo back the command or data line that triggered an error,
- * possibly with a caret indicating the token the was not accepted.
- */
-static void
-print_line_with_error(int t_num)
-{
-    int true_line_num = inline_num;
+#define PRINT_SPACES_UPTO_TOKEN						\
+do {									\
+    int i;								\
+									\
+    for (i = 0; i < token[t_num].start_index; i++)			\
+	(void) fputc((gp_input_line[i] == '\t') ? '\t' : ' ', stderr);	\
+} while(0)
 
-    if (t_num == DATAFILE) {
-	/* Print problem line from data file to the terminal */
-	df_showdata();
+#define PRINT_CARET fputs("^\n",stderr);
 
-    } else {
-
-	/* If the current line was built by concatenation of lines inside */
-	/* a {bracketed clause}, try to reconstruct the true line number  */
-	/* FIXME:  This seems to no longer work reliably */
-	char *copy_of_input_line = gp_strdup(gp_input_line);
-	char *minimal_input_line = copy_of_input_line;
-	char *trunc;
-	while ((trunc = strrchr(copy_of_input_line, '\n')) != NULL) {
-	    int current = (t_num == NO_CARET) ? c_token : t_num;
-	    if (trunc < &copy_of_input_line[token[current].start_index]) {
-		minimal_input_line = trunc+1;
-		t_num = NO_CARET;
-		break;
-	    }
-	    *trunc = '\0';
-	    true_line_num--;
-	}
-
-	if (t_num != NO_CARET) {
-	    int i;
-	    int caret = GPMIN(token[t_num].start_index, strlen(minimal_input_line));
-
-	    /* Refresh current command line */
-	    if (!screen_ok)
-		fprintf(stderr, "\n%s%s\n",
-		    current_prompt ? current_prompt : "",
-		    minimal_input_line);
-
-	    PRINT_SPACES_UNDER_PROMPT;
-
-	    /* Print spaces up to token */
-	    for (i = 0; i < caret; i++)
-		fputc((minimal_input_line[i] == '\t') ? '\t' : ' ', stderr);
-
-	    /* Print token */
-	    fputs("^\n",stderr);
-	}
-	free(copy_of_input_line);
-    }
-
-    PRINT_SPACES_UNDER_PROMPT;
-
-    if (!interactive) {
-	LFS *lf = lf_head;
-	/* Back out of any nested if/else clauses */
-	while (lf && !lf->fp && !lf->name && lf->prev)
-	    lf = lf->prev;
-	if (lf && lf->name)
-	    fprintf(stderr, "\"%s\" ", lf->name);
-	fprintf(stderr, "line %d: ", true_line_num);
-    }
+#define PRINT_FILE_AND_LINE						\
+if (!interactive) {							\
+    if (lf_head && lf_head->name)                                       \
+	fprintf(stderr, "\"%s\", line %d: ", lf_head->name, inline_num);\
+    else fprintf(stderr, "line %d: ", inline_num);			\
 }
 
-/*
- * os_error() is just like int_error() except that it calls perror().
+/* TRUE if command just typed; becomes FALSE whenever we
+ * send some other output to screen.  If FALSE, the command line
+ * will be echoed to the screen before the ^ error message.
  */
+TBOOLEAN screen_ok;
+
 #if defined(VA_START) && defined(STDC_HEADERS)
 void
 os_error(int t_num, const char *str,...)
@@ -1117,9 +1040,22 @@ os_error(int t_num, const char *str, va_dcl)
 #ifdef VA_START
     va_list args;
 #endif
-    /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
+#ifdef VMS
+    static status[2] = { 1, 0 };		/* 1 is count of error msgs */
+#endif /* VMS */
 
+    /* reprint line if screen has been written to */
+
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) {	/* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
     PRINT_SPACES_UNDER_PROMPT;
 
 #ifdef VA_START
@@ -1135,11 +1071,21 @@ os_error(int t_num, const char *str, va_dcl)
 #endif
     putc('\n', stderr);
 
-    perror("system error");
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
 
+#ifdef VMS
+    status[1] = vaxc$errno;
+    sys$putmsg(status);
+    (void) putc('\n', stderr);
+#else /* VMS */
+    perror("util.c");
     putc('\n', stderr);
-    fill_gpval_string("GPVAL_ERRMSG", strerror(errno));
-    common_error_exit();
+#endif /* VMS */
+
+    scanning_range_in_progress = FALSE;
+
+    bail_to_command_line();
 }
 
 
@@ -1158,7 +1104,19 @@ int_error(int t_num, const char str[], va_dcl)
     char error_message[128] = {'\0'};
 
     /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
+
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) { /* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
 
 #ifdef VA_START
     VA_START(args, str);
@@ -1175,31 +1133,19 @@ int_error(int t_num, const char str[], va_dcl)
 #endif
 
     fputs("\n\n", stderr);
-    fill_gpval_string("GPVAL_ERRMSG", error_message);
-    common_error_exit();
-}
 
-
-void
-common_error_exit()
-{
     /* We are bailing out of nested context without ever reaching */
     /* the normal cleanup code. Reset any flags before bailing.   */
     df_reset_after_error();
     eval_reset_after_error();
     clause_reset_after_error();
     parse_reset_after_error();
-    pm3d_reset_after_error();
-    set_iterator = cleanup_iteration(set_iterator);
-    plot_iterator = cleanup_iteration(plot_iterator);
     scanning_range_in_progress = FALSE;
     inside_zoom = FALSE;
-#ifdef HAVE_LOCALE_H
-    setlocale(LC_NUMERIC, "C");
-#endif
 
     /* Load error state variables */
     update_gpval_variables(2);
+    fill_gpval_string("GPVAL_ERRMSG", error_message);
 
     bail_to_command_line();
 }
@@ -1218,7 +1164,19 @@ int_warn(int t_num, const char str[], va_dcl)
 #endif
 
     /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
+
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) { /* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
 
     fputs("warning: ", stderr);
 #ifdef VA_START
@@ -1233,28 +1191,57 @@ int_warn(int t_num, const char str[], va_dcl)
     fprintf(stderr, str, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif /* VA_START */
     putc('\n', stderr);
+}
 
-#if defined(_WIN32) || defined(__KLIBC__)
-    /* Check asynchronously for Ctrl-C */
-    if (ctrlc_flag) {
-	ctrlc_flag = FALSE;
-	term_reset();
-	putc('\n', stderr);
-	fprintf(stderr, "Ctrl-C detected!\n");
-	bail_to_command_line();	/* return to prompt */
-    }
+/*{{{  graph_error() */
+/* handle errors during graph-plot in a consistent way */
+/* HBB 20000430: move here, from graphics.c */
+#if defined(VA_START) && defined(STDC_HEADERS)
+void
+graph_error(const char *fmt, ...)
+#else
+void
+graph_error(const char *fmt, va_dcl)
 #endif
+{
+#ifdef VA_START
+    va_list args;
+#endif
+
+    multiplot = FALSE;
+    term_end_plot();
+
+#ifdef VA_START
+    VA_START(args, fmt);
+    /* HBB 20001120: instead, copy the core code from int_error() to
+     * here: */
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
+
+# if defined(HAVE_VFPRINTF) || _LIBC
+    vfprintf(stderr, fmt, args);
+# else
+    _doprnt(fmt, args, stderr);
+# endif
+    va_end(args);
+    fputs("\n\n", stderr);
+
+    bail_to_command_line();
+    va_end(args);
+#else
+    int_error(NO_CARET, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
+#endif
+
 }
 
 /*}}} */
 
 
-/*
- * Reduce all multiple white-space chars to single spaces (if remain == 1)
- * or remove altogether (if remain == 0).  Modifies the original string.
- */
+/* Squash spaces in the given string (DFK) */
+/* That is, reduce all multiple white-space chars to single spaces */
+/* Done in place. Currently used only by help_command() */
 void
-squash_spaces(char *s, int remain)
+squash_spaces(char *s)
 {
     char *r = s;	/* reading point */
     char *w = s;	/* writing point */
@@ -1263,7 +1250,7 @@ squash_spaces(char *s, int remain)
     for (w = r = s; *r != NUL; r++) {
 	if (isspace((unsigned char) *r)) {
 	    /* white space; only copy if we haven't just copied a space */
-	    if (!space && remain > 0) {
+	    if (!space) {
 		space = TRUE;
 		*w++ = ' ';
 	    }			/* else ignore multiple spaces */
@@ -1335,14 +1322,9 @@ parse_esc(char *instr)
 		    *t++ = '\\';
 		    *t++ = *s++;
 		}
-	    } else if (s[0] == 'U' && s[1] == '+') {
-		/* Unicode escape:  \U+hhhh
-		 * Keep backslash; translation will be handled elsewhere.
-		 */
-		*t++ = '\\';
 	    }
 	} else if (df_separators && *s == '\"' && *(s+1) == '\"') {
-	    /* For parsing CSV strings with quoted quotes */
+	/* EAM Mar 2003 - For parsing CSV strings with quoted quotes */
 	    *t++ = *s++; s++;
 	} else {
 	    *t++ = *s++;
@@ -1352,17 +1334,30 @@ parse_esc(char *instr)
 }
 
 
-/* This function does nothing if dirent.h and windows.h not available. */
+/* FIXME HH 20020915: This function does nothing if dirent.h and windows.h
+ * not available. */
 TBOOLEAN
-existdir(const char *name)
+existdir (const char *name)
 {
-#if defined(HAVE_DIRENT)
+#ifdef HAVE_DIRENT_H
     DIR *dp;
-    if ((dp = opendir(name)) == NULL)
+    if (! (dp = opendir(name) ) )
 	return FALSE;
 
     closedir(dp);
     return TRUE;
+#elif defined(_Windows)
+    HANDLE FileHandle;
+    WIN32_FIND_DATA finddata;
+
+    FileHandle = FindFirstFile(name, &finddata);
+    if (FileHandle != INVALID_HANDLE_VALUE) {
+	if (finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+	    return TRUE;
+    }
+    return FALSE;
+#elif defined(VMS)
+    return FALSE;
 #else
     int_warn(NO_CARET,
 	     "Test on directory existence not supported\n\t('%s!')",
@@ -1375,7 +1370,7 @@ existdir(const char *name)
 TBOOLEAN
 existfile(const char *name)
 {
-#ifdef _MSC_VER
+#ifdef __MSC__
     return (_access(name, 0) == 0);
 #else
     return (access(name, F_OK) == 0);
@@ -1395,13 +1390,107 @@ getusername()
     return gp_strdup(username);
 }
 
+TBOOLEAN contains8bit(const char *s)
+{
+    while (*s) {
+	if ((*s++ & 0x80))
+	    return TRUE;
+    }
+    return FALSE;
+}
+
+#define INVALID_UTF8 0xfffful
+
+/* Read from second byte to end of UTF-8 sequence.
+   used by utf8toulong() */
+static TBOOLEAN
+utf8_getmore (unsigned long * wch, const char **str, int nbytes)
+{
+  int i;
+  unsigned char c;
+  unsigned long minvalue[] = {0x80, 0x800, 0x10000, 0x200000, 0x4000000};
+
+  for (i = 0; i < nbytes; i++) {
+    c = (unsigned char) **str;
+
+    if ((c & 0xc0) != 0x80) {
+      *wch = INVALID_UTF8;
+      return FALSE;
+    }
+    *wch = (*wch << 6) | (c & 0x3f);
+    (*str)++;
+  }
+
+  /* check for overlong UTF-8 sequences */
+  if (*wch < minvalue[nbytes-1]) {
+    *wch = INVALID_UTF8;
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/* Convert UTF-8 multibyte sequence from string to unsigned long character.
+   Returns TRUE on success.
+*/
+TBOOLEAN
+utf8toulong (unsigned long * wch, const char ** str)
+{
+  unsigned char c;
+
+  c =  (unsigned char) *(*str)++;
+  if ((c & 0x80) == 0) {
+    *wch = (unsigned long) c;
+    return TRUE;
+  }
+
+  if ((c & 0xe0) == 0xc0) {
+    *wch = c & 0x1f;
+    return utf8_getmore(wch, str, 1);
+  }
+
+  if ((c & 0xf0) == 0xe0) {
+    *wch = c & 0x0f;
+    return utf8_getmore(wch, str, 2);
+  }
+
+  if ((c & 0xf8) == 0xf0) {
+    *wch = c & 0x07;
+    return utf8_getmore(wch, str, 3);
+  }
+
+  if ((c & 0xfc) == 0xf8) {
+    *wch = c & 0x03;
+    return utf8_getmore(wch, str, 4);
+  }
+
+  if ((c & 0xfe) == 0xfc) {
+    *wch = c & 0x01;
+    return utf8_getmore(wch, str, 5);
+  }
+
+  *wch = INVALID_UTF8;
+  return FALSE;
+}
+
+/*
+ * Returns number of (possibly multi-byte) characters in a UTF-8 string
+ */
+size_t
+strlen_utf8(const char *s)
+{
+    int i = 0, j = 0;
+    while (s[i]) {
+	if ((s[i] & 0xc0) != 0x80) j++;
+	i++;
+    }
+    return j;
+}
+
 size_t
 gp_strlen(const char *s)
 {
     if (encoding == S_ENC_UTF8)
 	return strlen_utf8(s);
-    else if (encoding == S_ENC_SJIS)
-	return strlen_sjis(s);
     else
 	return strlen(s);
 }
@@ -1460,7 +1549,7 @@ streq(const char *a, const char *b)
 
 
 /* append string src to dest
-   re-allocates memory if necessary, (re-)determines the length of the
+   re-allocates memory if necessary, (re-)determines the length of the 
    destination string only if len==0
  */
 size_t
@@ -1469,8 +1558,7 @@ strappend(char **dest, size_t *size, size_t len, const char *src)
     size_t destlen = (len != 0) ? len : strlen(*dest);
     size_t srclen = strlen(src);
     if (destlen + srclen + 1 > *size) {
-	while (destlen + srclen + 1 > *size)
-	    *size *= 2;
+	*size *= 2;
 	*dest = (char *) gp_realloc(*dest, *size, "strappend");
     }
     memcpy(*dest + destlen, src, srclen + 1);
@@ -1496,7 +1584,7 @@ value_to_str(struct value *val, TBOOLEAN need_quotes)
 
     switch (val->type) {
     case INTGR:
-	sprintf(s[j], PLD, val->v.int_val);
+	sprintf(s[j], "%d", val->v.int_val);
 	break;
     case CMPLX:
 	if (isnan(val->v.cmplx_val.real))
@@ -1533,23 +1621,13 @@ value_to_str(struct value *val, TBOOLEAN need_quotes)
 	break;
     case DATABLOCK:
 	{
-	sprintf(s[j], "<%d line data block>", datablock_size(val));
-	break;
+	char **dataline = val->v.data_array;
+	int nlines = 0;
+	if (dataline != NULL) {
+	    while (*dataline++ != NULL)
+		nlines++;
 	}
-    case ARRAY:
-	{
-	sprintf(s[j], "<%d element array>", (int)(val->v.value_array->v.int_val));
-	break;
-	}
-    case VOXELGRID:
-	{
-	int N = val->v.vgrid->size;
-	sprintf(s[j], "%d x %d x %d voxel grid", N, N, N);
-	break;
-	}
-    case NOTDEFINED:
-	{
-	sprintf(s[j], "<undefined>");
+	sprintf(s[j], "<%d line data block>", nlines);
 	break;
 	}
     default:
@@ -1564,7 +1642,7 @@ value_to_str(struct value *val, TBOOLEAN need_quotes)
  * format. Rotates through 4 buffers 's[j]', and returns pointers to
  * them, to avoid execution ordering problems if this function is
  * called more than once between sequence points. */
-static char *
+char *
 num_to_str(double r)
 {
     static int i = 0;
@@ -1583,23 +1661,3 @@ num_to_str(double r)
     return s[j];
 }
 
-/* Auto-generated titles need modification to be compatible with LaTeX.
- * For example filenames may contain underscore characters or dollar signs.
- * Function plots auto-generate a copy of the function or expression,
- * which probably looks better in math mode.
- * We could perhaps go further and texify syntax such as a**b -> a^b
- */
-char *
-texify_title(char *str, int plot_type)
-{
-    static char *latex_title = NULL;
-
-	if (plot_type == DATA || plot_type == DATA3D) {
-	    latex_title = escape_reserved_chars(str,"#$%^&_{}\\");
-	} else {
-	    latex_title = gp_realloc(latex_title, strlen(str) + 4, NULL);
-	    sprintf(latex_title, "$%s$", str);
-	}
-
-    return latex_title;
-}
