@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util3d.c,v 1.55 2017/05/05 06:10:31 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util3d.c,v 1.48.2.2 2015/10/31 04:39:20 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util3d.c */
@@ -52,6 +52,7 @@ static char *RCSid() { return RCSid("$Id: util3d.c,v 1.55 2017/05/05 06:10:31 sf
 #include "pm3d.h"
 #include "term_api.h"
 
+/* EAM DEBUG - moved these here from axis.h */
 #define AXIS_ACTUAL_MIN(axis) GPMIN(axis_array[axis].max, axis_array[axis].min)
 #define AXIS_ACTUAL_MAX(axis) GPMAX(axis_array[axis].max, axis_array[axis].min)
 
@@ -59,17 +60,6 @@ static char *RCSid() { return RCSid("$Id: util3d.c,v 1.55 2017/05/05 06:10:31 sf
 static void mat_unit __PROTO((transform_matrix mat));
 static GP_INLINE void draw3d_point_unconditional __PROTO((p_vertex, struct lp_style_type *));
 
-#ifdef NONLINEAR_AXES
-static double map_x3d __PROTO((double));
-static double map_y3d __PROTO((double));
-static double map_z3d __PROTO((double));
-#else
-/* Function macros to map from user 3D space into normalized -1..1 */
-#define map_x3d(x) ((x-X_AXIS.min)*xscale3d + xcenter3d - 1.0)
-#define map_y3d(y) ((y-Y_AXIS.min)*yscale3d + ycenter3d - 1.0)
-#define map_z3d(z) ((z-floor_z)*zscale3d + zcenter3d - 1.0)
-#endif
-
 static void
 mat_unit(transform_matrix mat)
 {
@@ -237,7 +227,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_Z_AXIS), iz, oz))
 		*ez = AXIS_ACTUAL_MIN(FIRST_Z_AXIS);
 	    else {
-		int_error(NO_CARET,"error in edge3d_intersect");
+		graph_error("error in edge3d_intersect");
 	    }
 
 	    return;
@@ -254,7 +244,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_Y_AXIS), iy, oy))
 		*ey = AXIS_ACTUAL_MIN(FIRST_Y_AXIS);
 	    else {
-		int_error(NO_CARET,"error in edge3d_intersect");
+		graph_error("error in edge3d_intersect");
 	    }
 
 	    return;
@@ -304,7 +294,7 @@ edge3d_intersect(
 	    else if (inrange(AXIS_ACTUAL_MIN(FIRST_X_AXIS), ix, ox))
 		*ex = AXIS_ACTUAL_MIN(FIRST_X_AXIS);
 	    else {
-		int_error(NO_CARET,"error in edge3d_intersect");
+		graph_error("error in edge3d_intersect");
 	    }
 
 	    return;
@@ -1033,43 +1023,13 @@ polyline3d_next(p_vertex v2, struct lp_style_type *lp)
     polyline3d_previous_vertex = *v2;
 }
 
-#ifdef NONLINEAR_AXES
-static double
-map_x3d(double x)
+/*
+ * Dummy up an x-axis scale so that we can share the 2D arrowhead routine.
+ */
+void
+apply_3dhead_properties(struct arrow_style_type *arrow_properties)
 {
-    AXIS *xaxis = &axis_array[FIRST_X_AXIS];
-
-    if (xaxis->linked_to_primary) {
-	xaxis = xaxis->linked_to_primary;
-	x = eval_link_function(xaxis, x);
-    }
-
-    return ((x - xaxis->min)*xscale3d + xcenter3d - 1.0);
+    X_AXIS.term_scale = (plot_bounds.xright - plot_bounds.xleft)
+			/ (X_AXIS.max - X_AXIS.min);
+    apply_head_properties(arrow_properties);
 }
-
-static double
-map_y3d(double y)
-{
-    AXIS *yaxis = &axis_array[FIRST_Y_AXIS];
-
-    if (yaxis->linked_to_primary) {
-	yaxis = yaxis->linked_to_primary;
-	y = eval_link_function(yaxis, y);
-    }
-
-    return ((y - yaxis->min)*yscale3d + ycenter3d - 1.0);
-}
-
-static double
-map_z3d(double z)
-{
-    AXIS *zaxis = &axis_array[FIRST_Z_AXIS];
-
-    if (zaxis->linked_to_primary) {
-	zaxis = zaxis->linked_to_primary;
-	z = eval_link_function(zaxis, z);
-    }
-
-    return ((z - floor_z1)*zscale3d + zcenter3d - 1.0);
-}
-#endif
