@@ -1,3 +1,7 @@
+/*
+ * $Id: gp_cairo.h,v 1.22 2014/04/18 04:12:46 sfeam Exp $
+ */
+
 /* GNUPLOT - gp_cairo.h */
 
 /*[
@@ -81,7 +85,7 @@ extern "C" {
 # include <cairo.h>
 
 /* oversampling scale */
-#define GP_CAIRO_SCALE 200
+#define GP_CAIRO_SCALE 20
 
 typedef struct rgba_color {
 	double r;
@@ -118,15 +122,10 @@ typedef struct plot_struct {
 	/* either GP_CAIRO_SCALE or 1, depending on rendering */
 	int oversampling_scale;
 
-	/* The caiolatex performs upsampling of the canvas to achieve
-	 * higher pixel density.  We have to account for that when
-	 * applying OPERATOR_SATURATE. */
-	double upsampling_rate;
-
 	/* handle vertical/horizontal lines properly */
 	double current_x, current_y;
 	double orig_current_x, orig_current_y;
-
+	
 	/* style data used while processing gnuplot commands */
 	JUSTIFY justify_mode;
 	int linetype;
@@ -186,7 +185,7 @@ GP_CAIRO_DASH,
 GP_CAIRO_DOTS
 };
 
-/* correspondence between gnuplot's linetypes and colors */
+/* correspondance between gnuplot's linetypes and colors */
 rgb_color gp_cairo_linetype2color( int linetype );
 void gp_cairo_set_background(rgb_color background);
 
@@ -223,7 +222,7 @@ void gp_cairo_set_linestyle(plot_struct *plot, int linestyle);
 void gp_cairo_set_linetype(plot_struct *plot, int linetype);
 void gp_cairo_set_pointsize(plot_struct *plot, double pointsize);
 void gp_cairo_set_justify(plot_struct *plot, JUSTIFY mode);
-void gp_cairo_set_font(plot_struct *plot, const char *name, float fontsize);
+void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize);
 void gp_cairo_set_linewidth(plot_struct *plot, double linewidth);
 void gp_cairo_set_textangle(plot_struct *plot, double angle);
 
@@ -242,17 +241,13 @@ const char* gp_cairo_get_encoding(plot_struct *plot);
 /* determine default font to use */
 const char * gp_cairo_default_font(void);
 
-/* work-around for "bold font gets stuck" bug */
-void gp_cairo_clear_bold_font(plot_struct *plot);
-
+#ifdef EAM_BOXED_TEXT
 /* Text boxes */
 void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option);
+#endif
 
 void gp_cairo_set_dashtype(plot_struct *plot, int type, t_dashtype *custom_dash_pattern);
 
-/* explicitly set resolution */
-void gp_cairo_set_resolution(int dpi);
-
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
