@@ -294,6 +294,53 @@ static const char *prologue_cp1251_ps[] = {
 NULL
 };
 
+static const char *prologue_cp1252_ps[] = {
+"%\n",
+"% Encoding for Windows-1252 (Western European)\n",
+"%\n",
+"/reencodeCP1252 {\n",
+"dup dup findfont dup length dict begin\n",
+"{ 1 index /FID ne { def }{ pop pop } ifelse } forall\n",
+"currentdict /CharStrings known {\n",
+"\tCharStrings /Idieresis known {\n",
+"\t\t/Encoding CP1252Encoding def } if\n",
+"} if\n",
+"currentdict end definefont\n",
+"} def\n",
+"/CP1252Encoding [\n",
+"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n",
+"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n",
+"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n",
+"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n",
+"/space/exclam/quotedbl/numbersign/dollar/percent/ampersand/quoteright\n",
+"/parenleft/parenright/asterisk/plus/comma/minus/period/slash\n",
+"/zero/one/two/three/four/five/six/seven/eight/nine/colon/semicolon\n",
+"/less/equal/greater/question/at/A/B/C/D/E/F/G/H/I/J/K/L/M/N\n",
+"/O/P/Q/R/S/T/U/V/W/X/Y/Z/bracketleft/backslash/bracketright\n",
+"/asciicircum/underscore/quoteleft/a/b/c/d/e/f/g/h/i/j/k/l/m\n",
+"/n/o/p/q/r/s/t/u/v/w/x/y/z/braceleft/bar/braceright/asciitilde\n",
+"/.notdef/Euro/.notdef/quotesinglbase/florin/quotedblbase/ellipsis/dagger\n",
+"/daggerdbl/circumflex/perthousand/Scaron/guilsinglleft/OE/.notdef/Zcaron\n",
+"/.notdef/.notdef/quoteleft/quoteright/quotedblleft/quotedblright\n",
+"/bullet/endash/emdash/tilde/trademark/scaron/guilsinglright/oe/.notdef\n",
+"/zcaron/Ydieresis/space/exclamdown/cent/sterling/currency/yen/brokenbar\n",
+"/section/dieresis/copyright/ordfeminine/guillemotleft/logicalnot\n",
+"/hyphen/registered/macron/degree/plusminus/twosuperior/threesuperior\n",
+"/acute/mu/paragraph/periodcentered/cedilla/onesuperior/ordmasculine\n",
+"/guillemotright/onequarter/onehalf/threequarters/questiondown\n",
+"/Agrave/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla\n",
+"/Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute/Icircumflex\n",
+"/Idieresis/Eth/Ntilde/Ograve/Oacute/Ocircumflex/Otilde/Odieresis\n",
+"/multiply/Oslash/Ugrave/Uacute/Ucircumflex/Udieresis/Yacute\n",
+"/Thorn/germandbls/agrave/aacute/acircumflex/atilde/adieresis\n",
+"/aring/ae/ccedilla/egrave/eacute/ecircumflex/edieresis/igrave\n",
+"/iacute/icircumflex/idieresis/eth/ntilde/ograve/oacute/ocircumflex\n",
+"/otilde/odieresis/divide/oslash/ugrave/uacute/ucircumflex/udieresis\n",
+"/yacute/thorn/ydieresis\n",
+"] def\n",
+NULL
+};
+
 static const char *prologue_cp437_ps[] = {
 "%\n",
 "% encoding for code page 437 (original MSDOS)\n",
@@ -538,7 +585,7 @@ NULL
 
 static const char *prologue_prologue_ps[] = {
 "%\n",
-"% Gnuplot Prolog Version 4.6 (November 2011)\n",
+"% Gnuplot Prolog Version 5.0.2 (Oct 2015)\n",
 "%\n",
 "%/SuppressPDFMark true def\n",
 "%\n",
@@ -555,11 +602,11 @@ static const char *prologue_prologue_ps[] = {
 "/vpt2 vpt 2 mul def\n",
 "/hpt2 hpt 2 mul def\n",
 "/Lshow {currentpoint stroke M 0 vshift R \n",
-"\tBlacktext {gsave 0 setgray show grestore} {show} ifelse} def\n",
+"\tBlacktext {gsave 0 setgray textshow grestore} {textshow} ifelse} def\n",
 "/Rshow {currentpoint stroke M dup stringwidth pop neg vshift R\n",
-"\tBlacktext {gsave 0 setgray show grestore} {show} ifelse} def\n",
+"\tBlacktext {gsave 0 setgray textshow grestore} {textshow} ifelse} def\n",
 "/Cshow {currentpoint stroke M dup stringwidth pop -2 div vshift R \n",
-"\tBlacktext {gsave 0 setgray show grestore} {show} ifelse} def\n",
+"\tBlacktext {gsave 0 setgray textshow grestore} {textshow} ifelse} def\n",
 "/UP {dup vpt_ mul /vpt exch def hpt_ mul /hpt exch def\n",
 "  /hpt2 hpt 2 mul def /vpt2 vpt 2 mul def} def\n",
 "/DL {Color {setrgbcolor Solid {pop []} if 0 setdash}\n",
@@ -573,7 +620,7 @@ static const char *prologue_prologue_ps[] = {
 "/PL {stroke userlinewidth setlinewidth\n",
 "\tRounded {1 setlinejoin 1 setlinecap} if} def\n",
 "3.8 setmiterlimit\n",
-"% Default Line colors\n",
+"% Classic Line colors (version 5.0)\n",
 "/LCw {1 1 1} def\n",
 "/LCb {0 0 0} def\n",
 "/LCa {0 0 0} def\n",
@@ -586,19 +633,21 @@ static const char *prologue_prologue_ps[] = {
 "/LC6 {0 0 0} def\n",
 "/LC7 {1 0.3 0} def\n",
 "/LC8 {0.5 0.5 0.5} def\n",
-"% Default Line Types\n",
+"% Default dash patterns (version 5.0)\n",
+"/LTB {BL [] LCb DL} def\n",
 "/LTw {PL [] 1 setgray} def\n",
-"/LTb {BL [] LCb DL} def\n",
+"/LTb {PL [] LCb DL} def\n",
 "/LTa {AL [1 udl mul 2 udl mul] 0 setdash LCa setrgbcolor} def\n",
 "/LT0 {PL [] LC0 DL} def\n",
-"/LT1 {PL [4 dl1 2 dl2] LC1 DL} def\n",
-"/LT2 {PL [2 dl1 3 dl2] LC2 DL} def\n",
-"/LT3 {PL [1 dl1 1.5 dl2] LC3 DL} def\n",
-"/LT4 {PL [6 dl1 2 dl2 1 dl1 2 dl2] LC4 DL} def\n",
-"/LT5 {PL [3 dl1 3 dl2 1 dl1 3 dl2] LC5 DL} def\n",
-"/LT6 {PL [2 dl1 2 dl2 2 dl1 6 dl2] LC6 DL} def\n",
-"/LT7 {PL [1 dl1 2 dl2 6 dl1 2 dl2 1 dl1 2 dl2] LC7 DL} def\n",
-"/LT8 {PL [2 dl1 2 dl2 2 dl1 2 dl2 2 dl1 2 dl2 2 dl1 4 dl2] LC8 DL} def\n",
+"/LT1 {PL [2 dl1 3 dl2] LC1 DL} def\n",
+"/LT2 {PL [1 dl1 1.5 dl2] LC2 DL} def\n",
+"/LT3 {PL [6 dl1 2 dl2 1 dl1 2 dl2] LC3 DL} def\n",
+"/LT4 {PL [1 dl1 2 dl2 6 dl1 2 dl2 1 dl1 2 dl2] LC4 DL} def\n",
+"/LT5 {PL [4 dl1 2 dl2] LC5 DL} def\n",
+"/LT6 {PL [1.5 dl1 1.5 dl2 1.5 dl1 1.5 dl2 1.5 dl1 6 dl2] LC6 DL} def\n",
+"/LT7 {PL [3 dl1 3 dl2 1 dl1 3 dl2] LC7 DL} def\n",
+"/LT8 {PL [2 dl1 2 dl2 2 dl1 6 dl2] LC8 DL} def\n",
+"/SL {[] 0 setdash} def\n",
 "/Pnt {stroke [] 0 setdash gsave 1 setlinecap M 0 0 V stroke grestore} def\n",
 "/Dia {stroke [] 0 setdash 2 copy vpt add M\n",
 "  hpt neg vpt neg V hpt vpt neg V\n",
@@ -805,7 +854,8 @@ static const char *prologue_prologue_ps[] = {
 "/PatternFill {gsave /PFa [ 9 2 roll ] def\n",
 "  PFa 0 get PFa 2 get 2 div add PFa 1 get PFa 3 get 2 div add translate\n",
 "  PFa 2 get -2 div PFa 3 get -2 div PFa 2 get PFa 3 get Rec\n",
-"  gsave 1 setgray fill grestore clip\n",
+"  TransparentPatterns {} {gsave 1 setgray fill grestore} ifelse\n",
+"  clip\n",
 "  currentlinewidth 0.5 mul setlinewidth\n",
 "  /PFs PFa 2 get dup mul PFa 3 get dup mul add sqrt def\n",
 "  0 0 M PFa 5 get rotate PFs -2 div dup translate\n",
@@ -819,9 +869,14 @@ static const char *prologue_prologue_ps[] = {
 "%\n",
 "/languagelevel where\n",
 " {pop languagelevel} {1} ifelse\n",
-" 2 lt\n",
-"\t{/InterpretLevel1 true def}\n",
-"\t{/InterpretLevel1 Level1 def}\n",
+"dup 2 lt\n",
+"\t{/InterpretLevel1 true def\n",
+"\t /InterpretLevel3 false def}\n",
+"\t{/InterpretLevel1 Level1 def\n",
+"\t 2 gt\n",
+"\t    {/InterpretLevel3 Level3 def}\n",
+"\t    {/InterpretLevel3 false def}\n",
+"\t ifelse }\n",
 " ifelse\n",
 "%\n",
 "% PostScript level 2 pattern fill definitions\n",
@@ -910,6 +965,7 @@ static const char *prologue_prologue_ps[] = {
 "/Symbol-Oblique /Symbol findfont [1 0 .167 1 0 0] makefont\n",
 "dup length dict begin {1 index /FID eq {pop pop} {def} ifelse} forall\n",
 "currentdict end definefont pop\n",
+"%\n",
 NULL
 };
 
